/*
    AQTRONIX C++ Library
    Copyright 2005-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// Firewall.cpp: implementation of the CFirewall class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Firewall.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFirewall::CFirewall(CFirewallSettings& _Settings):Settings(_Settings)
{

}

CFirewall::~CFirewall()
{

}

bool CFirewall::DenyChar(const char *p, char c, const char *LogMsg)
{
	if(strchr(p,c)!=NULL) {
		logger.Append(LogMsg);
		return true;
	}else{
		return false;
	}
}

bool CFirewall::DenyChar(const char *p, char c, const char *LogMsg, CStringList& Log)
{
	if(strchr(p,c)!=NULL) {
		Log.AddTail(LogMsg);
		return true;
	}else{
		return false;
	}
}

bool CFirewall::DenyString(const char *p, const char *pStr, const char *LogMsg)
{
	if(strstr(p,pStr)!=NULL) {
		logger.Append(LogMsg);
		return true;
	}else{
		return false;
	}
}

bool CFirewall::DenyString(const char *p, const char *pStr, const char *LogMsg, CStringList& Log)
{
	if(strstr(p,pStr)!=NULL) {
		Log.AddTail(LogMsg);
		return true;
	}else{
		return false;
	}
}

bool CFirewall::DenySequences(CString &ScanString, CStringList &Sequences, CString LogPre, CString LogPost)
{
	CString seq;
	bool ret(false);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(ScanString.Find(seq)!=-1){
			ret = true;
			logger.Append(LogPre + seq + LogPost);
		}
	}
	return ret;
}

bool CFirewall::DenySequences(CString &ScanString, CStringList &Sequences, CString LogPre, CString LogPost, CStringList& Log)
{
	CString seq;
	bool ret(false);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(ScanString.Find(seq)!=-1){
			ret = true;
			Log.AddTail(LogPre + seq + LogPost);
		}
	}
	return ret;
}

bool CFirewall::DenySequences(CString &ScanString1, CString& ScanString2, CStringList &Sequences, CString LogPre, CString LogPost)
{
	CString seq;
	bool ret(false);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(ScanString1.Find(seq)!=-1 || ScanString2.Find(seq)!=-1){
			ret = true;
			logger.Append(LogPre + seq + LogPost);
		}
	}
	return ret;
}

bool CFirewall::DenySequences(CString &ScanString1, CString& ScanString2, CStringList &Sequences, CString LogPre, CString LogPost, CStringList& Log)
{
	CString seq;
	bool ret(false);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(ScanString1.Find(seq)!=-1 || ScanString2.Find(seq)!=-1){
			ret = true;
			Log.AddTail(LogPre + seq + LogPost);
		}
	}
	return ret;
}

bool CFirewall::DenySequences(CStringList &ScanList1, CStringList& ScanList2, CStringList &Sequences, CString LogPre, CString LogPost)
{
	CString seq;
	bool ret(false);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(IsInList(seq,ScanList1) || IsInList(seq,ScanList2)){
			ret = true;
			logger.Append(LogPre + seq + LogPost);
		}
	}
	return ret;
}

bool CFirewall::DenySequences(CStringList &ScanList1, CStringList& ScanList2, CStringList &Sequences, CString LogPre, CString LogPost, CStringList& Log)
{
	CString seq;
	bool ret(false);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(IsInList(seq,ScanList1) || IsInList(seq,ScanList2)){
			ret = true;
			Log.AddTail(LogPre + seq + LogPost);
		}
	}
	return ret;
}

bool CFirewall::DenySequences(CStringList &ScanList, CStringList &Sequences, CString LogPre, CString LogPost)
{
	if(ScanList.GetCount()<Sequences.GetCount()){
		return DenySequences(Sequences,ScanList,LogPre,LogPost);
	}else{
		CString seq;
		bool ret(false);
		POSITION pos = Sequences.GetHeadPosition();
		while(pos!=NULL){
			seq = Sequences.GetNext(pos);
			if(IsInList(seq,ScanList)){
				ret = true;
				logger.Append(LogPre + seq + LogPost);
			}
		}
		return ret;
	}
}

bool CFirewall::DenySequences(CStringList &ScanList, CStringList &Sequences, CString LogPre, CString LogPost, CStringList& Log)
{
	if(ScanList.GetCount()<Sequences.GetCount()){
		return DenySequences(Sequences,ScanList,LogPre,LogPost,Log);
	}else{
		CString seq;
		bool ret(false);
		POSITION pos = Sequences.GetHeadPosition();
		while(pos!=NULL){
			seq = Sequences.GetNext(pos);
			if(IsInList(seq,ScanList)){
				ret = true;
				Log.AddTail(LogPre + seq + LogPost);
			}
		}
		return ret;
	}
}

int CFirewall::DenyCharactersCount(CString &ScanString, CString &Characters, CString LogPre, CString LogPost)
{
	int i; char c;
	int ret(0);
	for(i=0;i<Characters.GetLength();i++){
		c = Characters.GetAt(i);
		if(ScanString.Find(c)>-1){
			ret++;
			logger.Append(LogPre + CString(c) + LogPost);
		}
	}
	return ret;
}

int CFirewall::DenyCharactersCount(CString &ScanString, CString &Characters, CString LogPre, CString LogPost, CStringList& Log)
{
	int i; char c;
	int ret(0);
	for(i=0;i<Characters.GetLength();i++){
		c = Characters.GetAt(i);
		if(ScanString.Find(c)>-1){
			ret++;
			Log.AddTail(LogPre + CString(c) + LogPost);
		}
	}
	return ret;
}

int CFirewall::DenyCharactersCount(CString &ScanString1, CString& ScanString2, CString &Characters, CString LogPre, CString LogPost)
{
	int i; char c;
	int ret(0);
	for(i=0;i<Characters.GetLength();i++){
		c = Characters.GetAt(i);
		if(ScanString1.Find(c)!=-1 || ScanString2.Find(c)!=-1){
			ret++;
			logger.Append(LogPre + CString(c) + LogPost);
		}
	}
	return ret;
}

int CFirewall::DenyCharactersCount(CString &ScanString1, CString& ScanString2, CString &Characters, CString LogPre, CString LogPost, CStringList& Log)
{
	int i; char c;
	int ret(0);
	for(i=0;i<Characters.GetLength();i++){
		c = Characters.GetAt(i);
		if(ScanString1.Find(c)!=-1 || ScanString2.Find(c)!=-1){
			ret++;
			Log.AddTail(LogPre + CString(c) + LogPost);
		}
	}
	return ret;
}

int CFirewall::DenySequencesCount(CString &ScanString, CStringList &Sequences, CString LogPre, CString LogPost)
{
	CString seq;
	int ret(0);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(ScanString.Find(seq)!=-1){
			ret++;
			logger.Append(LogPre + seq + LogPost);
		}
	}
	return ret;
}

int CFirewall::DenySequencesCount(CString &ScanString, CStringList &Sequences, CString LogPre, CString LogPost, CStringList& Log)
{
	CString seq;
	int ret(0);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(ScanString.Find(seq)!=-1){
			ret++;
			Log.AddTail(LogPre + seq + LogPost);
		}
	}
	return ret;
}

int CFirewall::DenySequencesCount(CString &ScanString1, CString& ScanString2, CStringList &Sequences, CString LogPre, CString LogPost)
{
	CString seq;
	int ret(0);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(ScanString1.Find(seq)!=-1 || ScanString2.Find(seq)!=-1){
			ret++;
			logger.Append(LogPre + seq + LogPost);
		}
	}
	return ret;
}

int CFirewall::DenySequencesCount(CString &ScanString1, CString& ScanString2, CStringList &Sequences, CString LogPre, CString LogPost, CStringList& Log)
{
	CString seq;
	int ret(0);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(ScanString1.Find(seq)!=-1 || ScanString2.Find(seq)!=-1){
			ret++;
			Log.AddTail(LogPre + seq + LogPost);
		}
	}
	return ret;
}

int CFirewall::DenySequencesCount(CStringList &ScanList, CStringList &Sequences, CString LogPre, CString LogPost)
{
	CString seq;
	int ret(0);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(IsInList(seq,ScanList)){
			ret++;
			logger.Append(LogPre + seq + LogPost);
		}
	}
	return ret;
}

int CFirewall::DenySequencesCount(CStringList &ScanList, CStringList &Sequences, CString LogPre, CString LogPost, CStringList& Log)
{
	CString seq;
	int ret(0);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(IsInList(seq,ScanList)){
			ret++;
			Log.AddTail(LogPre + seq + LogPost);
		}
	}
	return ret;
}

int CFirewall::DenySequencesCount(CStringList &ScanList1, CStringList& ScanList2, CStringList &Sequences, CString LogPre, CString LogPost)
{
	CString seq;
	int ret(0);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(IsInList(seq,ScanList1) || IsInList(seq,ScanList2)){
			ret++;
			logger.Append(LogPre + seq + LogPost);
		}
	}
	return ret;
}

int CFirewall::DenySequencesCount(CStringList &ScanList1, CStringList& ScanList2, CStringList &Sequences, CString LogPre, CString LogPost, CStringList& Log)
{
	CString seq;
	int ret(0);
	POSITION pos = Sequences.GetHeadPosition();
	while(pos!=NULL){
		seq = Sequences.GetNext(pos);
		if(IsInList(seq,ScanList1) || IsInList(seq,ScanList2)){
			ret++;
			Log.AddTail(LogPre + seq + LogPost);
		}
	}
	return ret;
}

CString CFirewall::LoadSettingFile(CString& Path, LPCTSTR SettingFile, LPCTSTR BackupFile1, LPCTSTR BackupFile2)
{
	CString strinit("INFO: Settings loaded from file: ");

	//initialize settings
	if(Settings.ReadFromFile(Path + SettingFile)){
		strinit += SettingFile;
	}else{
		if(BackupFile1!=NULL){
			if(Settings.ReadFromFile(Path + BackupFile1)){
				strinit += BackupFile1;
			}else{
				if(BackupFile2!=NULL){
					if(Settings.ReadFromFile(Path + BackupFile2)){
						strinit += BackupFile2;
					}else{
						strinit = "WARNING: Failed to load settings from " + CString(SettingFile) + ", " + BackupFile1 + " or " + BackupFile2 + " file (defaults loaded)";
					}
				}else{
					strinit = "WARNING: Failed to load settings from " + CString(SettingFile) + ", " + BackupFile1 + " file (defaults loaded)";
				}
			}
		}else{
			strinit = "WARNING: Failed to load settings from " + CString(SettingFile) + " file (defaults loaded)";
		}
	}
	return strinit;
}

CString CFirewall::SaveDefaultSettingFile(CString& Path, LPCTSTR SettingFile)
{
	//make a settings file if it doesn't exist
	DWORD attr = GetFileAttributes(Path + SettingFile);
	if (attr == 0xFFFFFFFF){ //doesn't exist yet
		if(Settings.WriteToFile(Path + SettingFile)){
			Settings.ReadFromFile(Path + SettingFile);//for runtime update
			return "INFO: A file '" + CString(SettingFile) + "' was made with the currently loaded settings.";
		}else{
			return "WARNING: Failed to make a settings file: " + CString(SettingFile);
		}
	}
	return "";
}

CString CFirewall::InitializeLogger(LPCTSTR DefaultLogFolder, LPCTSTR SoftwareHeader)
{
	logger.SetHeader(SoftwareHeader);
	SetLoggerFields();

	CString dir = Settings.Logging.LogDirectory;
	if(dir==""){
		dir = DefaultLogFolder;
	}
	if(Settings.Logging.PerProcess){
		DWORD id = GetCurrentProcessId();
		dir += CStringHelper::Safe_DWORDToAscii(id);
		dir += '\\';
	}
	logger.SetLogDir(dir);
	logger.SetRetention(Settings.Logging.Retention);
	logger.Enabled = Settings.Logging.Enabled;
	logger.GMTLogging = Settings.Logging.UseGMT;

#ifdef ENABLE_ODBC
	if(Settings.Logging.LogODBC){
		if(logger.SetODBC(Settings.Logging.ODBCDSN,Settings.Logging.ODBCTable)){
			return "INFO: ODBC logging enabled";
		}else{
			return "WARNING: ODBC logging failed at startup, will retry on every new log entry!";
		}
	}
#endif
	return "";
}

void CFirewall::SetLoggerFields()
{
	//set logger fields
	CStringList fields;
	fields.AddTail("Date");
	fields.AddTail("Time");
	fields.AddTail("Site Instance");
	fields.AddTail("Event");
	if(Settings.Logging.LogUserName) fields.AddTail("Username");
	fields.AddTail("Additional info about request (event specific)");
	logger.SetHeaderFields(fields);
}

void CFirewall::UpdateSettings()
{
	if (Settings.Update()){	//run-time update
		SetLoggerFields(); //update logger header
		LoadCache();
		CFileName cfn(Settings.GetReadSettingsFileName());
		CString fnLoaded = cfn.Path + "Loaded" + cfn.Extension;
		Settings.WriteToFile(fnLoaded);//write loaded settings to a debug file
		logger.NewEntryFlush("INFO: Detected change in settings, reload done!");
		logger.NewEntryFlush("INFO: You can have a look at the currently loaded settings in the file '" + fnLoaded + "'.");
	}
}

void CFirewall::SaveRunningValues(CString &Path, LPCTSTR FileName)
{
	//write loaded settings to a file (debug/additional check)
	if(Settings.WriteToFile(Path + FileName))
		logger.NewEntryFlush("INFO: To check if loaded correctly, you can have a look at the currently loaded settings in the file '" + CString(FileName) + "' (start config.exe and open this file).");
	//if(Settings.WriteToFile(Path + "Loaded.ini"))
	//	logger.NewEntryFlush("INFO: To check if WebKnight is loaded correctly, you can have a look at the currently loaded settings in the file 'Loaded.ini'.");
}

void CFirewall::WarnLogonlyMode()
{
	//are we in logging only mode? if so, warning!
	if(Settings.Response.LogOnly)
		logger.NewEntryFlush("WARNING: Firewall is in logging only mode! Nothing will be blocked!");
}