/*
    AQTRONIX C++ Library
    Copyright 2005-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// FirewallSettings.cpp: implementation of the CFirewallSettings class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FirewallSettings.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define MAX_INI_FILE_SIZE 50000

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFirewallSettings::CFirewallSettings()
{

}

CFirewallSettings::~CFirewallSettings()
{

}

CString CFirewallSettings::GetXMLLogging(CFirewallSettings& d)
{
	CString xml("");
	//Logging
	WRITE_XML_BOOL(Logging,Enabled,"Enabled","Enable or disable logging.\r\nRequires restart of firewall!")
	xml += ConvertStringToXML("Log_Directory","Text",Logging.LogDirectory,LogDirDefault,"The directory where the log files will be placed.\r\nRequires restart of firewall!");
#ifdef ENABLE_ODBC
	WRITE_XML_BOOL(Logging,LogODBC,"Use_ODBC","Log to ODBC database.\r\nRequires restart of firewall!")
	WRITE_XML_STRING(Logging,ODBCDSN,"Log_ODBC_DSN","ODBC connection string like: 'DSN=mydsn;DATABASE=mydatabase;'.\r\nRequires restart of firewall!")
	WRITE_XML_STRING(Logging,ODBCTable,"Log_ODBC_Table","Database table to log to.\r\nRequires restart of firewall!")
#endif
	WRITE_XML_BOOL(Logging,UseGMT,"Use_GMT","Log dates and times in GMT/UTC.\r\nRequires restart of firewall!")
	WRITE_XML_BOOL(Logging,PerProcess,"Per_Process_Logging","Make a unique log file per web server process. This is for web servers that can host the filter in more than 1 process concurrently.\r\nRequires restart of firewall!")
	WRITE_XML_INT(Logging,Retention,"Log_Retention","The rotation period (in days) to keep the log files.\r\nRequires restart of firewall!")
	WRITE_XML_BOOL(Logging,LogClientIP,"Log_Client_IP","Log the client IP address.")
	WRITE_XML_BOOL(Logging,LogUserName,"Log_User_Name","Log the username the client is logged on with.")
	WRITE_XML_BOOL(Logging,LogAllowed,"Log_Allowed","In addition to logging blocked requests you can log allowed requests as well. This has high performance impact on heavy loaded systems and is not recommended!")
	return xml;
}

CString CFirewallSettings::GetXMLResponseHandling(CFirewallSettings& d)
{
	CString xml("");
	//Response Handling
	WRITE_XML_BOOL(Response,LogOnly,"Response_Log_Only","If an attack is detected, only log and do not block it.")
	return xml;
}

bool CFirewallSettings::WriteToFileINI(LPCTSTR fn)
{
	//Response handling
	WRITE_INI_BOOL(Response,LogOnly)

	//Logging
	WRITE_INI_BOOL(Logging,Enabled)
	WRITE_INI_STRING(Logging,LogDirectory)
#ifdef ENABLE_ODBC
	WRITE_INI_BOOL(Logging,LogODBC)
	WRITE_INI_STRING(Logging,ODBCDSN)
	WRITE_INI_STRING(Logging,ODBCTable)
#endif
	WRITE_INI_BOOL(Logging,UseGMT)
	WRITE_INI_BOOL(Logging,PerProcess)
	WRITE_INI_INT(Logging,Retention)
	WRITE_INI_BOOL(Logging,LogClientIP)
	WRITE_INI_BOOL(Logging,LogUserName)
	WRITE_INI_BOOL(Logging,LogAllowed)

	return true;
}

bool CFirewallSettings::ReadFromFileINI(LPCTSTR fn)
{
	const int sz = MAX_INI_FILE_SIZE;
	char buf[sz];

	//Response
	READ_INI_BOOL(Response,LogOnly)

	//Logging
	READ_INI_BOOL(Logging,Enabled)
	READ_INI_STRING(Logging,LogDirectory)
#ifdef ENABLE_ODBC
	READ_INI_BOOL(Logging,LogODBC)
	READ_INI_STRING(Logging,ODBCDSN)
	READ_INI_STRING(Logging,ODBCTable)
#endif
	READ_INI_BOOL(Logging,UseGMT)
	READ_INI_BOOL(Logging,PerProcess)
	READ_INI_INT(Logging,Retention)
	READ_INI_BOOL(Logging,LogClientIP)
	READ_INI_BOOL(Logging,LogUserName)
	READ_INI_BOOL(Logging,LogAllowed)

	return true;
}

bool CFirewallSettings::ParseXML(CString &key, CString &val)
{
	//if not a valid key
	if (key=="")
		return false;

	CString name = XMLDecode(XMLKeyParameter(key,"name"));
	
	//Response Handling
	PARSE_XML_BOOL(Response,LogOnly,"Response_Log_Only")

	//Logging
	else PARSE_XML_BOOL(Logging,Enabled,"Enabled")
	else PARSE_XML_STRING(Logging,LogDirectory,"Log_Directory")
	else PARSE_XML_BOOL(Logging,UseGMT,"Use_GMT")
	else PARSE_XML_BOOL(Logging,PerProcess,"Per_Process_Logging")
	else PARSE_XML_INT(Logging,Retention,"Log_Retention",(int))
	else PARSE_XML_BOOL(Logging,LogAllowed,"Log_Allowed_Requests")
	else PARSE_XML_BOOL(Logging,LogAllowed,"Log_Allowed")
	else PARSE_XML_BOOL(Logging,LogClientIP,"Log_Client_IP")
	else PARSE_XML_BOOL(Logging,LogUserName,"Log_User_Name")
#ifdef ENABLE_ODBC
	else PARSE_XML_BOOL(Logging,LogODBC,"Use_ODBC")
	else PARSE_XML_STRING(Logging,ODBCDSN,"Log_ODBC_DSN")
	else PARSE_XML_STRING(Logging,ODBCTable,"Log_ODBC_Table")
#endif
	else{
		return false;
	}
	return true;
}

CString CFirewallSettings::ToXML()
{
	CString xml("");
	CFirewallSettings d;
	xml += GetXMLResponseHandling(d);
	xml += GetXMLLogging(d);
	return xml;
}

void CFirewallSettings::LoadDefaults()
{
#pragma message("***** When redistributing your settings file, remove the LogDirectory entry, so it will take the default! *****")
}

bool CFirewallSettings::WriteToFileXML(LPCTSTR fn)
{
	return WriteXMLEntityToFile(fn,"CFirewallSettings",ToXML());
}
