/*
    AQTRONIX C++ Library
    Copyright 2004-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// HTTPFirewallSettings.cpp: implementation of the CHTTPFirewallSettings class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "HTTPFirewallSettings.h"

#define MAX_INI_FILE_SIZE 50000	//TODO

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHTTPFirewallSettings::CHTTPFirewallSettings()
{

}

CHTTPFirewallSettings::~CHTTPFirewallSettings()
{

}

void CHTTPFirewallSettings::LoadDefaults()
{
	HTTPResponse.Contents = "HTTP Firewall Alert";
}

bool CHTTPFirewallSettings::ProcessFile(LPCTSTR fn)
{
	//if file is a urlscan.ini file
	CFileName cfn(fn);
	if(cfn.FileNameLCase == "urlscan.ini"){		//urlscan.ini file
		return ReadFromFileURLScan(fn);
	}else if(cfn.ExtensionLCase == ".ini"){		//.ini file
		return ReadFromFileINI(fn);
	}else if(cfn.ExtensionLCase == ".xml"){		//.xml file
		return ReadFromFileXML(fn);
	}else{
		return false;							//unknown file type
	}
}

bool CHTTPFirewallSettings::ReadFromFileURLScan(LPCTSTR fn)
{
	/* Read settings from a urlscan.ini file
	 * All checks on the file (exists? access allowed? ...)
	 * should have been done before this function is called
	 */
	try{
		const int sz = MAX_INI_FILE_SIZE;
		char buf[sz];

		//[options]
		Verbs.AllowVerbs.Enabled = GetPrivateProfileInt("options","UseAllowVerbs",Verbs.AllowVerbs.Enabled,fn)>0;
		Verbs.DenyVerbs.Enabled = !GetPrivateProfileInt("options","UseAllowVerbs",!Verbs.DenyVerbs.Enabled,fn);
		Extensions.AllowExtensions.Enabled = GetPrivateProfileInt("options","UseAllowExtensions",Extensions.AllowExtensions.Enabled,fn)>0;
		Extensions.DenyExtensions.Enabled = !GetPrivateProfileInt("options","UseAllowExtensions",!Extensions.DenyExtensions.Enabled,fn);
		Path.DenyDot = !GetPrivateProfileInt("options","AllowDotInPath",!Path.DenyDot,fn);
		Logging.Enabled = GetPrivateProfileInt("options","EnableLogging",Logging.Enabled,fn)>0;
		Engine.AllowLateScanning = GetPrivateProfileInt("options","AllowLateScanning",Engine.AllowLateScanning,fn)>0;
		
		CString url(HTTPResponse.RedirectURL);
		GetPrivateProfileString("options","RejectResponseUrl","",buf,sz,fn);
		HTTPResponse.RedirectURL = buf; CleanupString(HTTPResponse.RedirectURL);
		if(HTTPResponse.RedirectURL=="") HTTPResponse.RedirectURL=url;
		HTTPResponse.Redirect = !GetPrivateProfileInt("options","UseFastPathReject",!HTTPResponse.Redirect,fn);
		HTTPResponse.Directly = GetPrivateProfileInt("options","UseFastPathReject",0,fn)>0;

		Headers.RemoveServerHeader = GetPrivateProfileInt("options","RemoveServerHeader",Headers.RemoveServerHeader,fn)>0;
		GetPrivateProfileString("options","AlternateServerName","",buf,sz,fn);
		Headers.ServerHeader = buf; CleanupString(Headers.ServerHeader);
		Headers.ChangeServerHeader = Headers.ServerHeader.GetLength()>0;

		CString ldir(Logging.LogDirectory);
		GetPrivateProfileString("options","LoggingDirectory","",buf,sz,fn);
		Logging.LogDirectory = buf; CleanupString(Logging.LogDirectory);
		if(Logging.LogDirectory=="") Logging.LogDirectory=ldir;

		//NormalizeURLBeforeScan = GetPrivateProfileInt("options","NormalizeURLBeforeScan",1,fn);
		//VerifyNormalization = GetPrivateProfileInt("options","VerifyNormalization",1,fn);
		URL.DenyHighBitShellcode = !GetPrivateProfileInt("options","AllowHighBitCharacters",!URL.DenyHighBitShellcode,fn);
	
		//LogLongUrls = GetPrivateProfileInt("options","LogLongUrls",0,fn);
		//PerDayLogging = GetPrivateProfileInt("options","PerDayLogging",1,fn);
		Logging.PerProcess = GetPrivateProfileInt("options","PerProcessLogging",0,fn)>0;
		
		DWORD l;

		//[AllowVerbs]
		l = GetPrivateProfileSection("AllowVerbs",buf, sz,fn);
		Split(buf,l,Verbs.AllowVerbs.List);
		CleanupList(Verbs.AllowVerbs.List);
		
		//[DenyVerbs]
		l = GetPrivateProfileSection("DenyVerbs",buf, sz,fn);
		Split(buf,l,Verbs.DenyVerbs.List);
		CleanupListUCase(Verbs.DenyVerbs.List);
		
		//[DenyHeaders]
		l = GetPrivateProfileSection("DenyHeaders",buf, sz,fn);
		Split(buf,l,Headers.DenyHeaders.List);
		CleanupList(Headers.DenyHeaders.List);
		Headers.DenyHeaders.Enabled = l>0;

		//[AllowExtensions]
		l = GetPrivateProfileSection("AllowExtensions",buf, sz,fn);
		Split(buf,l,Extensions.AllowExtensions.List);
		if(l) Extensions.AllowExtensions.List.AddHead("");			//always allow default document!!!
		CleanupListLCase(Extensions.AllowExtensions.List);

		//[DenyExtensions]
		l = GetPrivateProfileSection("DenyExtensions",buf, sz,fn);
		Split(buf,l,Extensions.DenyExtensions.List);
		CleanupListLCase(Extensions.DenyExtensions.List);
		
		//[DenyUrlSequences]
		l = GetPrivateProfileSection("DenyUrlSequences",buf, sz,fn);
		Split(buf,l,URL.DenySequences.List);
		CleanupListLCase(URL.DenySequences.List);
		URL.DenySequences.Enabled = l>0;

		//[RequestLimits]
		GetPrivateProfileString("RequestLimits","MaxAllowedContentLength",Limits.MaxContentLength,buf,sz,fn);
		Limits.MaxContentLength = buf; CleanupString(Limits.MaxContentLength);
		Limits.LimitContentLength = Limits.MaxContentLength.GetLength()>0;

		Limits.MaxURL = GetPrivateProfileInt("RequestLimits","MaxUrl",Limits.MaxURL,fn);
		Limits.LimitURL = Limits.MaxURL>0;

		Limits.MaxQueryString = GetPrivateProfileInt("RequestLimits","MaxQueryString",Limits.MaxQueryString,fn);
		Limits.LimitQueryString = Limits.MaxQueryString>0;

		l = GetPrivateProfileSection("RequestLimits",buf, sz,fn);
		CStringList MaxLimits; POSITION oldpos;
		Split(buf,l,MaxLimits);
		POSITION pos = MaxLimits.GetHeadPosition();
		while (pos != NULL){
			oldpos = pos;
			CString& str = MaxLimits.GetNext(pos);
			CleanupString(str);
			if (str.Left(4)!="Max-")
				MaxLimits.RemoveAt(oldpos);
		}
		if(l>0){
			Limits.MaxHeaders.Map.RemoveAll();	//if section is found, remove defaults
			pos = MaxLimits.GetHeadPosition();
			CString val; int poseq;
			while (pos!=NULL){
				CString& str = MaxLimits.GetNext(pos);
				poseq = str.Find('=');
				if(poseq!=-1){
					if(str.Mid(poseq+1) != "")
						Limits.MaxHeaders.Map.SetAt(str.Mid(4,poseq-4)+":",str.Mid(poseq+1));
				}
			}
			Limits.MaxHeaders.Enabled = true;
		}else{
			Limits.MaxHeaders.Enabled = false;
		}

		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

bool CHTTPFirewallSettings::ReadFromActiveDirectory()
{
	/* Read settings from Active Directory (enterprise policy)
	 * If you want to apply the same settings for multiple
	 * machines, use this.
	 * Requires domain controller (LDAP server)
	 */
	try{
		//TODO
		
		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

bool CHTTPFirewallSettings::ReadFromRegistry()
{
	/*
	 * read settings from the windows registry
	 */
	try{
		//TODO
		//parser gebruiken?

		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

bool CHTTPFirewallSettings::WriteToRegistry()
{
	/*
	 * Write settings to the windows registry (machine policy)
	 */
	try{
		//TODO
		
		return true;
	}catch(...){
		return false;
	}
}

bool CHTTPFirewallSettings::WriteToActiveDirectory()
{
	/*
	 * Write settings to Active Directory (enterprise policy)
	 */
	try{
		//TODO

		return true;
	}catch(...){
		return false;
	}
}

bool CHTTPFirewallSettings::ReadFromFileINI(LPCTSTR fn)
{
	try{
		const int sz = MAX_INI_FILE_SIZE;
		char buf[sz];
		DWORD l;

		CFirewallSettings::ReadFromFileINI(fn);

		//Scanning Engine
		READ_INI_BOOL(Engine,AllowLateScanning)
		READ_INI_BOOL(Engine,ScanNonSecurePort)
		READ_INI_BOOL(Engine,ScanSecurePort)
		READ_INI_LIST(Engine,ExcludedInstances,CleanupList)
		READ_INI_LIST(Engine,ExcludedIPAddresses,CleanupList)
				
		//Intrusion Response Handling
		HTTPResponseParameters& Response = HTTPResponse;
		READ_INI_BOOL(Response,Directly)
		READ_INI_BOOL(Response,Redirect)
		READ_INI_STRING(Response,RedirectURL)
		READ_INI_BOOL(Response,UseStatus)
		READ_INI_STRING(Response,Status)
		READ_INI_BOOL(Response,DropConnection)
		READ_INI_BOOL(Response,MonitorIP)
		READ_INI_INT(Response,MonitorIPTimeout)
		READ_INI_BOOL(Response,BlockIP)
		READ_INI_INT(Response,BlockIPTimeout)

		//Logging
		HTTPLoggingParameters& Logging = HTTPLogging;
		READ_INI_BOOL(Logging,Log_HTTP_VIA)
		READ_INI_BOOL(Logging,Log_HTTP_X_FORWARDED_FOR)
		READ_INI_BOOL(Logging,LogHostHeader)
		READ_INI_BOOL(Logging,LogUserAgent)
		READ_INI_BOOL(Logging,LogClientErrors)
		READ_INI_BOOL(Logging,LogServerErrors)

		//Connection
		READ_INI_LIST(Connection,MonitorAddresses,CleanupList)
		READ_INI_LIST(Connection,DenyAddresses,CleanupList)
		READ_INI_CACHE(Connection,RequestsLimit)

		//Authentication
		READ_INI_BOOL(Authentication,ScanExcludedInstances)
		READ_INI_BOOL(Authentication,DenyBlankPasswords)
		READ_INI_LIST(Authentication,DenyDefaultPasswords,CleanupList)
		READ_INI_BOOL(Authentication,DenySamePasswordAsUsername)
		READ_INI_BOOL(Authentication,DenySystemAccounts)
		READ_INI_CACHE(Authentication,DenyBruteForceAttack)
		READ_INI_BOOL(Authentication,ScanAccountAllEvents)
		READ_INI_LIST(Authentication,AllowAccounts,CleanupList)
		READ_INI_LIST(Authentication,DenyAccounts,CleanupList)
		
		//Request Limits
		READ_INI_BOOL(Limits,LimitContentLength)
		READ_INI_BOOL(Limits,LimitURL)
		READ_INI_BOOL(Limits,LimitQueryString)
		READ_INI_BOOL(Limits,LimitHTTPVersion)
		READ_INI_STRING(Limits,MaxContentLength)
		READ_INI_INT(Limits,MaxURL)
		READ_INI_INT(Limits,MaxQueryString)
		READ_INI_INT(Limits,MaxHTTPVersion)
		READ_INI_LIST(Limits,AllowHTTPVersions,CleanupListUCase)
		READ_INI_MAP(Limits,MaxHeaders,CleanupList)
		//TODO: Add ':' to header name

		//URL Scanning
		READ_INI_BOOL(URL,RFCCompliantURL)
		READ_INI_BOOL(URL,RFCCompliantHTTPURL)
		READ_INI_BOOL(URL,UseRawScan)
		READ_INI_BOOL(URL,DenyEncodingExploits)
		READ_INI_BOOL(URL,DenyParentPath)
		READ_INI_BOOL(URL,DenyTrailingDotInDir)
		READ_INI_BOOL(URL,DenyBackslash)
		READ_INI_BOOL(URL,DenyAlternateStream)
		READ_INI_BOOL(URL,DenyEscaping)
		READ_INI_BOOL(URL,DenyMultipleCGI)
		READ_INI_STRING(URL,DenyCharacters)
		READ_INI_BOOL(URL,DenyHighBitShellcode)
		READ_INI_LIST(URL,DenySequences,CleanupListLCase)
		READ_INI_LIST(URL,AllowURL,CleanupListLCase)

		//Mapped Path
		READ_INI_BOOL(Path,DenyParentPath)
		READ_INI_BOOL(Path,DenyBackspace)
		READ_INI_BOOL(Path,DenyCarriageReturn)
		READ_INI_BOOL(Path,DenyNewLine)
		READ_INI_BOOL(Path,DenyEscaping)
		READ_INI_BOOL(Path,DenyDot)
		READ_INI_STRING(Path,DenyCharacters)
		READ_INI_LIST(Path,AllowPaths,CleanupListLCase)
		
		//Requested File
		READ_INI_BOOL(Filename,UseRawScan)
		READ_INI_STRING(Filename,DenyCharacters)
		READ_INI_BOOL(Filename,DenyDefaultDocument)
		READ_INI_LIST(Filename,DenyFiles,CleanupListLCase)
		READ_INI_LIST(Filename,MonitorFiles,CleanupListLCase)
		
		//Requested Extension
		READ_INI_LIST(Extensions,AllowExtensions,CleanupListLCase)
		READ_INI_LIST(Extensions,DenyExtensions,CleanupListLCase)
		if(!IsInList("",Extensions.AllowExtensions.List))
			Extensions.AllowExtensions.List.AddHead(""); //always allow default document
		
		//HTTP Headers
		READ_INI_BOOL(Headers,RemoveServerHeader)
		READ_INI_BOOL(Headers,ChangeServerHeader)
		READ_INI_STRING(Headers,ServerHeader)
		READ_INI_BOOL(Headers,RFCCompliantHostHeader)
		READ_INI_BOOL(Headers,DenySQLInjection)
		READ_INI_BOOL(Headers,DenyEncodingExploits)
		READ_INI_BOOL(Headers,DenyDirectoryTraversal)
		READ_INI_BOOL(Headers,DenyHighBitShellcode)
		READ_INI_LIST(Headers,DenyHeaders,CleanupList)
		READ_INI_LIST(Headers,DenySequences,CleanupListLCase)

		//CONTENTTYPE
		READ_INI_LIST(ContentType,AllowContentTypes,CleanupList)
		if(!IsInList("",ContentType.AllowContentTypes.List))
			ContentType.AllowContentTypes.List.AddHead(""); //always allow no content-type
		READ_INI_LIST(ContentType,DenyContentTypes,CleanupList)

		//COOKIE
		READ_INI_BOOL(Cookie,DenySQLInjection)
		READ_INI_BOOL(Cookie,DenyEncodingExploits)
		READ_INI_BOOL(Cookie,DenyDirectoryTraversal)
		READ_INI_BOOL(Cookie,DenyHighBitShellcode)
		READ_INI_LIST(Cookie,DenySequences,CleanupListLCase)

		//USER-AGENT
		READ_INI_BOOL(UserAgent,DenyEmpty)
		READ_INI_BOOL(UserAgent,DenyNonRFC)
		READ_INI_STRING(UserAgent,RequireCharacter)
		READ_INI_LIST(UserAgent,DenyUserAgents,CleanupList)
		READ_INI_LIST(UserAgent,DenySequences,CleanupListLCase)

		//REFERRER
		READ_INI_BOOL(Referrer,UseScan)
		READ_INI_BOOL(Referrer,RFCCompliantURL)
		READ_INI_BOOL(Referrer,RFCCompliantHTTPURL)
		READ_INI_BOOL(Referrer,DenyEncodingExploits)
		READ_INI_BOOL(Referrer,DenyHotLinking)
		READ_INI_CSTRINGLIST(Referrer,HotLinkingFileExtensions,CleanupListLCase)
		READ_INI_LIST(Referrer,HotLinkingAllowDomains,CleanupListLCase)
		READ_INI_LIST(Referrer,HotLinkingDenyDomains,CleanupListLCase)
		READ_INI_BOOL(Referrer,HotLinkingUseHostHeader)
		READ_INI_BOOL(Referrer,HotLinkingDenyBlankReferrer)
		READ_INI_STRING(Referrer,DenyCharacters)
		READ_INI_BOOL(Referrer,DenyHighBitShellcode)
		READ_INI_BOOL(Referrer,DenySQLInjection)
		READ_INI_LIST(Referrer,DenySequences,CleanupListLCase)

		//HTTP Verbs (Request Methods)
		READ_INI_LIST(Verbs,AllowVerbs,CleanupList)
		READ_INI_LIST(Verbs,DenyVerbs,CleanupListUCase)

		//SQL Injection Keywords (TODO: ';' cannot be read again)
		//l = GetPrivateProfileSection("SQL Injection Keywords",buf,sz,fn);
		//Split(buf,l,SQLKeyWords);
		//CleanupListLCase(SQLKeyWords);

		//Querystring Scanning
		READ_INI_BOOL(QueryString,UseRawScan)
		READ_INI_BOOL(QueryString,DenySQLInjection)
		READ_INI_BOOL(QueryString,DenyEncodingExploits)
		READ_INI_BOOL(QueryString,DenyDirectoryTraversal)
		READ_INI_BOOL(QueryString,DenyHighBitShellcode)
		READ_INI_LIST(QueryString,DenySequences,CleanupListLCase)

		//Global Filter Capabilities
		READ_INI_BOOL(Global,IsInstalledAsGlobalFilter)
		READ_INI_BOOL(Global,DenyPostSQLInjection)
		READ_INI_BOOL(Global,DenyPostEncodingExploits)
		READ_INI_BOOL(Global,DenyPostDirectoryTraversal)
		READ_INI_BOOL(Global,DenyPostHighBitShellcode)
		READ_INI_LIST(Global,DenyPostSequences,CleanupListLCase)

		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

bool CHTTPFirewallSettings::ParseXML(CString &key, CString &val)
{
	//if not a valid key
	if (key=="")
		return false;

	if(!CFirewallSettings::ParseXML(key,val)){

		CString name = XMLDecode(XMLKeyParameter(key,"name"));

		bool done(true);

		//Scanning Engine
		PARSE_XML_BOOL(Engine,AllowLateScanning,"Allow_Late_Scanning")
		else PARSE_XML_BOOL(Engine,ScanNonSecurePort,"Scan_Non_Secure_Port")
		else PARSE_XML_BOOL(Engine,ScanSecurePort,"Scan_Secure_Port")
		else PARSE_XML_LIST(Engine,ExcludedInstances,"Excluded_Web_Instances")
		else PARSE_XML_LIST(Engine,ExcludedIPAddresses,"Excluded_IP_Addresses")

		//Response Handling
		else PARSE_XML_BOOL(HTTPResponse,Directly,"Response_Directly")
		else PARSE_XML_BOOL(HTTPResponse,Redirect,"Response_Redirect")
		else PARSE_XML_STRING(HTTPResponse,RedirectURL,"Response_Redirect_URL")
		else PARSE_XML_BOOL(HTTPResponse,UseStatus,"Use_Response_Status")
		else PARSE_XML_STRING(HTTPResponse,Status,"Response_Status")
		else PARSE_XML_BOOL(HTTPResponse,DropConnection,"Response_Drop_Connection")
		else PARSE_XML_BOOL(HTTPResponse,MonitorIP,"Response_Monitor_IP")
		else PARSE_XML_INT(HTTPResponse,MonitorIPTimeout,"Response_Monitor_IP_Timeout",)
		else PARSE_XML_BOOL(HTTPResponse,BlockIP,"Response_Block_IP")
		else PARSE_XML_INT(HTTPResponse,BlockIPTimeout,"Response_Block_IP_Timeout",)

		//Logging
		else PARSE_XML_BOOL(HTTPLogging,Log_HTTP_VIA,"Log_HTTP_VIA")
		else PARSE_XML_BOOL(HTTPLogging,Log_HTTP_X_FORWARDED_FOR,"Log_HTTP_X_FORWARDED_FOR")
		else PARSE_XML_BOOL(HTTPLogging,LogHostHeader,"Log_Host_Header")
		else PARSE_XML_BOOL(HTTPLogging,LogUserAgent,"Log_User_Agent")
		else PARSE_XML_BOOL(HTTPLogging,LogClientErrors,"Log_HTTP_Client_Errors")
		else PARSE_XML_BOOL(HTTPLogging,LogServerErrors,"Log_HTTP_Server_Errors")

		//Connection
		else PARSE_XML_LIST(Connection,MonitorAddresses,"Monitored_IP_Addresses")
		else PARSE_XML_LIST(Connection,DenyAddresses,"Denied_IP_Addresses")
		else PARSE_XML_CACHE(Connection,RequestsLimit,"Connection_Requests_Limit")

		//Authentication
		else PARSE_XML_BOOL(Authentication,ScanExcludedInstances,"Scan_Authentication_Excluded_Web_Instances")
		else PARSE_XML_BOOL(Authentication,DenyBlankPasswords,"Deny_Blank_Passwords")
		else PARSE_XML_BOOL(Authentication,DenySamePasswordAsUsername,"Deny_Same_Password_As_Username")
		else PARSE_XML_LIST(Authentication,DenyDefaultPasswords,"Denied_Default_Passwords")
		else PARSE_XML_BOOL(Authentication,DenySystemAccounts,"Deny_System_Accounts")
		else PARSE_XML_CACHE(Authentication,DenyBruteForceAttack,"Deny_Account_Brute_Force_Attack")
		else PARSE_XML_LIST(Authentication,AllowAccounts,"Allowed_Accounts")
		else PARSE_XML_LIST(Authentication,DenyAccounts,"Denied_Accounts")
		else PARSE_XML_BOOL(Authentication,ScanAccountAllEvents,"Scan_Account_All_Events")
			
		//Request Limits
		else PARSE_XML_BOOL(Limits,LimitContentLength,"Limit_Content_Length")
		else PARSE_XML_BOOL(Limits,LimitURL,"Limit_URL")
		else PARSE_XML_BOOL(Limits,LimitQueryString,"Limit_Querystring")
		else PARSE_XML_BOOL(Limits,LimitHTTPVersion,"Limit_HTTP_Version")
		else PARSE_XML_STRING(Limits,MaxContentLength,"Max_Content_Length")
		else PARSE_XML_INT(Limits,MaxURL,"Max_URL",)
		else PARSE_XML_INT(Limits,MaxQueryString,"Max_Querystring",)
		else PARSE_XML_INT(Limits,MaxHTTPVersion,"Max_HTTP_Version",)
		else PARSE_XML_LIST(Limits,AllowHTTPVersions,"Allowed_HTTP_Versions")
		else PARSE_XML_MAP(Limits,MaxHeaders,"Max_Headers")

		//URL Scanning
		else PARSE_XML_BOOL(URL,RFCCompliantURL,"RFC_Compliant_Url")
		else PARSE_XML_BOOL(URL,RFCCompliantHTTPURL,"RFC_Compliant_HTTP_Url")
		else PARSE_XML_BOOL(URL,UseRawScan,"Use_Url_Raw_Scan")
		else PARSE_XML_BOOL(URL,DenyEncodingExploits,"Deny_Url_Encoding_Exploits")
		else PARSE_XML_BOOL(URL,DenyParentPath,"Deny_Url_Parent_Path")
		else PARSE_XML_BOOL(URL,DenyTrailingDotInDir,"Deny_Url_Trailing_Dot_In_Dir")
		else PARSE_XML_BOOL(URL,DenyBackslash,"Deny_Url_Backslash")
		else PARSE_XML_BOOL(URL,DenyAlternateStream,"Deny_Url_Alternate_Stream")
		else PARSE_XML_BOOL(URL,DenyEscaping,"Deny_Url_Escaping")
		else PARSE_XML_BOOL(URL,DenyMultipleCGI,"Deny_Url_Running_Multiple_CGI")
		else PARSE_XML_STRING(URL,DenyCharacters,"Deny_Url_Characters")
		else PARSE_XML_BOOL(URL,DenyHighBitShellcode,"Deny_Url_HighBitShellcode")

		else PARSE_XML_BOOL(URL,DenySequences.Enabled,"Use_Denied_Url_Sequences")
		else PARSE_XML_CSTRINGLIST(URL,DenySequences.List,"URL_Denied_Sequences")

		else PARSE_XML_BOOL(URL,AllowURL.Enabled,"UseAllowedUrlStarts")		//for backward compatibility with WebKnight 1.0 & 1.1
		else PARSE_XML_BOOL(URL,AllowURL.Enabled,"Use_Allowed_Url_Starts")
		else PARSE_XML_CSTRINGLIST(URL,AllowURL.List,"URL_Allowed_starts")
				
		//Mapped Path
		else PARSE_XML_BOOL(Path,DenyParentPath,"Deny_Parent_Path")
		else PARSE_XML_BOOL(Path,DenyBackspace,"Deny_Backspace")
		else PARSE_XML_BOOL(Path,DenyCarriageReturn,"Deny_Carriage_Return")
		else PARSE_XML_BOOL(Path,DenyNewLine,"Deny_New_Line")
		else PARSE_XML_BOOL(Path,DenyEscaping,"Deny_Escaping")
		else PARSE_XML_BOOL(Path,DenyDot,"Deny_Dot_In_Path")
		else PARSE_XML_STRING(Path,DenyCharacters,"Deny_Characters")
		else PARSE_XML_LIST(Path,AllowPaths,"Allowed_Paths")

		//performance tuning
		else{	done = false;	}
		if(!done){

			//Requested File
			PARSE_XML_BOOL(Filename,UseRawScan,"Use_Filename_Raw_Scan")
			else PARSE_XML_STRING(Filename,DenyCharacters,"Deny_Filename_Characters")
			else PARSE_XML_BOOL(Filename,DenyDefaultDocument,"Deny_Default_Document")
			else PARSE_XML_LIST(Filename,DenyFiles,"Denied_Files")
			else PARSE_XML_LIST(Filename,MonitorFiles,"Monitored_Files")

			//Requested Extension
			else PARSE_XML_LIST(Extensions,AllowExtensions,"Allowed_Extensions")
			else PARSE_XML_LIST(Extensions,DenyExtensions,"Denied_Extensions")

			//HTTP Headers
			else PARSE_XML_BOOL(Headers,RemoveServerHeader,"Remove_Server_Header")
			else PARSE_XML_BOOL(Headers,ChangeServerHeader,"Change_Server_Header")
			else PARSE_XML_STRING(Headers,ServerHeader,"Server_Header")
			else PARSE_XML_BOOL(Headers,RFCCompliantHostHeader,"RFC_Compliant_Host_Header")
			else PARSE_XML_BOOL(Headers,DenySQLInjection,"Deny_Header_SQL_Injection")
			else PARSE_XML_BOOL(Headers,DenyEncodingExploits,"Deny_Header_Encoding_Exploits")
			else PARSE_XML_BOOL(Headers,DenyDirectoryTraversal,"Deny_Header_Directory_Traversal")
			else PARSE_XML_BOOL(Headers,DenyHighBitShellcode,"Deny_Header_High_Bit_Shellcode")
			else PARSE_XML_LIST(Headers,DenyHeaders,"Denied_Headers")
			else PARSE_XML_LIST(Headers,DenySequences,"Denied_Header_Sequences")

			//CONTENTYPE
			else PARSE_XML_LIST(ContentType,AllowContentTypes,"Allowed_Content_Types")
			else PARSE_XML_LIST(ContentType,DenyContentTypes,"Deny_Content_Types")

			//COOKIE
			else PARSE_XML_BOOL(Cookie,DenySQLInjection,"Deny_Cookie_SQL_Injection")
			else PARSE_XML_BOOL(Cookie,DenyEncodingExploits,"Deny_Cookie_Encoding_Exploits")
			else PARSE_XML_BOOL(Cookie,DenyDirectoryTraversal,"Deny_Cookie_Directory_Traversal")
			else PARSE_XML_BOOL(Cookie,DenyHighBitShellcode,"Deny_Cookie_High_Bit_Shellcode")
			else PARSE_XML_LIST(Cookie,DenySequences,"Denied_Cookie_Sequences")
			
			//USER-AGENT
			else PARSE_XML_BOOL(UserAgent,DenyEmpty,"Deny_User_Agent_Empty")
			else PARSE_XML_BOOL(UserAgent,DenyNonRFC,"Deny_User_Agent_Non_RFC")
			else PARSE_XML_STRING(UserAgent,RequireCharacter,"Require_User_Agent_Character")
			else PARSE_XML_LIST(UserAgent,DenyUserAgents,"Denied_User_Agents")
			else PARSE_XML_LIST(UserAgent,DenySequences,"Denied_User_Agent_Sequences")

			//REFERRER
			else PARSE_XML_BOOL(Referrer,UseScan,"Use_Referrer_Scanning")
			else PARSE_XML_BOOL(Referrer,RFCCompliantURL,"Referrer_URL_RFC_Compliant")
			else PARSE_XML_BOOL(Referrer,RFCCompliantHTTPURL,"Referrer_URL_RFC_HTTPCompliant")
			else PARSE_XML_BOOL(Referrer,DenyEncodingExploits,"Deny_Referrer_Encoding_Exploits")
			else PARSE_XML_BOOL(Referrer,DenyHotLinking,"Deny_Referrer_Hot_Linking")
			else PARSE_XML_CSTRINGLIST(Referrer,HotLinkingFileExtensions,"Referrer_Hot_Linking_File_Extensions")
			else PARSE_XML_LIST(Referrer,HotLinkingAllowDomains,"Referrer_Hot_Linking_Allow_Domains")
			else PARSE_XML_LIST(Referrer,HotLinkingDenyDomains,"Referrer_Hot_Linking_Deny_Domains")
			else PARSE_XML_BOOL(Referrer,HotLinkingUseHostHeader,"Referrer_Hot_Linking_Use_Host_Header")
			else PARSE_XML_BOOL(Referrer,HotLinkingDenyBlankReferrer,"Referrer_Hot_Linking_Deny_Blank_Referrer")
			else PARSE_XML_STRING(Referrer,DenyCharacters,"Deny_Referrer_Characters")
			else PARSE_XML_BOOL(Referrer,DenyHighBitShellcode,"Deny_Referrer_High_Bit_Shellcode")
			else PARSE_XML_BOOL(Referrer,DenySQLInjection,"Deny_Referrer_SQL_Injection")
			else PARSE_XML_LIST(Referrer,DenySequences,"Deny_Referrer_Sequences")
			
			//HTTP Verbs (Request Methods)
			else PARSE_XML_LIST(Verbs,AllowVerbs,"Allowed_Verbs")
			else PARSE_XML_LIST(Verbs,DenyVerbs,"Denied_Verbs")

			//Querystring Scanning
			else PARSE_XML_BOOL(QueryString,UseRawScan,"Use_Querystring_Raw_Scan")
			else PARSE_XML_BOOL(QueryString,DenySQLInjection,"Deny_Querystring_SQL_Injection")
			else PARSE_XML_BOOL(QueryString,DenyEncodingExploits,"Deny_Querystring_Encoding_Exploits")
			else PARSE_XML_BOOL(QueryString,DenyDirectoryTraversal,"Deny_Querystring_Directory_Traversal")
			else PARSE_XML_BOOL(QueryString,DenyHighBitShellcode,"Deny_Querystring_High_Bit_Shellcode")
			else PARSE_XML_LIST(QueryString,DenySequences,"Denied_Querystring_Sequences")
					
			//Global Filter Capabilities
			else PARSE_XML_BOOL(Global,IsInstalledAsGlobalFilter,"Is_Installed_As_Global_Filter")
			else PARSE_XML_BOOL(Global,DenyPostSQLInjection,"Deny_Postdata_SQL_Injection")
			else PARSE_XML_BOOL(Global,DenyPostEncodingExploits,"Deny_Postdata_Encoding_Exploits")
			else PARSE_XML_BOOL(Global,DenyPostDirectoryTraversal,"Deny_Postdata_Directory_Traversal")
			else PARSE_XML_BOOL(Global,DenyPostHighBitShellcode,"Deny_Postdata_High_Bit_Shellcode")
			else PARSE_XML_LIST(Global,DenyPostSequences,"Denied_Post_Sequences")

			//SQL Injection Keywords
			else if(name.CompareNoCase("SQL_Injection_Keywords")==0){
				ConvertXMLToList(val,SQLKeyWords);
			}else{
				return false; //not parsed
			}
		}
	}
	return true;
}

bool CHTTPFirewallSettings::CheckConstraints()
{
	//base class check
	CFirewallSettings::CheckConstraints();

	//lowercase
	LCase(SQLKeyWords);
	LCase(Headers.DenySequences.List);
	LCase(Global.DenyPostSequences.List);
	LCase(URL.AllowURL.List);
	LCase(URL.DenySequences.List);
	LCase(Path.AllowPaths.List);
	LCase(QueryString.DenySequences.List);
	LCase(Filename.DenyFiles.List);
	LCase(Extensions.AllowExtensions.List);
	LCase(Extensions.DenyExtensions.List);
	LCase(UserAgent.DenySequences.List);
	LCase(Filename.MonitorFiles.List);

	LCase(Referrer.HotLinkingFileExtensions);
	LCase(Referrer.HotLinkingAllowDomains.List);
	LCase(Referrer.HotLinkingDenyDomains.List);
	LCase(Referrer.DenySequences.List);

	//uppercase
	UCase(Verbs.DenyVerbs.List);

	return true;
}

bool CHTTPFirewallSettings::WriteToFileINI(LPCTSTR fn)
{
	/* 
	 * Write all settings to a simple INI file
	 * All checks on the file should have been
	 * done before this function is called
	 */
	try{

	//Scanning engine
	WRITE_INI_BOOL(Engine,AllowLateScanning)
	WRITE_INI_BOOL(Engine,ScanNonSecurePort)
	WRITE_INI_BOOL(Engine,ScanSecurePort)
	WRITE_INI_LIST(Engine,ExcludedInstances)
	WRITE_INI_LIST(Engine,ExcludedIPAddresses)

	//Response Handling
	const HTTPResponseParameters& Response = HTTPResponse;
	WRITE_INI_BOOL(Response,Directly)
	WRITE_INI_BOOL(Response,Redirect)
	WRITE_INI_STRING(Response,RedirectURL)
	WRITE_INI_BOOL(Response,UseStatus)
	WRITE_INI_STRING(Response,Status)
	WRITE_INI_BOOL(Response,DropConnection)
	WRITE_INI_BOOL(Response,MonitorIP)
	WRITE_INI_INT(Response,MonitorIPTimeout)
	WRITE_INI_BOOL(Response,BlockIP)
	WRITE_INI_INT(Response,BlockIPTimeout)

	CFirewallSettings::WriteToFileINI(fn);

	//Logging
	const HTTPLoggingParameters& Logging = HTTPLogging;
	WRITE_INI_BOOL(Logging,Log_HTTP_VIA)
	WRITE_INI_BOOL(Logging,Log_HTTP_X_FORWARDED_FOR)
	WRITE_INI_BOOL(Logging,LogHostHeader)
	WRITE_INI_BOOL(Logging,LogUserAgent)
	WRITE_INI_BOOL(Logging,LogClientErrors)
	WRITE_INI_BOOL(Logging,LogServerErrors)

	//Connection control
	WRITE_INI_LIST(Connection,MonitorAddresses)
	WRITE_INI_LIST(Connection,DenyAddresses)
	WRITE_INI_CACHE(Connection,RequestsLimit)
	
	//Authentication
	WRITE_INI_BOOL(Authentication,ScanExcludedInstances)
	WRITE_INI_BOOL(Authentication,DenyBlankPasswords)
	WRITE_INI_BOOL(Authentication,DenySamePasswordAsUsername)
	WRITE_INI_LIST(Authentication,DenyDefaultPasswords)
	WRITE_INI_BOOL(Authentication,DenySystemAccounts)
	WRITE_INI_CACHE(Authentication,DenyBruteForceAttack)
	WRITE_INI_BOOL(Authentication,ScanAccountAllEvents)
	WRITE_INI_LIST(Authentication,AllowAccounts)
	WRITE_INI_LIST(Authentication,DenyAccounts)

	//Request Limits
	WRITE_INI_BOOL(Limits,LimitContentLength)
	WRITE_INI_BOOL(Limits,LimitURL)
	WRITE_INI_BOOL(Limits,LimitQueryString)
	WRITE_INI_BOOL(Limits,LimitHTTPVersion)
	WRITE_INI_STRING(Limits,MaxContentLength)
	WRITE_INI_INT(Limits,MaxURL)
	WRITE_INI_INT(Limits,MaxQueryString)
	WRITE_INI_INT(Limits,MaxHTTPVersion)
	WRITE_INI_LIST(Limits,AllowHTTPVersions)
	WRITE_INI_MAP(Limits,MaxHeaders)

	//URL
	WRITE_INI_BOOL(URL,RFCCompliantURL)
	WRITE_INI_BOOL(URL,RFCCompliantHTTPURL)
	WRITE_INI_BOOL(URL,UseRawScan)
	WRITE_INI_BOOL(URL,DenyEncodingExploits)
	WRITE_INI_BOOL(URL,DenyParentPath)
	WRITE_INI_BOOL(URL,DenyTrailingDotInDir)
	WRITE_INI_BOOL(URL,DenyBackslash)
	WRITE_INI_BOOL(URL,DenyAlternateStream)
	WRITE_INI_BOOL(URL,DenyEscaping)
	WRITE_INI_BOOL(URL,DenyMultipleCGI)
	WRITE_INI_STRING(URL,DenyCharacters)
	WRITE_INI_BOOL(URL,DenyHighBitShellcode)
	WRITE_INI_LIST(URL,DenySequences)
	WRITE_INI_LIST(URL,AllowURL)

	//PATH
	WRITE_INI_BOOL(Path,DenyParentPath)
	WRITE_INI_BOOL(Path,DenyBackspace)
	WRITE_INI_BOOL(Path,DenyCarriageReturn)
	WRITE_INI_BOOL(Path,DenyNewLine)
	WRITE_INI_BOOL(Path,DenyEscaping)
	WRITE_INI_BOOL(Path,DenyDot)
	WRITE_INI_STRING(Path,DenyCharacters)
	WRITE_INI_LIST(Path,AllowPaths)

	//FILENAME
	WRITE_INI_BOOL(Filename,UseRawScan)
	WRITE_INI_STRING(Filename,DenyCharacters)
	WRITE_INI_BOOL(Filename,DenyDefaultDocument)
	WRITE_INI_LIST(Filename,DenyFiles)
	WRITE_INI_LIST(Filename,MonitorFiles)
		
	//EXTENSION
	WRITE_INI_LIST(Extensions,AllowExtensions)
	WRITE_INI_LIST(Extensions,DenyExtensions)

	//HEADERS
	WRITE_INI_BOOL(Headers,RemoveServerHeader)
	WRITE_INI_BOOL(Headers,ChangeServerHeader)
	WRITE_INI_STRING(Headers,ServerHeader)
	WRITE_INI_BOOL(Headers,RFCCompliantHostHeader)
	WRITE_INI_BOOL(Headers,DenySQLInjection)
	WRITE_INI_BOOL(Headers,DenyEncodingExploits)
	WRITE_INI_BOOL(Headers,DenyDirectoryTraversal)
	WRITE_INI_BOOL(Headers,DenyHighBitShellcode)
	WRITE_INI_LIST(Headers,DenyHeaders)
	WRITE_INI_LIST(Headers,DenySequences)

	//CONTENTTYPE
	WRITE_INI_LIST(ContentType,AllowContentTypes)
	WRITE_INI_LIST(ContentType,DenyContentTypes)

	//COOKIE
	WRITE_INI_BOOL(Cookie,DenySQLInjection)
	WRITE_INI_BOOL(Cookie,DenyEncodingExploits)
	WRITE_INI_BOOL(Cookie,DenyDirectoryTraversal)
	WRITE_INI_BOOL(Cookie,DenyHighBitShellcode)
	WRITE_INI_LIST(Cookie,DenySequences)

	//USER-AGENT
	WRITE_INI_BOOL(UserAgent,DenyEmpty)
	WRITE_INI_BOOL(UserAgent,DenyNonRFC)
	WRITE_INI_STRING(UserAgent,RequireCharacter)
	WRITE_INI_LIST(UserAgent,DenyUserAgents)
	WRITE_INI_LIST(UserAgent,DenySequences)

	//REFERRER
	WRITE_INI_BOOL(Referrer,UseScan)
	WRITE_INI_BOOL(Referrer,RFCCompliantURL)
	WRITE_INI_BOOL(Referrer,RFCCompliantHTTPURL)
	WRITE_INI_BOOL(Referrer,DenyEncodingExploits)
	WRITE_INI_BOOL(Referrer,DenyHotLinking)
	WRITE_INI_CSTRINGLIST(Referrer,HotLinkingFileExtensions)
	WRITE_INI_LIST(Referrer,HotLinkingAllowDomains)
	WRITE_INI_LIST(Referrer,HotLinkingDenyDomains)
	WRITE_INI_BOOL(Referrer,HotLinkingUseHostHeader)
	WRITE_INI_BOOL(Referrer,HotLinkingDenyBlankReferrer)
	WRITE_INI_STRING(Referrer,DenyCharacters)
	WRITE_INI_BOOL(Referrer,DenyHighBitShellcode)
	WRITE_INI_BOOL(Referrer,DenySQLInjection)
	WRITE_INI_LIST(Referrer,DenySequences)

	//VERBS
	WRITE_INI_LIST(Verbs,AllowVerbs)
	WRITE_INI_LIST(Verbs,DenyVerbs)

	//SQL Injection
	//WritePrivateProfileSection("SQL Injection Keywords",ConvertListToString(SQLKeyWords),fn);
	//TODO: writing ';' to ini file cannot be read again
	
	//QUERYSTRING
	WRITE_INI_BOOL(QueryString,UseRawScan)
	WRITE_INI_BOOL(QueryString,DenySQLInjection)
	WRITE_INI_BOOL(QueryString,DenyEncodingExploits)
	WRITE_INI_BOOL(QueryString,DenyDirectoryTraversal)
	WRITE_INI_BOOL(QueryString,DenyHighBitShellcode)
	WRITE_INI_LIST(QueryString,DenySequences)
	
	//Raw data (headers & POST data)
	WRITE_INI_BOOL(Global,IsInstalledAsGlobalFilter)
	WRITE_INI_BOOL(Global,DenyPostSQLInjection)
	WRITE_INI_BOOL(Global,DenyPostEncodingExploits)
	WRITE_INI_BOOL(Global,DenyPostDirectoryTraversal)
	WRITE_INI_BOOL(Global,DenyPostHighBitShellcode)
	WRITE_INI_LIST(Global,DenyPostSequences)
	
		return true;
	}catch(...){
		return false;
	}
}

bool CHTTPFirewallSettings::WriteToFileXML(LPCTSTR fn)
{
	CString xml("");

	//xml += "<!DOCTYPE CISAPIFilter[\r\n";			//start dtd
	//xml += "<!ELEMENT CHTTPFirewallSettings ANY >\r\n";
	//xml += " ]>\r\n";							//end dtd

	xml += "<CHTTPFirewallSettings>\r\n";	//start entity
	xml += "<AQTRONIX_CHTTPFirewallSettings App='Title'/>\r\n";
	xml += ToXML();
	xml += "</CHTTPFirewallSettings>\r\n";		//end entity

	return WriteXMLEntityToFile(fn,xml);
}

CString CHTTPFirewallSettings::GetXMLScanningEngine(CHTTPFirewallSettings& d)
{
	CString xml("");
	//Scanning engine
	xml += "<Scanning_Engine App='Separator'/>\r\n";
	WRITE_XML_BOOL(Engine,AllowLateScanning,"Allow_Late_Scanning","This will run the filter as a low priority filter instead of high priority. Recommended is high priority (is more secure as it precedes other ISAPI filters which might have potential buffer overflows, etc.).\r\nRequires restart of IIS!")
	WRITE_XML_BOOL(Engine,ScanNonSecurePort,"Scan_Non_Secure_Port","Scan unencrypted (HTTP) web traffic (default port 80, but can be anything else).\r\nRequires restart of IIS!")
	WRITE_XML_BOOL(Engine,ScanSecurePort,"Scan_Secure_Port","Scan encrypted (HTTPS) web traffic (default port 443, but can be anything else).\r\nRequires restart of IIS!")
	WRITE_XML_LIST(Engine,ExcludedInstances,"Excluded_Web_Instances","Exclude certain web instances, recommended for web instances which have their own ISAPI filter like Outlook Web Access.","These are the web instances excluded from scanning. Examples are Outlook Web Access web sites (starting at site instance 100)")
	WRITE_XML_LIST(Engine,ExcludedIPAddresses,"Excluded_IP_Addresses","Exclude certain IP addresses or ranges, this allows certain hosts to have unfiltered access to your web services.","These are the IP addresses or ranges excluded from scanning.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLResponseHandling(CHTTPFirewallSettings& d)
{
	CString xml("");
	//Response Handling
	xml += "<Incident_Response_Handling App='Separator'/>\r\n";
	WRITE_XML_BOOL(HTTPResponse,Directly,"Response_Directly","If an attack is detected, send an immediate response to the client with a standard message. The message sent back is the contents of the denied.htm file (in the directory of the firewall)")
	WRITE_XML_BOOL(HTTPResponse,Redirect,"Response_Redirect","If an attack is detected, redirect the client to a custom url (the url specified in 'Response Redirect URL').")
	WRITE_XML_STRING(HTTPResponse,RedirectURL,"Response_Redirect_URL","This is the URL the client is redirected to if 'Response Redirect' is chosen and 'Response Directly' is disabled. This can be an absolute URL (like \"http://www.aqtronix.com\") or a relative one (like \"/denied.htm\").")
	WRITE_XML_BOOL(HTTPResponse,UseStatus,"Use_Response_Status","Whenever an attack is detected, use the value in 'Response Status' as the HTTP response status that will be sent back to the client. This only works if you don't redirect the client to a custom URL.")
	WRITE_XML_STRING(HTTPResponse,Status,"Response_Status","This is the HTTP response status like '31337 No Hacking' or '404 Object Not Found' that is sent back to the client when an attack is detected.")
	WRITE_XML_BOOL(HTTPResponse,DropConnection,"Response_Drop_Connection","Whenever an attack is detected, drop the existing connection (even if keep-alive was requested).")
	WRITE_XML_BOOL(HTTPResponse,MonitorIP,"Response_Monitor_IP","Monitor traffic coming from that IP address for a certain time-out period.")
	WRITE_XML_INT(HTTPResponse,MonitorIPTimeout,"Response_Monitor_IP_Timeout","The time-out (in hours) to monitor traffic coming from that IP address.")
	WRITE_XML_BOOL(HTTPResponse,BlockIP,"Response_Block_IP","Block the IP address for a certain time-out period.")
	WRITE_XML_INT(HTTPResponse,BlockIPTimeout,"Response_Block_IP_Timeout","The time-out (in hours) to block the IP address.")
	xml += CFirewallSettings::GetXMLResponseHandling(d);
	return xml;
}

CString CHTTPFirewallSettings::GetXMLConnectionControl(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Connection control
	xml += "<Connection App='Separator'/>\r\n";
	WRITE_XML_LIST(Connection,MonitorAddresses,"Monitored_IP_Addresses","Monitor the traffic of IP addresses specified in 'Monitored IP Addresses' by logging their requests.","These are the monitored IP addresses and IP ranges. For ranges you can use wildcards ('*' like in '1.*.*.*') and CIDR notation (like '1.0.0.0/24') or hyphen (like '10.0.0.1-10.0.0.5').")
	WRITE_XML_LIST(Connection,DenyAddresses,"Denied_IP_Addresses","Deny access from IP addresses specified in 'Denied IP Addresses' and log their requests.","These are the denied IP addresses and IP ranges. For ranges you can use wildcards ('*' like in '1.*.*.*') and CIDR notation (like '1.0.0.0/24') or hyphen (like '10.0.0.1-10.0.0.5').")
	WRITE_XML_CACHE(Connection,RequestsLimit,"Connection_Requests_Limit","Limit the number of requests an IP address can make.","The number of requests that can be made in a certain amount of time.","The time frame in which the requests are counted (in minutes).")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLAuthentication(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Authentication
	xml += "<Authentication App='Separator'/>\r\n";
	WRITE_XML_BOOL(Authentication,ScanExcludedInstances,"Scan_Authentication_Excluded_Web_Instances","Also scan the excluded web instances in this event. Excluded web instances are not scanned by the firewall except for authentication attempts.")
	WRITE_XML_BOOL(Authentication,DenyBlankPasswords,"Deny_Blank_Passwords","This will block authentication attempts with blank passwords.")
	WRITE_XML_BOOL(Authentication,DenySamePasswordAsUsername,"Deny_Same_Password_As_Username","This will block authentication attempts with passwords equal to the username.")
	WRITE_XML_LIST(Authentication,DenyDefaultPasswords,"Denied_Default_Passwords","This will block authentication attempts with default and most used passwords in 'Denied Default Passwords'.","These are the default passwords that cannot be used to authenticate.")
	WRITE_XML_BOOL(Authentication,DenySystemAccounts,"Deny_System_Accounts","This will block authentication attempts with a system critical account (like IUSR_SERVERNAME, IWAM_SERVERNAME, SYSTEM, NETWORK SERVICE, TsInternetUser...).")
	WRITE_XML_CACHE(Authentication,DenyBruteForceAttack,"Deny_Account_Brute_Force_Attack","This will block brute force attacks and possible account lockout Denial-of-Service. Detecting this is done by counting the authentication attempts within a certain period.","The maximum number an ip address is allowed to authenticate within a certain time frame.","The time frame (in minutes) within the number of authentication attempts are count.")
	WRITE_XML_LIST(Authentication,AllowAccounts,"Allowed_Accounts","Only allow authentication attempts for accounts in the 'Allowed Accounts'.","These are the accounts allowed to authenticate.")
	WRITE_XML_LIST(Authentication,DenyAccounts,"Denied_Accounts","Block authentication attempts for accounts in the 'Denied Accounts'.","These are the accounts explicitly denied to authenticate.")
	WRITE_XML_BOOL(Authentication,ScanAccountAllEvents,"Scan_Account_All_Events","This will scan the used account (logged on user) in all other ISAPI events and possibly block the request if the account is not allowed to authenticate.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLRequestLimits(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Request Limits
	xml += "<Request_Limits App='Separator'/>\r\n";
	WRITE_XML_BOOL(Limits,LimitContentLength,"Limit_Content_Length","Limit the value of the Content-Length header in a request to the value specified in 'Max Content Length'. This way you can limit the number of bytes sent to the server in requests.")
	WRITE_XML_BOOL(Limits,LimitURL,"Limit_URL","Limit the length of the url (more precisely everything in the url before the '?') to the value specified in 'Max URL'. Certain attacks involve long urls. You should not allow urls longer than the longest path your operating system allows.")
	WRITE_XML_BOOL(Limits,LimitQueryString,"Limit_Querystring","Limit the length of the querystring (everything after the '?' in a url) to the value specified in 'Max Querystring'.")
	WRITE_XML_BOOL(Limits,LimitHTTPVersion,"Limit_HTTP_Version","Every request to the web server involves specifying the HTTP version (like 'HTTP/1.1'). You can limit the length of this to the value specified in 'Max HTTP Version'.")
	WRITE_XML_STRING(Limits,MaxContentLength,"Max_Content_Length","The maximum value of the Content-Length header in requests sent to the web server.")
	WRITE_XML_INT(Limits,MaxURL,"Max_URL","The maximum length of the url.")
	WRITE_XML_INT(Limits,MaxQueryString,"Max_Querystring","The maximum length of the querystring.")
	WRITE_XML_INT(Limits,MaxHTTPVersion,"Max_HTTP_Version","The maximum length of the HTTP version string.")
	WRITE_XML_LIST(Limits,AllowHTTPVersions,"Allowed_HTTP_Versions","Only allow certain HTTP versions specified in 'Allowed HTTP Versions'","These are the allowed HTTP versions. An empty line means you allow the HTTP version 0.9 (no http version)")
	WRITE_XML_MAP(Limits,MaxHeaders,"Max_Headers","Limit the length of additional headers. You can specify a custom header and a maximum length for each header in 'Max Headers'.","These are the headers of which the length is limited to a specific value.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLURLScanning(CHTTPFirewallSettings &d)
{
	CString xml("");
	//URL Scanning
	xml += "<URL_Scanning App='Separator'/>\r\n";
	WRITE_XML_BOOL(URL,RFCCompliantURL,"RFC_Compliant_Url","Check if the URL is RFC compliant. If it is not the request will be blocked.")
	WRITE_XML_BOOL(URL,RFCCompliantHTTPURL,"RFC_Compliant_HTTP_Url","Check if the HTTP URL is RFC compliant. This will block authentication in the HTTP url (absolute URLs only).")
	WRITE_XML_BOOL(URL,UseRawScan,"Use_Url_Raw_Scan","Besides using the default scanning, also use the raw scanning capability to scan the URL before the web server decoded the URL (with built-in decoding engine).")
	WRITE_XML_BOOL(URL,DenyEncodingExploits,"Deny_Url_Encoding_Exploits","Do not allow encoding exploits (embedded encoding) in the URL.")
	WRITE_XML_BOOL(URL,DenyParentPath,"Deny_Url_Parent_Path","Deny parent path ('..') attempt in the requested url.")
	WRITE_XML_BOOL(URL,DenyTrailingDotInDir,"Deny_Url_Trailing_Dot_In_Dir","Deny a trailing dot in a directory name. This will block all requests with './'.")
	WRITE_XML_BOOL(URL,DenyBackslash,"Deny_Url_Backslash","Deny backward slashes ('\\') in the url.")
	WRITE_XML_BOOL(URL,DenyAlternateStream,"Deny_Url_Alternate_Stream","This will block all requests with a ':' in the url.")
	WRITE_XML_BOOL(URL,DenyEscaping,"Deny_Url_Escaping","Do not allow '%' in the url after decoding. This will block encoding exploits (embedded encoding) in the url.")
	WRITE_XML_BOOL(URL,DenyMultipleCGI,"Deny_Url_Running_Multiple_CGI","Do not allow using the ampersand ('&') in a url. This can be used to run multiple CGI applications.")
	WRITE_XML_STRING(URL,DenyCharacters,"Deny_Url_Characters","Additional characters to block. If a requested url contains one of these, the request will be blocked.")
	WRITE_XML_BOOL(URL,DenyHighBitShellcode,"Deny_Url_HighBitShellcode","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended on non-US-English web sites. This will also block Unicode/UTF-8 and MBCS in the URL.")
	WRITE_XML_BOOL(URL,DenySequences.Enabled,"Use_Denied_Url_Sequences","Block the request if the url contains one or more sequences specified in 'URL Denied Sequences'.")
	WRITE_XML_CSTRINGLIST(URL,DenySequences.List,"URL_Denied_Sequences","These are the sequences not allowed in a url.")
	WRITE_XML_BOOL(URL,AllowURL.Enabled,"Use_Allowed_Url_Starts","Only allow certain characters a url may start with. These are specified in 'URL Allowed Starts'.")
	WRITE_XML_CSTRINGLIST(URL,AllowURL.List,"URL_Allowed_starts","These are the allowed character sequences a url may start with.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLMappedPath(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Mapped Path
	xml += "<Mapped_Path App='Separator'/>\r\n";
	WRITE_XML_BOOL(Path,DenyParentPath,"Deny_Parent_Path","Deny parent path ('..') attempt in the mapped path.")
	WRITE_XML_BOOL(Path,DenyBackspace,"Deny_Backspace","Deny backspace characters in the mapped path.")
	WRITE_XML_BOOL(Path,DenyCarriageReturn,"Deny_Carriage_Return","Deny carriage return characters in the mapped path.")
	WRITE_XML_BOOL(Path,DenyNewLine,"Deny_New_Line","Deny newline characters in the mapped path.")
	WRITE_XML_BOOL(Path,DenyEscaping,"Deny_Escaping","Deny encoding exploits in the mapped path by blocking '%'.")
	WRITE_XML_BOOL(Path,DenyDot,"Deny_Dot_In_Path","Deny a dot in the path (except for the filename).")
	WRITE_XML_STRING(Path,DenyCharacters,"Deny_Characters","Deny the request if one or more of these characters are present in the mapped path.")
	WRITE_XML_LIST(Path,AllowPaths,"Allowed_Paths","Only allow mapped paths which start with a path specified in 'Allowed Paths'","These are the allowed paths the mapped path may start with.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLRequestedFile(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Requested File
	xml += "<Requested_File App='Separator'/>\r\n";
	WRITE_XML_BOOL(Filename,UseRawScan,"Use_Filename_Raw_Scan","Besides using the default scanning, also use the raw scanning capability to scan the requested file before the web server decoded the URL (with built-in decoding engine).")
	WRITE_XML_STRING(Filename,DenyCharacters,"Deny_Filename_Characters","Deny the request if the filename contains one of these characters.")
	WRITE_XML_BOOL(Filename,DenyDefaultDocument,"Deny_Default_Document","Deny default document requests. The client can only request a specific file, not a directory.")
	WRITE_XML_LIST(Filename,DenyFiles,"Denied_Files","Deny the filenames/CGI applications specified in 'Denied Files' from being accessed or run.","These are the filenames/CGI applications that are not allowed.")
	WRITE_XML_LIST(Filename,MonitorFiles,"Monitored_Files","Monitor access to files specified in 'Monitored Files'.","These are the filenames that will be monitored if 'Use Monitered Files' is used.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLRequestedExtension(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Requested Extension
	WRITE_XML_LIST(Extensions,AllowExtensions,"Allowed_Extensions","Only allow requests for files with extensions specified in 'Allowed Extensions'.","These are the allowed extensions for the requested files.")
	WRITE_XML_LIST(Extensions,DenyExtensions,"Denied_Extensions","Deny requests for files with extensions specified in 'Denied Extensions'.","These are the denied extensions for the requested files.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLHTTPHeaders(CHTTPFirewallSettings &d)
{
	CString xml("");
	//HTTP Headers
	xml += "<Headers App='Separator'/>\r\n";
	WRITE_XML_BOOL(Headers,RemoveServerHeader,"Remove_Server_Header","Remove the 'Server:' header in every response from the web server to the client. The server header is sensitive information that can be used by hackers or worms to probe for vulnerable systems or to know what they're up against.")
	WRITE_XML_BOOL(Headers,ChangeServerHeader,"Change_Server_Header","Instead of removing the server header, you can change it. This way you can fool hackers and worms by specifying another commercial web server in 'Server Header'. Note: 'Remove Server Header' has priority over this setting, so don't enable 'Remove Server Header' if you want to change the server header.")
	WRITE_XML_STRING(Headers,ServerHeader,"Server_Header","The server header sent back to the client in every response. This has to be in the form '<software>/<version>' (like 'Apache/0.8.4'.")
	WRITE_XML_BOOL(Headers,RFCCompliantHostHeader,"RFC_Compliant_Host_Header","Block the HTTP 1.1 request if it does not include a 'Host:' header (RFC compliant).")
	WRITE_XML_LIST(Headers,DenyHeaders,"Denied_Headers","Block the request if any of the headers specified in 'Denied Headers' are present.","These are the headers not allowed in requests to the web server.")

	WRITE_XML_BOOL(Headers,DenySQLInjection,"Deny_Header_SQL_Injection","Do not allow SQL injection in the headers sent to the web server.")
	WRITE_XML_BOOL(Headers,DenyEncodingExploits,"Deny_Header_Encoding_Exploits","Do not allow encoding exploits (embedded encoding) in the headers sent to the web server.")
	WRITE_XML_BOOL(Headers,DenyDirectoryTraversal,"Deny_Header_Directory_Traversal","Do not allow directory traversal (parent path) in the headers sent to the web server. This will block any '..' preceding or following a slash ('/' or '\\').")
	WRITE_XML_BOOL(Headers,DenyHighBitShellcode,"Deny_Header_High_Bit_Shellcode","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended for non-US-English web sites.")
	WRITE_XML_LIST(Headers,DenySequences,"Denied_Header_Sequences","Block the request if any of the character sequences specified in 'Denied Header Sequences' are present in the headers.","These are the character sequences not allowed in the headers.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLContentType(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Content-Type
	xml += "<ContentType App='Separator'/>\r\n";
	WRITE_XML_LIST(ContentType,AllowContentTypes,"Allowed_Content_Types","Enable the checking of the Content-Type header in requests and deny the request if the Content-Type is not in the 'Allowed Content Types' list.","These are the allowed Content-Types of requests. If for instance you want to enable all multipart types simply add 'multipart/'. This way you effectively enable 'multipart/form-data', 'multipart/mixed',... A blank line means no Content-Type is also allowed.")
	WRITE_XML_LIST(ContentType,DenyContentTypes,"Deny_Content_Types","Enable the checking of the Content-Type header in requests and deny the request if the Content-Type is in the 'Denied Content Types' list.","These are the denied Content-Types of requests. Examples are 'application/' (will block all application content-types), 'application/octet-stream', 'application/*' , ...")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLQuerystringScanning(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Querystring Scanning
	xml += "<Querystring App='Separator'/>\r\n";
	WRITE_XML_BOOL(QueryString,UseRawScan,"Use_Querystring_Raw_Scan","Besides using the default scanning, also use the raw scanning capability to scan the querystring before the web server decoded the URL (with built-in decoding engine).")
	WRITE_XML_BOOL(QueryString,DenySQLInjection,"Deny_Querystring_SQL_Injection","Do not allow SQL injection in the querystring.")
	WRITE_XML_BOOL(QueryString,DenyEncodingExploits,"Deny_Querystring_Encoding_Exploits","Do not allow encoding exploits (embedded encoding) in the querystring.")
	WRITE_XML_BOOL(QueryString,DenyDirectoryTraversal,"Deny_Querystring_Directory_Traversal","Do not allow directory traversal in the querystring. This will block any '..' preceding or following a slash ('/' or '\\').")
	WRITE_XML_BOOL(QueryString,DenyHighBitShellcode,"Deny_Querystring_High_Bit_Shellcode","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended for non-US-English web sites.")
	WRITE_XML_LIST(QueryString,DenySequences,"Denied_Querystring_Sequences","Block the request if any of the sequences specified in 'Denied Querystring Sequences' are present.","These are the character sequences not allowed in the querystring.")

	return xml;
}

CString CHTTPFirewallSettings::GetXMLGlobalFilterCapabilities(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Global Filter Capabilities
	xml += "<Global_Filter_Capabilities App='Separator'/>\r\n";
	WRITE_XML_BOOL(Global,IsInstalledAsGlobalFilter,"Is_Installed_As_Global_Filter","Register for the OnReadRawData event. This is required for the global filter capabilities. This event can only be called if the filter is installed as a global filter. If this is not the case then the filter will fail to load. For IIS 6, you need to run in IIS 5.0 Isolation mode (not Worker Process mode). Not supported in IIS7 and later. Requires restart of IIS!")
	WRITE_XML_BOOL(Global,DenyPostSQLInjection,"Deny_Postdata_SQL_Injection","Do not allow SQL injection in the data (e.g. postdata) sent to the web server.")
	WRITE_XML_BOOL(Global,DenyPostEncodingExploits,"Deny_Postdata_Encoding_Exploits","Do not allow encoding exploits (embedded encoding) in the data (e.g. postdata) sent to the web server.")
	WRITE_XML_BOOL(Global,DenyPostDirectoryTraversal,"Deny_Postdata_Directory_Traversal","Do not allow directory traversal (parent path) in the data (e.g. postdata) sent to the web server. This will block any '..' preceding or following a slash ('/' or '\\').")
	WRITE_XML_BOOL(Global,DenyPostHighBitShellcode,"Deny_Postdata_High_Bit_Shellcode","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended for non-US-English web sites.")
	WRITE_XML_LIST(Global,DenyPostSequences,"Denied_Post_Sequences","Block the request if any of the character sequences specified in 'Denied Post Sequences' are present in the data.","These are the character sequences not allowed in data (e.g. postdata).")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLSQLInjection(CHTTPFirewallSettings &d)
{
	CString xml("");
	//SQL Injection Keywords
	xml += "<SQL_Injection App='Separator'/>\r\n";
	xml += ConvertListToXML("SQL_Injection_Keywords",SQLKeyWords,"These are the SQL keywords for the SQL injection scanning. If two or more are found an alert is triggered and the request will be blocked.");
	return xml;
}

CString CHTTPFirewallSettings::GetXMLHTTPVerbs(CHTTPFirewallSettings &d)
{
	CString xml("");
	//HTTP Verbs (Request Methods)
	xml += "<Methods App='Separator'/>\r\n";
	WRITE_XML_LIST(Verbs,AllowVerbs,"Allowed_Verbs","Only allow the request methods (HTTP verbs) specified in 'Allowed Verbs'.","These are request methods (HTTP verbs) that are allowed.")
	WRITE_XML_LIST(Verbs,DenyVerbs,"Denied_Verbs","Block request if the request method (HTTP verb) is one of those specified in 'Denied Verbs'.","These are the denied request methods (HTTP verbs).")
	return xml;
}

void CHTTPFirewallSettings::LoadDefaultsSQLInjection(CStringList& List)
{
	//SQL Injection (words in list should be lowercase, if 2 or more are found, request will be blocked)
	//used resources: next generation security software documents, Snort rules
	List.RemoveAll();
	List.AddTail("'");	//don't allow escaping string
	List.AddTail("`");	//MySQL string escape character
	List.AddTail(";");	//don't allow escaping numeric value
	List.AddTail("--");	//don't allow commenting
	List.AddTail("select");
	List.AddTail("insert");
	List.AddTail("update");
	List.AddTail("delete");
	List.AddTail("drop ");
	List.AddTail("alter ");	//false positive: "aalter"
	List.AddTail("create ");
	List.AddTail("inner join");
	List.AddTail("from ");
	List.AddTail("where ");
	List.AddTail("union ");
	List.AddTail("group by");
	List.AddTail("having ");
	List.AddTail("table ");
	List.AddTail("shutdown");
	List.AddTail("kill ");
	List.AddTail("declare");
	List.AddTail("openrowset");
	List.AddTail("opendatasource");
	List.AddTail("pwdencrypt");
	List.AddTail("msdasql");
	List.AddTail("sqloledb");
	List.AddTail("char(");
	List.AddTail("char(124)");
	List.AddTail("cast(");
	List.AddTail("fetch next");
	List.AddTail("allocate");	//also deallocate
	List.AddTail("syslogins");
	List.AddTail("sysxlogins");
	List.AddTail("sysdatabases");
	List.AddTail("sysobjects");
	List.AddTail("syscomments");
	List.AddTail("raiserror");
	//List.AddTail("execute");  //detected by "exec"
	List.AddTail("exec");
	List.AddTail("=!(");
	List.AddTail("= !(");
	List.AddTail("xp_");
	List.AddTail("sp_");
	//these below will be blocked (because the 2 lines above "xp_" & "sp_" will also be detected)
	List.AddTail("xp_cmdshell");
	List.AddTail("xp_reg");
	//List.AddTail("xp_regread");
	//List.AddTail("xp_regwrite");
	//List.AddTail("xp_regdeletekey");
	//List.AddTail("xp_regdeletevalue");
	//List.AddTail("xp_regaddmultistring");
	//List.AddTail("xp_regenumkeys");
	//List.AddTail("xp_regenumvalues");
	//List.AddTail("xp_regremovemultistring");
	List.AddTail("xp_servicecontrol");
	List.AddTail("xp_setsqlsecurity");
	List.AddTail("xp_readerrorlog");
	List.AddTail("xp_controlqueueservice"); //buffer overflow extended procedures
	List.AddTail("xp_createprivatequeue");
	List.AddTail("xp_decodequeuecommand");
	List.AddTail("xp_deleteprivatequeue");
	List.AddTail("xp_deletequeue");
	List.AddTail("xp_displayqueuemesgs");
	List.AddTail("xp_dsinfo");
	List.AddTail("xp_mergelineages");
	List.AddTail("xp_readpkfromqueue");
	List.AddTail("xp_readpkfromvarbin");
	List.AddTail("xp_repl_encrypt");
	List.AddTail("xp_resetqueue");
	List.AddTail("xp_sqlinventory");
	List.AddTail("xp_unpackcab");
	List.AddTail("xp_sprintf");
	List.AddTail("xp_displayparamstmt");
	List.AddTail("xp_enumresult");
	List.AddTail("xp_showcolv");
	List.AddTail("xp_updatecolvbm");
	List.AddTail("xp_execresultset"); //can be used to bypass access control
	List.AddTail("xp_printstatements");
	List.AddTail("xp_peekqueue");
	List.AddTail("xp_proxiedmetadata");
	List.AddTail("xp_displayparamstmt");
	List.AddTail("xp_availablemedia");
	List.AddTail("xp_enumdsn");
	List.AddTail("xp_filelist");
	List.AddTail("sp_password");
	List.AddTail("sp_adduser");
	List.AddTail("sp_addextendedproc");
	List.AddTail("sp_dropextendedproc");
	List.AddTail("sp_add_job"); //sp_add_jobstep
	List.AddTail("sp_start_job");
	List.AddTail("sp_delete_alert");
	List.AddTail("sp_msrepl_startup");
}

void CHTTPFirewallSettings::LoadDefaultsAuthPasswords(CStringList& List)
{
	//AUTH - Default passwords
	List.RemoveAll();
	List.AddTail("12345");
	List.AddTail("54321");
	List.AddTail("adm");
	List.AddTail("admin");
	List.AddTail("administrator");
	List.AddTail("access");
	List.AddTail("account");
	List.AddTail("all");
	List.AddTail("anyone");
	List.AddTail("backup");
	List.AddTail("control");
	List.AddTail("database");
	List.AddTail("dba");
	List.AddTail("debug");
	List.AddTail("default");
	List.AddTail("develop");
	List.AddTail("developer");
	List.AddTail("development");
	List.AddTail("enter");
	List.AddTail("ftp");
	List.AddTail("ftproot");
	List.AddTail("god");
	List.AddTail("guest");
	List.AddTail("login");
	List.AddTail("master");
	List.AddTail("oracle");
	List.AddTail("oracle8");
	List.AddTail("password");
	List.AddTail("pass");
	List.AddTail("pwrchute");
	List.AddTail("root");
	List.AddTail("sa");
	List.AddTail("secret");
	List.AddTail("sql");
	List.AddTail("sqlserver");
	List.AddTail("tech");
	List.AddTail("test");
	List.AddTail("user");
	List.AddTail("web");
	List.AddTail("webmaster");
	List.AddTail("work");
	List.AddTail("www");
	//TODO: more!
}

void CHTTPFirewallSettings::LoadDefaultsAuthAccounts(CStringList& List)
{
	List.RemoveAll();
	List.AddTail("Guest");
	List.AddTail("SQLDebugger");
	List.AddTail("TsInetnetUser");
	List.AddTail("Test");
	List.AddTail("SQL");
	List.AddTail("SQLServer");
	List.AddTail("Webserver");
	List.AddTail("nobody");
	List.AddTail("postmaster");
	List.AddTail("operator");
	List.AddTail("Mail");
	List.AddTail("Replicator");
	List.AddTail("Replication");
	List.AddTail("Debug");
	List.AddTail("DBA");
	List.AddTail("abc");
	List.AddTail("access");
	List.AddTail("admin");
	//List.AddTail("administrator");
	List.AddTail("anon");
	List.AddTail("anonymous");
	List.AddTail("anyone");
	List.AddTail("backup");
	List.AddTail("batch");
	List.AddTail("config");
	List.AddTail("control");
	List.AddTail("data");
	List.AddTail("database");
	List.AddTail("ftp");
	List.AddTail("ftproot");
	List.AddTail("install");
	List.AddTail("login");
	List.AddTail("master");
	List.AddTail("oracle");
	List.AddTail("oracle8");
	List.AddTail("public");
	List.AddTail("pwrchute");
	List.AddTail("recovery");
	List.AddTail("remote");
	List.AddTail("root");
	List.AddTail("router");
	List.AddTail("sa");
	List.AddTail("security");
	List.AddTail("server");
	List.AddTail("service");
	List.AddTail("setup");
	List.AddTail("sybase");
	List.AddTail("test");
	List.AddTail("user");
	List.AddTail("web");
	List.AddTail("webdb");
	List.AddTail("webmaster");
	List.AddTail("www");
	List.AddTail("wwwadmin");
}

void CHTTPFirewallSettings::LoadDefaultsUnixCommands(CStringList& List)
{
	//Default *nix commands
	//List.AddTail("ps");
	List.AddTail("wget"); //too much collateral damage
	List.AddTail("uname");
	//List.AddTail("id");
	List.AddTail("echo");
	List.AddTail("kill");
	List.AddTail("chmod");
	List.AddTail("chgrp");
	List.AddTail("chsh");
	//List.AddTail("cc");
	//List.AddTail("cpp");
	List.AddTail("gcc");
	List.AddTail("g++");
	List.AddTail("python");	//too much collateral damage
	List.AddTail("tclsh");
	List.AddTail("nasm");
	//List.AddTail("perl");	//too much collateral damage: hyperlink.htm
	List.AddTail("traceroute");
	List.AddTail("nmap");	//too much collateral damage
	List.AddTail("lsof");	
	//List.AddTail("mail");
	List.AddTail("inetd.conf");
	List.AddTail("motd");
	List.AddTail("shadow");
	List.AddTail("httpd.conf"); //*/
}

void CHTTPFirewallSettings::LoadDefaultsHTTPVersions(CStringList& List)
{
	List.RemoveAll();
	List.AddTail("");			//HTTP/0.9
	List.AddTail("HTTP/0.9");	//IIS 6 reports empty version as "HTTP/0.9"
	List.AddTail("HTTP/1.0");
	List.AddTail("HTTP/1.1");
}

CString CHTTPFirewallSettings::GetXMLLogging(CHTTPFirewallSettings &d)
{
	CString xml("");
	xml += "<Logging App='Separator'/>\r\n";
	xml += CFirewallSettings::GetXMLLogging(d);
	WRITE_XML_BOOL(HTTPLogging,Log_HTTP_VIA,"Log_HTTP_VIA","Log the 'Via:' header to have a clue where the original request came from (if the client uses 1 or more proxies). Note: you will not be able to log all used proxies (certain proxies don't have or remove this header)!")
	WRITE_XML_BOOL(HTTPLogging,Log_HTTP_X_FORWARDED_FOR,"Log_HTTP_X_FORWARDED_FOR","Log the 'X_Forwarded_For:' header. Certain proxies (like NetCache) add this header to the request which indicate the source IP address of the request.")
	WRITE_XML_BOOL(HTTPLogging,LogHostHeader,"Log_Host_Header","Log the host header. This will log the host header the request, so you will have a clue to what web site the request was intended.")
	WRITE_XML_BOOL(HTTPLogging,LogUserAgent,"Log_User_Agent","Log the client user agent. This can indicate what software/tool is used to perform the attack. However it is not essential information for reporting an abuse.")
	WRITE_XML_BOOL(HTTPLogging,LogClientErrors,"Log_HTTP_Client_Errors","Log HTTP client side errors like '404 Not Found'. These errors start with a '4'.")
	WRITE_XML_BOOL(HTTPLogging,LogServerErrors,"Log_HTTP_Server_Errors","Log HTTP server side errors like '501 Not Implemented'. These errors start with a '5'.")
	return xml;
}

CString CHTTPFirewallSettings::ToXML()
{
	CString xml;
	CHTTPFirewallSettings d;
	xml += GetXMLScanningEngine(d);
	xml += GetXMLResponseHandling(d);
	xml += GetXMLLogging(d);
	xml += GetXMLConnectionControl(d);
	xml += GetXMLAuthentication(d);
	xml += GetXMLRequestLimits(d);
	xml += GetXMLURLScanning(d);
	xml += GetXMLMappedPath(d);
	xml += GetXMLRequestedFile(d);
	xml += GetXMLRequestedExtension(d);
	xml += GetXMLHTTPHeaders(d);
	xml += GetXMLContentType(d);
	xml += GetXMLCookie(d);
	xml += GetXMLUserAgent(d);
	xml += GetXMLReferrer(d);
	xml += GetXMLHTTPVerbs(d);
	xml += GetXMLQuerystringScanning(d);
	xml += GetXMLGlobalFilterCapabilities(d);
	xml += GetXMLSQLInjection(d);
	return xml;
}

bool CHTTPFirewallSettings::ReadFromFile(LPCTSTR fn)
{
	bool ret(false);

	//read settings file
	ret = CSettings::ReadFromFile(fn);
	//load agents settings
	LoadAgents();

	return ret;
}

void CHTTPFirewallSettings::ReadAgentsFile(LPCTSTR Path)
{
	//load database
	if(Agents.AgentList.ReadFromFile(Path)){
		LoadAgents();
	}else{
		//if database does not exist yet, make one with defaults
		/*
		DWORD attr;
		attr = GetFileAttributes(Path);
		if (0xFFFFFFFF == attr){
			Agents.AgentList.WriteToFile(Path);
		}//*/
	}
	//Agents.AgentList.WriteToFile("C:\\Agents.xml");
}

CString CHTTPFirewallSettings::GetXMLCookie(CHTTPFirewallSettings &d)
{
	//Cookie
	CString xml("");
	xml += "<Cookie App='Separator'/>\r\n";
	WRITE_XML_BOOL(Cookie,DenySQLInjection,"Deny_Cookie_SQL_Injection","Deny SQL injection in the 'Cookie:' header. This can be useful if your website is using a database and you are using cookies for storing information related to the database.")
	WRITE_XML_BOOL(Cookie,DenyEncodingExploits,"Deny_Cookie_Encoding_Exploits","Do not allow encoding exploits (embedded encoding) in cookies (in the 'Cookie:' header).")
	WRITE_XML_BOOL(Cookie,DenyDirectoryTraversal,"Deny_Cookie_Directory_Traversal","Do not allow directory traversal (parent path) in the cookie sent to the web server. This will block any '..' preceding or following a slash ('/' or '\\').")
	WRITE_XML_BOOL(Cookie,DenyHighBitShellcode,"Deny_Cookie_High_Bit_Shellcode","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended for non-US-English web sites.")
	WRITE_XML_LIST(Cookie,DenySequences,"Denied_Cookie_Sequences","Block the request if any of the character sequences specified in 'Denied Cookie Sequences' are present in the cookie.","These are the character sequences not allowed in the cookie.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLUserAgent(CHTTPFirewallSettings &d)
{
	//User-Agent
	CString xml("");
	xml += "<User_Agent App='Separator'/>\r\n";
	WRITE_XML_BOOL(UserAgent,DenyEmpty,"Deny_User_Agent_Empty","Deny the request if the user agent is empty or not present.");
	WRITE_XML_BOOL(UserAgent,DenyNonRFC,"Deny_User_Agent_Non_RFC","Deny the request if the user agent is not RFC compliant.");
	WRITE_XML_STRING(UserAgent,RequireCharacter,"Require_User_Agent_Character","Deny the request if the user agent does not contain at least one of these characters.");
	WRITE_XML_LIST(UserAgent,DenyUserAgents,"Denied_User_Agents","Deny the request if the User Agent is in the 'Denied User Agents' list.","These are the denied User Agents of requests.")
	WRITE_XML_LIST(UserAgent,DenySequences,"Denied_User_Agent_Sequences","Deny the request if the User Agent contains a character sequence specified in 'Denied User Agent Sequences'.","These are the denied User Agents sequences.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLReferrer(CHTTPFirewallSettings &d)
{
	//Referrer
	CString xml("");
	xml += "<Referrer App='Separator'/>\r\n";
	WRITE_XML_BOOL(Referrer,UseScan,"Use_Referrer_Scanning","Scan the referrer URL. Enabling this allows the other checks in this section.")
	WRITE_XML_BOOL(Referrer,RFCCompliantURL,"Referrer_URL_RFC_Compliant","The referrer URL has to be RFC compliant.")
	WRITE_XML_BOOL(Referrer,RFCCompliantHTTPURL,"Referrer_URL_RFC_HTTPCompliant","The referrer URL has to be HTTP RFC compliant (no authentication).")
	WRITE_XML_BOOL(Referrer,DenyEncodingExploits,"Deny_Referrer_Encoding_Exploits","Deny encoding exploits and embedded encoding in the referrer URL.")
	WRITE_XML_BOOL(Referrer,DenyHotLinking,"Deny_Referrer_Hot_Linking","Deny hot linking (also called direct linking, inline linking) to certain files from certain domains.")
	WRITE_XML_CSTRINGLIST(Referrer,HotLinkingFileExtensions,"Referrer_Hot_Linking_File_Extensions","The file extensions hot linking to is denied.")
	WRITE_XML_LIST(Referrer,HotLinkingAllowDomains,"Referrer_Hot_Linking_Allow_Domains","Only allow certain domains to use hot linking.","The domains (FQDN) or IP addresses that are allowed to use hot linking. You do not need to add your own domain to this list, see setting: \"Use Host Header\".")
	WRITE_XML_LIST(Referrer,HotLinkingDenyDomains,"Referrer_Hot_Linking_Deny_Domains","Deny certain domains to use hot linking.","The domains (FQDN) or IP addresses that are denied to use hot linking.")
	WRITE_XML_BOOL(Referrer,HotLinkingUseHostHeader,"Referrer_Hot_Linking_Use_Host_Header","Allow the Host header domain to use hot linking. This is allowing the local web site to refer to itself without needing to add the domain names to the allowed list above.")
	WRITE_XML_BOOL(Referrer,HotLinkingDenyBlankReferrer,"Referrer_Hot_Linking_Deny_Blank_Referrer","Deny requests with no referrer to the protected file extensions. This will block some leeching tools but also some proxy servers and browsers with additional security applications that remove the referrer header.")
	WRITE_XML_STRING(Referrer,DenyCharacters,"Deny_Referrer_Characters","Deny certain characters in the referrer URL.")
	WRITE_XML_BOOL(Referrer,DenyHighBitShellcode,"Deny_Referrer_High_Bit_Shellcode","Deny high bit shell code in the referrer URL. This will block ASCII>127 in the referrer URL and possibly blocking non US-ASCII web sites from linking to your site.")
	WRITE_XML_BOOL(Referrer,DenySQLInjection,"Deny_Referrer_SQL_Injection","Do not allow SQL injection in the referrer URL sent to the web server.")
	WRITE_XML_LIST(Referrer,DenySequences,"Deny_Referrer_Sequences","Deny certain character sequences in the referrer URL.","The list of denied character sequences in the referrer URL.")
	return xml;
}

bool CHTTPFirewallSettings::Update()
{
	if(Agents.AgentList.Update()){
		//reload main settings + agents
		if(UpdateStamp.FileName!=""){
			ReadFromFile(UpdateStamp.FileName);
			return true;
		}else{
			return false;//TODO: no filename like when defaults loaded
		}
	}else{
		return CFirewallSettings::Update(); //reload agents is actually done in ReadFromFile()
	}
}