/*
    AQTRONIX C++ Library
    Copyright 2003-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// IPAddress.cpp: implementation of the CIPAddress class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "IPAddress.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define IP4_QUADS 4
#define IP6_QUADS 16

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/* Changelog
 * 2006.03.20 Added ToHexString() and ToStringTriplets
 * 2006.02.18 Added IsIPAddress (IPv4+IPv6 ready)
 * 2006.02.05 Added Reverse() for DNS blacklist lookups
 * 2005.10.25 Conversion int -> unsigned char for performance
 *			  moved Safe_atoi to CStringHelper class and use new
 *            function Safe_AsciiToUChar
 * 2005.10.02 Added SetIP(LPCTSTR) and CIPAddress(LPCTSTR)
 *            Moved ConvertCIDRToSubnet() from class CIPRange to here
 * 2005.07.06 Added SetAllValues() and operator = and CIPAddress(CIPAddress&)
 * 2004.01.16 First version of class finished
 * 2003.10.15 Created class for MailKnight
 */

/* TODO
 * - IPv6 ready
 */

CIPAddress::CIPAddress()
{
	SetAllValues(0);	//defaults
}

CIPAddress::CIPAddress(CString IP)
{
	SetAllValues(0);	//defaults
	//Set IP address
	if(IP!="")
		SetIP(IP);
}

CIPAddress::CIPAddress(LPCTSTR IP)
{
	SetAllValues(0);	//defaults
	if(IP!=NULL)
		SetIP(IP);
}

CIPAddress::CIPAddress(const CIPAddress& IP)
{
	SetIP(IP);
}

CIPAddress::~CIPAddress()
{

}

bool CIPAddress::SetIP(LPCTSTR IP)
{
	SetAllValues(0);
	CString byte("");
	unsigned char quadcounter = 0;
	for(UINT i=0;i<strlen(IP);i++){
		if(IP[i] == '.' && quadcounter<CIPADDRESS_MAX_BYTES){
			m_IP[quadcounter] = CStringHelper::Safe_AsciiToUChar(byte);
			byte = "";
			quadcounter++;
		}else{
			byte += IP[i];
		}
	}
	m_IP[quadcounter] = CStringHelper::Safe_AsciiToUChar(byte);
	return true;
}

bool CIPAddress::SetIP(CString &IP)
{
	SetAllValues(0);
	CString byte("");
	unsigned int quadcounter = 0;
	for(int i=0;i<IP.GetLength();i++){
		if(IP[i] == '.' && quadcounter<CIPADDRESS_MAX_BYTES){
			m_IP[quadcounter] = CStringHelper::Safe_AsciiToUChar(byte);
			byte = "";
			quadcounter++;
		}else{
			byte += IP[i];
		}
	}
	m_IP[quadcounter] = CStringHelper::Safe_AsciiToUChar(byte);
	return true;
}

bool CIPAddress::SetIP(const CIPAddress& IP)
{
	for(unsigned char i=0;i<CIPADDRESS_MAX_BYTES;i++){
		m_IP[i] = IP.m_IP[i];
	}
	return true;
}

CIPAddress& CIPAddress::operator = (const CIPAddress& IP)
{
	SetIP(IP);
	return *this;
}

bool CIPAddress::operator > (const CIPAddress& IP) const
{
	for(unsigned char i=0;i<CIPADDRESS_MAX_BYTES;i++){
		if(m_IP[i]>IP.m_IP[i]){
			return true;
		}else{
			if(m_IP[i]<IP.m_IP[i])
				return false;
		}
	}
	return false;	//the're equal!
}

bool CIPAddress::operator < (const CIPAddress& IP) const
{
	for(unsigned int i=0;i<CIPADDRESS_MAX_BYTES;i++){
		if(m_IP[i]<IP.m_IP[i]){
			return true;
		}else{
			if(m_IP[i]>IP.m_IP[i])
				return false;
		}
	}
	return false;	//the're equal
}

bool CIPAddress::operator == (const CIPAddress& IP) const
{
	for(unsigned int i=0;i<CIPADDRESS_MAX_BYTES;i++){
		if(m_IP[i]!=IP.m_IP[i]){
			return false;
		}
	}
	return true;
}

bool CIPAddress::operator != (const CIPAddress& IP) const
{
	return !(this->operator==(IP));
}

CString CIPAddress::ToString() const
{
	CString IP("");
	char buf[34] = "\0";
	for(unsigned int i=0;i<CIPADDRESS_MAX_BYTES;i++){
		IP += itoa(m_IP[i],buf,10);
		if(i<CIPADDRESS_MAX_BYTES-1)
			IP+=".";
	}
	return IP;
}

CString CIPAddress::ToStringTriplets() const
{
	CString IP("");
	CString item;
	char buf[34] = "\0";
	for(unsigned int i=0;i<CIPADDRESS_MAX_BYTES;i++){
		item = itoa(m_IP[i],buf,10);
		switch(item.GetLength()){
		case 1:
			IP += "00";
			break;
		case 2:
			IP += "0";
			break;
		}
		IP+=item;
	}
	return IP;
}

CString CIPAddress::ToHexString() const
{
	CString IP("");
	char buf[34] = "\0";
	for(unsigned int i=0;i<CIPADDRESS_MAX_BYTES;i++){
		if(m_IP[i]<16){
			IP += "0";
		}
		IP += itoa(m_IP[i],buf,16);
	}
	IP.MakeUpper();
	return IP;
}

void CIPAddress::SetMax()
{
	//set max;
	SetAllValues(255);
}

void CIPAddress::SetMin()
{
	//set min
	SetAllValues(0);
}

inline void CIPAddress::SetAllValues(unsigned char Value)
{
	//Set all values
	for(unsigned char ic=0;ic<CIPADDRESS_MAX_BYTES;ic++){
		m_IP[ic] = Value;
	}
}

CIPAddress CIPAddress::ConvertCIDRToSubnet(unsigned char Subnet)
{
	CIPAddress SubnetMask;
	if(Subnet>CIPADDRESS_MAX_BYTES*8){
		SubnetMask.SetMax();
	}else{
		unsigned char i=0;
		while(Subnet>7){
			SubnetMask.m_IP[i] = 255;
			i++;
			Subnet -=8;
		}
		unsigned char chunk = 128;
		SubnetMask.m_IP[i] = 0;
		while(Subnet>0){
			SubnetMask.m_IP[i] += chunk;
			chunk /=2;
			Subnet--;
		}
	}
	return SubnetMask;
}

CIPAddress CIPAddress::Reverse()
{
	//reverse ip
	CIPAddress rev;
	for(unsigned char ic=0;ic<CIPADDRESS_MAX_BYTES;ic++){
		rev.m_IP[CIPADDRESS_MAX_BYTES-1-ic] = m_IP[ic];
	}
	return rev;
}

bool CIPAddress::IsIPAddress(CString &IP)
{
	//IPv4 + IPv6 ready
	CString byte("");
	unsigned int quadcounter = 0;
	for(int i=0;i<IP.GetLength();i++){
		if(IP[i] == '.'){
			if(!IsIPAddressQuad(byte)){
				return false;
			}
			byte = "";
			quadcounter++;
			if(quadcounter>IP6_QUADS){
				return false;
			}
		}else{
			byte += IP[i];
		}
	}
	if(IsIPAddressQuad(byte)){
		quadcounter++;
		if(quadcounter==IP4_QUADS || quadcounter==IP6_QUADS){
			return true;
		}else{
			return false;
		}
	}else{
		return false;
	}
}

bool CIPAddress::IsIPAddressQuad(CString &byte)
{
	if(byte.GetLength()>0){
		int ipseq = CStringHelper::Safe_AsciiToInt(byte,-1);
		if(ipseq<256 && ipseq > -1){
			return true;
		}
	}
	return false;
}