/*
    AQTRONIX C++ Library
    Copyright 2005-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// ISAPIFilter.cpp : implementation file
//

#include "stdafx.h"
#include "ISAPIFilter.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CISAPIFilter

CISAPIFilter::CISAPIFilter()
{
}

CISAPIFilter::~CISAPIFilter()
{
}


// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CISAPIFilter, CHttpFilter)
	//{{AFX_MSG_MAP(CISAPIFilter)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

/////////////////////////////////////////////////////////////////////////////
// CISAPIFilter member functions


CString CISAPIFilter::GetServerVariable(CHttpFilterContext* pCtxt,char* sv)
{
	/*
	 * This function will retrieve a server variable.
	 * Function starts with a buffer of MIN_HEADER_SIZE if this buffer is too
	 * small it will try again with the size the web server gave us, or if no
	 * size in indicated, try with a buffer of MAX_HEADER_SIZE
	 */
	DWORD bufsize = MIN_HEADER_SIZE;
	char buf[MIN_HEADER_SIZE]="";
    
	//get header value with MIN_HEADER_SIZE (average minimum header size)
	if(pCtxt->GetServerVariable(sv,buf,&bufsize)){
		for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
			if(buf[i]=='\0') buf[i]=' ';
		}
		buf[bufsize-1]='\0';			//it should be null terminated already, but make sure it is. You never know!
		return CString(buf);
	}else{
		switch(GetLastError()){
			case ERROR_INSUFFICIENT_BUFFER:{	//if the buffer was too small, try again with value set by web server in bufsize
				CString tmp;				
				char* buf2 = new char[bufsize];
				if(buf2==NULL){
					tmp = "Error retrieving server variable";
				}else{
					if(pCtxt->GetServerVariable(sv,buf2,&bufsize)){
						for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
							if(buf2[i]=='\0') buf2[i]=' ';
						}
						buf2[bufsize-1]='\0';	//it should be null terminated already, but make sure it is. You never know!
						tmp=buf2;
					}else{
						tmp="Error retrieving server variable"; //some error occured
					}
					delete[] buf2;
				}
				return tmp;
				break;
			}
			case ERROR_MORE_DATA: //buffer was too small, but no size set in bufsize, try again with MAX_HEADER_SIZE
				bufsize = MAX_HEADER_SIZE;
				char buf2[MAX_HEADER_SIZE];
				if(pCtxt->GetServerVariable(sv,buf2,&bufsize)){
					for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
						if(buf2[i]=='\0') buf2[i]=' ';
					}
					buf2[bufsize-1]='\0';	//it should be null terminated already, but make sure it is. You never know!
					return CString(buf2);
				}else{
					return "Error retrieving server variable"; //some error occured
				}
				break;
			default:
				return ""; //variable is not present or some other error occured
				break;
		}
	}
}

CString CISAPIFilter::GetHeader(CHttpFilterContext* pfc, PHTTP_FILTER_PREPROC_HEADERS pHeaders, char* h)
{
	/*
	 * This function will retrieve a header value.
	 * Function starts with a buffer of MIN_HEADER_SIZE if this buffer is too
	 * small it will try again with the size the web server gave us, or if no
	 * size in indicated, try with a buffer of MAX_HEADER_SIZE
	 */
	DWORD bufsize = MIN_HEADER_SIZE;
	char buf[MIN_HEADER_SIZE]="";
    
	//get header value with MIN_HEADER_SIZE (average minimum header size)
	if(pHeaders->GetHeader(pfc->m_pFC,h,buf,&bufsize)){
		for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
			if(buf[i]=='\0') buf[i]=' ';
		}
		buf[bufsize-1]='\0';			//it should be null terminated already, but make sure it is. You never know!
		return CString(buf);
	}else{
		switch(GetLastError()){
			case ERROR_INSUFFICIENT_BUFFER:{	//if the buffer was too small, try again with value set by web server in bufsize
				CString tmp;				
				char* buf2 = new char[bufsize];
				if(buf2==NULL){
					tmp = "Error retrieving header";
				}else{
					if(pHeaders->GetHeader(pfc->m_pFC,h,buf2,&bufsize)){
						for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
							if(buf2[i]=='\0') buf2[i]=' ';
						}
						buf2[bufsize-1]='\0';	//it should be null terminated already, but make sure it is. You never know!
						tmp=buf2;
					}else{
						tmp="Error retrieving header"; //some error occured
					}
					delete[] buf2;
				}
				return tmp;
				break;
			}
			case ERROR_MORE_DATA: //buffer was too small, but no size set in bufsize, try again with MAX_HEADER_SIZE
				bufsize = MAX_HEADER_SIZE;
				char buf2[MAX_HEADER_SIZE];
				if(pHeaders->GetHeader(pfc->m_pFC,h,buf2,&bufsize)){
					for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
						if(buf2[i]=='\0') buf2[i]=' ';
					}
					buf2[bufsize-1]='\0';	//it should be null terminated already, but make sure it is. You never know!
					return CString(buf2);
				}else{
					return "Error retrieving header"; //some error occured
				}
				break;
			default:
				return ""; //header is not present or some other error occured
				break;
		}
	}
}

bool CISAPIFilter::ChangeRawDataHeader(CString& RawData, LPCTSTR Header, LPCTSTR Value, bool RemoveHeader)
{
	int maxheader; int poscr;
	maxheader = RawData.Find(CRLFCRLF);				//maximum size header
	int length = RawData.GetLength();

	if(maxheader!=-1 && maxheader<MAX_HEADER_SIZE){	//only if end of headers are found
		CString headers = RawData.Left(maxheader);	//only look at the headers
		int pos = headers.Find(Header);				//position header

		if(pos!=-1){								//double CRLF is in header?
			CString preh = RawData.Left(pos);		//copy everything before header
			RawData = RawData.Mid(pos,length-pos);
			poscr = RawData.Find(CRLF);				//end of header

			if(poscr!=-1){							//if end is found
				CString posth(RawData.Mid(poscr+2,length-poscr-2));

				//copy everything after header
				RawData = preh;

				//change header or remove it (remove has priority)
				if (!RemoveHeader){			//it really has to be '!' :)
					RawData += Header;
					RawData += " ";
					RawData += Value;
					RawData += CRLF;
				}
				//append rest of raw data after removed/altered header
				RawData += posth;

				return true;
			}else{
				RawData = "WARNING: Error adjusting '";
				RawData += Header;
				RawData += "' header (could not find end of header)";
				return false;
			}
		}else{
			RawData = "WARNING: Error adjusting '";
			RawData += Header;
			RawData += "' header (header is outside of MAX_HEADER_SIZE or no header found)";
			return false;
		}
	}else{
		RawData = "WARNING: Error adjusting '";
		RawData += Header;
		RawData += "' header (could not determine end of headers)";
		return false;
	}
}

void CISAPIFilter::PrepareRawData(CHttpFilterContext* pCtxt,CString& RawData, CString& Headers, CString& Data)
{
	if(pCtxt->m_pFC->pFilterContext == CONTEXT_NEWREQUEST){	//new request / not all headers received?

		int maxheader = RawData.Find(CRLFCRLF);	//maximum size header
	
		if(maxheader!=-1){						//if end of headers are found (if not, raw data is not complete!)
			Headers = RawData.Left(maxheader);			//only headers
			//ignore the request line (!= header)
			int starth;
			starth = Headers.Find(CRLF);
			if (starth==-1){
				Headers = "";						//no headers (only request line)!
			}else{
				Headers = Headers.Right(Headers.GetLength()-starth-2);
			}
			Data = RawData.Right(RawData.GetLength()-maxheader-4);//only look at the body
			//mark request (all headers received)
			pCtxt->m_pFC->pFilterContext = CONTEXT_HEADERSRECEIVED;
		}else{
			//logger.Append("WARNING: Could not determine end of headers, will process all data as headers");
			Headers = RawData;
			//if double crlf was between events, process this as data as well
			if(RawData.Left(1)==CR || RawData.Left(1)==LF){
				//logger.Append("WARNING: Raw data starts with <CR> or <LF>, so this is probably data part and not headers. Will process it as data as well!");
				Data = RawData;
			}
		}

	}else{	//is additional data
		Data = RawData;
	}	
}
