/*
    AQTRONIX C++ Library
    Copyright 2005-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// ISAPIFirewall.cpp: implementation of the CISAPIFirewall class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ISAPIFirewall.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//Errorhandling
#define ERROR_HANDLE				//use errorhandling in events (recommended)
#define DENY_ON_ERROR true			//protect the webserver at ALL costs, even when filter crashes! (recommended)

#define DO_READRAWDATA_EVENT		//comment/remove this line to disable OnReadRawData event (for custom build request, see e-mail of Paulo Molina)

//Debug logging
//#define LOG_DEBUG_ONREADRAWDATA	//log raw read data
//#define LOG_DEBUG_ONSENDRAWDATA	//log raw sent data (not recommended: performance impact!)

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CISAPIFirewall::CISAPIFirewall(CHTTPFirewallSettings& _Settings):CHTTPFirewall(_Settings)
{

}

CISAPIFirewall::~CISAPIFirewall()
{

}

DWORD CISAPIFirewall::OnReadRawData(CHttpFilterContext* pCtxt, PHTTP_FILTER_RAW_DATA pRawData, LPCTSTR ServerHeader)
{

#ifdef ERROR_HANDLE
	try{
#endif

	/*
	 * This event is only available if the filter is installed as a global
	 * filter (not on individual web sites) and will only be called if
	 * IsInstalledAsGlobalFilter=1
	 * WARNING: if the value is set to 1 and the filter is not installed as
	 * a global filter then IIS will not load the filter (red arrow down)!
	 */

	if(pCtxt!=NULL && pRawData!=NULL){
		//reset keep-alive context
		if(pCtxt->m_pFC->pFilterContext==CONTEXT_SENDINGDATA || pCtxt->m_pFC->pFilterContext==CONTEXT_SENDINGDATALOG)
			pCtxt->m_pFC->pFilterContext=CONTEXT_NEWREQUEST;

		CSingleLock lock(&crit, TRUE);

#ifdef LOG_DEBUG_ONREADRAWDATA
		OnReadRawDataDebug(pCtxt,pRawData);
#endif

		logger.NewEntry("GLOBAL");
		logger.Append("OnReadRawData");
		LogClientInfo(pCtxt);

		bool logonly(Settings.Response.LogOnly);
		bool respdrop(Settings.HTTPResponse.DropConnection);
		bool hack(false);

		CString IP(GetServerVariable(pCtxt,"REMOTE_ADDR"));
		if(IsExcludedIP(IP)){
			logger.Append("IP address is excluded from scanning");
		}else{

			//prepare the raw data before processing
			CString Raw = ConvertBinaryData((BYTE*)pRawData->pvInData,pRawData->cbInData,' ');

			CString strh;				//for headers
			CString strd;				//for post data

			PrepareRawData(pCtxt,Raw,strh,strd);

	#ifdef LOG_DEBUG_ONREADRAWDATA
			logger.Append("\r\nRaw data:");		logger.Append(Raw);
			//logger.Append("\r\nHeaders:");	logger.Append(strh);
			//logger.Append("\r\nData:");		logger.Append(strd);
	#endif

			//HEADERS
			hack |= ScanRawHeaders(strh);
			//DATA
			hack |= ScanRawData(strd);

			//CONNECTION - Monitoring
			if(Settings.Connection.MonitorAddresses.Enabled){
				if(MonitorIPRangeList.IsInList(IP)){
					logger.Append("MONITORED: IP address '" + IP + "'");
					CString Rawl = Raw;
					Rawl.Replace(CR,' ');
					Rawl.Replace(LF,' ');
					logger.Append(Rawl);
					logger.Flush();
				}
			}
		}

		//no more code here logger.Flush() may already be called

		//if possible hack
		if (hack){
			logger.Flush();								//flush the log entry
			HackResponseBlockOrMonitor(IP);				//block or monitor IP address
			if(!logonly)								//if we have to do more than logging
				HackResponse(pCtxt,ServerHeader);		//show page (BUGS: server header not removed & possible loop)
			lock.Unlock();								//unlock globals
			if(logonly){								//if we are only here to log
				return SF_STATUS_REQ_NEXT_NOTIFICATION;
			}else{										//if we have to do more than logging
				if(respdrop){
					return SF_STATUS_REQ_FINISHED;		//make sure IIS does nothing with request
				}else{
					return SF_STATUS_REQ_FINISHED_KEEP_CONN;//make sure IIS does nothing with request
				}
			}
		}else{
			if(Settings.Logging.LogAllowed){			//do not log allowed requests if asked
				logger.Append("ALLOWED");
				logger.Flush();
			}else{
				logger.Cancel();						//clear the log entry
			}
			lock.Unlock();								//unlock globals
			return SF_STATUS_REQ_NEXT_NOTIFICATION;
		}
	}
	return CHttpFilter::OnReadRawData(pCtxt,pRawData);

#ifdef ERROR_HANDLE
	}catch(...){
		if(DENY_ON_ERROR){							//deny request on error if asked
			return SF_STATUS_REQ_FINISHED;			//don't keep connection, because of error
		}else{
			return SF_STATUS_REQ_NEXT_NOTIFICATION;
		}
	}
#endif
}

DWORD CISAPIFirewall::OnPreprocHeaders(CHttpFilterContext* pfc, PHTTP_FILTER_PREPROC_HEADERS pHeaders, LPCTSTR SiteInstance, LPCTSTR ServerHeader)
{
#ifdef ERROR_HANDLE
	try{
#endif

	if(pfc!=NULL && pHeaders!=NULL){
		CString instance = GetServerVariable(pfc,"INSTANCE_ID");

		//mark all headers received
		pfc->m_pFC->pFilterContext = CONTEXT_HEADERSRECEIVED;

		CSingleLock lock(&crit, TRUE);
		
		UpdateSettings();
		
		logger.NewEntry(SiteInstance + instance);
		logger.Append("OnPreprocHeaders");
		LogClientInfo(pfc);
		
		bool hack(false);
		bool logonly(Settings.Response.LogOnly);
		bool respdrop(Settings.HTTPResponse.DropConnection);

		CString IP(GetServerVariable(pfc,"REMOTE_ADDR"));
		if(IsExcluded(instance)){
			logger.Append("Web site instance is excluded from scanning");
		}else{
			if(IsExcludedIP(IP)){
				logger.Append("IP address is excluded from scanning");
			}else{
				CString HostHeader(GetHeader(pfc,pHeaders,"Host:"));

				//VERB
				hack |= ScanVerb(GetHeader(pfc,pHeaders,"method"));

				//RAWURL
				CURL RawUrl(GetHeader(pfc,pHeaders,"url"));
				hack |= ScanRawURL(RawUrl);

				//VERSION
				hack |= ScanVersion(GetHeader(pfc,pHeaders,"version"),HostHeader);

				//HEADERS: raw headers
				hack |= ScanRawHeaders(GetServerVariable(pfc,"ALL_RAW"));

				//HEADERS: denied list / length
				hack |= ScanHeaders(pfc,pHeaders);

				//HEADERS: content length / type
				hack |= ScanContent(GetHeader(pfc,pHeaders,"Content-Type:"),GetHeader(pfc,pHeaders,"Content-Length:"));

				//HEADERS: denied content-types?
				//TODO

				//Custom header scanning (virtual function)
				hack |= CustomHeaderScanning(pfc,pHeaders,IP,RawUrl);

				//COOKIE
				hack |= ScanCookie(GetHeader(pfc,pHeaders,"Cookie:"));

				//REFERRER
				hack |= ScanRawRefererURL(GetHeader(pfc,pHeaders,"Referer:"),CURL::URLEncodedToASCII(RawUrl.Url),HostHeader);

				//AUTH
				if(Settings.Authentication.ScanAccountAllEvents){
					hack |= ScanAccount(GetServerVariable(pfc,"LOGON_USER"));
				}

				//CONNECTION
				hack |= ScanConnection(pfc,IP);

				//no further code here (reason: logger.Flush line in IP Monitoring)
			}
		}

		//if possible hack
		if(hack){
			logger.Flush();								//flush the log entry
			HackResponseBlockOrMonitor(IP);				//block or monitor IP address
			if(!logonly)								//if we have to do more than logging
				HackResponse(pfc,ServerHeader);						//show page
			lock.Unlock();								//unlock globals
			if(logonly){								//if we are only here to log
				return CHttpFilter::OnPreprocHeaders(pfc, pHeaders);
			}else{										//if we have to do more than logging
				if(respdrop){
					return SF_STATUS_REQ_FINISHED;		//make sure IIS does nothing with request
				}else{
					return SF_STATUS_REQ_FINISHED_KEEP_CONN;//make sure IIS does nothing with request
				}
			}
		}else{
			if(Settings.Logging.LogAllowed){			//do not log allowed requests if asked
				logger.Append("ALLOWED");
				logger.Flush();						
			}else{
				logger.Cancel();						//clear the log entry
			}
			lock.Unlock();								//unlock globals
			return CHttpFilter::OnPreprocHeaders(pfc, pHeaders);
		}
	}
	return CHttpFilter::OnPreprocHeaders(pfc, pHeaders);

#ifdef ERROR_HANDLE
	}catch(...){
		if(DENY_ON_ERROR){							//deny request on error if asked
			return SF_STATUS_REQ_FINISHED;			//don't keep connection, because of error
		}else{
			return CHttpFilter::OnPreprocHeaders(pfc, pHeaders);
		}
	}
#endif
}

DWORD CISAPIFirewall::OnAuthentication(CHttpFilterContext* pfc, PHTTP_FILTER_AUTHENT pAuthent, LPCTSTR SiteInstance, LPCTSTR ServerHeader)
{
	/*
		CHAR*    pszUser;                           //IN/OUT
		DWORD    cbUserBuff;                        //IN
		CHAR*    pszPassword;                       //IN/OUT
		DWORD    cbPasswordBuff;                    //IN
	*/

#ifdef ERROR_HANDLE
	try{
#endif

	if(pfc!=NULL && pAuthent!=NULL){
		CString instance = GetServerVariable(pfc,"INSTANCE_ID");

		CSingleLock lock(&crit, TRUE);

		logger.NewEntry(SiteInstance + instance);
		logger.Append("OnAuthentication");
		LogClientInfo(pfc);

		bool hack(false);
		bool logonly(Settings.Response.LogOnly);
			
		CString IP(GetServerVariable(pfc,"REMOTE_ADDR"));
		if(!Settings.Authentication.ScanExcludedInstances && IsExcluded(instance)){
			logger.Append("Web site instance is excluded from scanning");
		}else{
			if(IsExcludedIP(IP)){
				logger.Append("IP address is excluded from scanning");
			}else{
		
				if(pAuthent->cbUserBuff==0 || pAuthent->cbPasswordBuff==0){
					//logger.Append("WARNING: There is a mistake in the MSDN!");
				}else{

					CString User(pAuthent->pszUser);
					CString Pass(pAuthent->pszPassword);

					logger.Append(User);
					//logger.Append(Pass);	//too critical to log!

					if(User!=""){
						hack |= ScanAuthentication(User,Pass,IP);
					}
				}
			}
		}
	
		//if possible hack attempt
		if(hack){
			logger.Flush();								//flush the log entry
			HackResponseBlockOrMonitor(IP);				//block or monitor IP address
			if(!logonly)								//if we have to do more than logging
				HackResponse(pfc,ServerHeader);			//show page
			lock.Unlock();								//unlock globals
			if(logonly){								//if we are only here to log
				return CHttpFilter::OnAuthentication(pfc, pAuthent);
			}else{										//if we have to do more than logging
				return SF_STATUS_REQ_FINISHED;			//make sure IIS does nothing with request
			}
		}else{
			if(Settings.Logging.LogAllowed){			//do not log allowed requests if asked
				logger.Append("ALLOWED");
				logger.Flush();							//clear the log entry
			}else{
				logger.Cancel();
			}
			lock.Unlock();								//unlock globals
			return CHttpFilter::OnAuthentication(pfc, pAuthent);
		}
	}
	return CHttpFilter::OnAuthentication(pfc, pAuthent);

#ifdef ERROR_HANDLE
	}catch(...){
		if(DENY_ON_ERROR){							//deny request on error if asked
			return SF_STATUS_REQ_FINISHED;			//don't keep connection, because of error
		}else{
			return CHttpFilter::OnAuthentication(pfc, pAuthent);
		}
	}
#endif
}

DWORD CISAPIFirewall::OnUrlMap(CHttpFilterContext* pfc, PHTTP_FILTER_URL_MAP pUrlMap, LPCTSTR SiteInstance, LPCTSTR ServerHeader) 
{
#ifdef ERROR_HANDLE
	try{
#endif

	if(pfc!=NULL && pUrlMap!=NULL){
		CString instance = GetServerVariable(pfc,"INSTANCE_ID");

		CSingleLock lock(&crit, TRUE);

		logger.NewEntry(SiteInstance + instance);
		logger.Append("OnUrlMap");
		LogClientInfo(pfc);

		const char* p;
		bool hack(false);
		bool logonly(Settings.Response.LogOnly);
		bool respdrop(Settings.HTTPResponse.DropConnection);

		CString IP(GetServerVariable(pfc,"REMOTE_ADDR"));
		if(IsExcluded(instance)){
			logger.Append("Web site instance is excluded from scanning");
		}else{
			if(IsExcludedIP(IP)){
				logger.Append("IP address is excluded from scanning");
			}else{

				//URL Scanning
				p = pUrlMap->pszURL;
				logger.Append(CString(p).Left(9000));
				hack |= ScanURL(p);
			
				//PATH
				p = pUrlMap->pszPhysicalPath;
				CFileName fn(p);
				hack |= ScanPath(fn);
				
				//QUERYSTRING
				hack |= ScanRawQuerystring(GetServerVariable(pfc,"QUERY_STRING"));

				//AUTH
				if(Settings.Authentication.ScanAccountAllEvents){
					hack |= ScanAccount(GetServerVariable(pfc,"LOGON_USER"));
				}

				//MONITOR - FILENAME
				if(Settings.Filename.MonitorFiles.Enabled){
					if(IsInList(fn.FileNameLCase,Settings.Filename.MonitorFiles.List)){
						logger.Append("MONITORED: File access: '" + fn.FileName + "'");
						LogClientInfoExtra(pfc);
						logger.Flush();
					}
				}

				//no more code here (logger.Flush())
			}
		}
			

		//if possible hack attempt
		if(hack){
			logger.Flush();								//flush the log entry
			HackResponseBlockOrMonitor(IP);				//block or monitor IP address
			if(!logonly)								//if we have to do more than logging
				HackResponse(pfc,ServerHeader);			//show page
			lock.Unlock();								//unlock globals
			if(logonly){								//if we are only here to log
				return CHttpFilter::OnUrlMap(pfc, pUrlMap);
			}else{										//if we have to do more than logging
				if(respdrop){
					return SF_STATUS_REQ_FINISHED;		//make sure IIS does nothing with request
				}else{
					return SF_STATUS_REQ_FINISHED_KEEP_CONN;//make sure IIS does nothing with request
				}
			}
		}else{
			if(Settings.Logging.LogAllowed){			//do not log allowed requests if asked
				logger.Append("ALLOWED");
				logger.Flush();
			}else{
				logger.Cancel();						//clear the log entry
			}
			lock.Unlock();								//unlock globals
			return CHttpFilter::OnUrlMap(pfc, pUrlMap);
		}
	}
	return CHttpFilter::OnUrlMap(pfc, pUrlMap);

#ifdef ERROR_HANDLE
	}catch(...){
		if(DENY_ON_ERROR){							//deny request on error if asked
			return SF_STATUS_REQ_FINISHED;			//don't keep connection, because of error
		}else{
			return CHttpFilter::OnUrlMap(pfc, pUrlMap);
		}
	}
#endif
}

DWORD CISAPIFirewall::OnSendRawData(CHttpFilterContext* pfc, PHTTP_FILTER_RAW_DATA pRawData, LPCTSTR SiteInstance)
{
	/*
	 * Log only if an error occured, but deny nothing!
	 */ 
#ifdef ERROR_HANDLE
	try{
#endif

	if(pfc!=NULL && pRawData!=NULL){
		
		if(pfc->m_pFC->pFilterContext!=CONTEXT_SENDINGDATA){	//only if headers are not sent back: not good with bad requests (stays NULL)

			bool doflush(false);
			bool reset(true);
			CString instance = GetServerVariable(pfc,"INSTANCE_ID");

			CSingleLock lock(&crit, TRUE);

			if (instance.GetLength()==0){
				logger.NewEntry("GLOBAL");
			}else{
				logger.NewEntry(SiteInstance + instance);
			}
			logger.Append("OnSendRawData");
			LogClientInfo(pfc);

			if(IsExcluded(instance)){
				logger.Append("Web site instance is excluded from scanning");
			}else{
				CString IP(GetServerVariable(pfc,"REMOTE_ADDR"));
				if(IsExcludedIP(IP)){
					logger.Append("IP address is excluded from scanning");
				}else{

					//only if we did not already process this request and this is just additional data
					//if(pfc->m_pFC->pFilterContext != 2){ //this did not work!!! (with HTTP 100 Continue response,
					//next response from server: no server header changed/removed)
					
					//prepare the raw data before processing	
					CString resp("");
					bool doCustomScan(true);
					if(!ConvertBinaryData((BYTE*)pRawData->pvInData,pRawData->cbInData,resp)){
						#ifdef LOG_DEBUG_ONSENDRAWDATA
						logger.Append("Could not convert binary to string (NULL characters present)");
						doflush = true;
						#endif
						resp = CString((char*)pRawData->pvInData);	//to first NULL
						doCustomScan = false;
					}

					#ifdef LOG_DEBUG_ONSENDRAWDATA
					logger.Append("\r\nRaw data:");
					logger.Append(resp);
					doflush = true;
					#endif

					//start of a http response
					if(resp.Left(5)=="HTTP/"){
						int pos;
						CString lresp;

						//status
						CString status("");
						pos = resp.Find(CRLF);
						status = resp.Left(pos==-1?resp.GetLength():pos);
						pos = status.Find(' ');
						if(pos!=-1){
							status = status.Mid(pos);

							if(status.GetLength()>2){

								bool logext(false);
								if(ScanResponseStatus(pfc,status,reset,logext)){
									lresp = resp;
									lresp.Replace('\r',' ');
									lresp.Replace('\n',' ');
									logger.Append(lresp.Left(1024));
									doflush = true;
									if(logext){ //server error
										//log all subsequent writes until another request is received
										reset = false;
										pfc->m_pFC->pFilterContext = CONTEXT_SENDINGDATALOG;
									}
								}
							}
						}						
						if(doCustomScan){//if not embedded NULL
							doflush |= CustomSendRawData(pfc,pRawData,resp);
						}
					}else{
						//ASP.NET errors (in separate data write after headers)
						if(pfc->m_pFC->pFilterContext==CONTEXT_SENDINGDATALOG){
							CString lresp = resp;
							lresp.Replace('\r',' ');
							lresp.Replace('\n',' ');
							logger.Append(lresp.Left(8192));
							doflush = true;
						}
					}
				}
			}

			//reset context
			if(reset)
				pfc->m_pFC->pFilterContext = CONTEXT_SENDINGDATA;

			//flush or cancel log entry
			if (doflush){
				logger.Flush();
			}else{
				logger.Cancel();
			}

			lock.Unlock();
		}
	}
	return CHttpFilter::OnSendRawData(pfc, pRawData);

#ifdef ERROR_HANDLE
	}catch(...){
		//do nothing, some error occured removing/changing server header, but allow request
		return CHttpFilter::OnSendRawData(pfc, pRawData);
	}
#endif
}

void CISAPIFirewall::HackResponse(CHttpFilterContext* pfc,LPCTSTR ServerHeader)
{
	if(Settings.HTTPResponse.Directly){
		//direct response to client (writeclient)
		CString buf("HTTP/1.1 ");
		if(Settings.HTTPResponse.UseStatus){
			buf += Settings.HTTPResponse.Status;
		}else{
			buf += "404 Object Not Found";
		}
		if(!Settings.Headers.RemoveServerHeader){
			buf += "\r\nServer: ";
			if(Settings.Headers.ChangeServerHeader){
				buf += Settings.Headers.ServerHeader;
			}else{
				buf += ServerHeader;
			}
		}
		_tzset();
		CString d = CTime::GetCurrentTime().FormatGmt("%a, %d %b %Y %H:%M:%S GMT");
		buf += "\r\nDate: " + d;
		buf += "\r\nContent-Type: text/html; charset=windows-1252";
		buf += "\r\nContent-Length: "; buf += CStringHelper::Safe_IntToAscii(Settings.HTTPResponse.Contents.GetLength());
		buf += "\r\nPragma: no-cache";
		buf += "\r\nCache-control: no-cache";
		buf += "\r\nExpires: " + d;
		buf += CRLFCRLF;
		buf += Settings.HTTPResponse.Contents;
		DWORD szbuf(buf.GetLength());
		pfc->WriteClient(buf.GetBuffer(0),&szbuf);
	}else if(Settings.HTTPResponse.Redirect){
		//redirect request to a url
		CString str("Location: " + Settings.HTTPResponse.RedirectURL + CRLF);
		LPSTR lps = new char[str.GetLength()+1];
		if(lps!=NULL){
			strcpy(lps, str);
			pfc->AddResponseHeaders(lps,0);
			delete[] lps;
		}
		pfc->ServerSupportFunction(SF_REQ_SEND_RESPONSE_HEADER,"302 Object Moved",NULL,NULL);
	}else if(Settings.HTTPResponse.UseStatus){
		//respond with special header only
		pfc->ServerSupportFunction(SF_REQ_SEND_RESPONSE_HEADER,(VOID*)(LPCSTR)Settings.HTTPResponse.Status,NULL,NULL);
	}
	pfc->m_pFC->pFilterContext = CONTEXT_NEWREQUEST; //reset for OnReadRawData
}

void CISAPIFirewall::HackResponseBlockOrMonitor(CString& IP)
{
	//block or monitor IP address
	if(Settings.HTTPResponse.MonitorIP){
		IP_HACK_Monitored.AddNotInCache(IP);
	}
	if(Settings.HTTPResponse.BlockIP){
		IP_HACK_Blocked.AddNotInCache(IP);
	}
}

void CISAPIFirewall::LogClientInfo(CHttpFilterContext* pfc)
{
	if(Settings.Logging.LogClientIP) logger.Append(GetServerVariable(pfc,"REMOTE_ADDR"));
	if(Settings.Logging.LogUserName) logger.Append(GetServerVariable(pfc,"LOGON_USER"));
	if(Settings.HTTPLogging.Log_HTTP_VIA) logger.Append(GetServerVariable(pfc,"HTTP_VIA"));
	if(Settings.HTTPLogging.Log_HTTP_X_FORWARDED_FOR) logger.Append(GetServerVariable(pfc,"HTTP_X_FORWARDED_FOR"));
	if(Settings.HTTPLogging.LogHostHeader) logger.Append(GetServerVariable(pfc,"HTTP_HOST"));
	if(Settings.HTTPLogging.LogUserAgent) logger.Append(GetServerVariable(pfc,"HTTP_USER_AGENT"));
}

inline void CISAPIFirewall::LogClientInfoExtra(CHttpFilterContext* pfc)
{
	logger.Append("Referer: '" + GetServerVariable(pfc,"HTTP_REFERER") + "'");
	logger.Append("User-Agent: '" + GetServerVariable(pfc,"HTTP_USER_AGENT") + "'");
	logger.Append("From: '" + GetServerVariable(pfc,"HTTP_FROM") + "'");
	logger.Append("Accept-Language: '" + GetServerVariable(pfc, "HTTP_ACCEPT_LANGUAGE") + "'");
	logger.Append("Accept: '" + GetServerVariable(pfc, "HTTP_ACCEPT") + "'");
	logger.Append("Content-Type: '" + GetServerVariable(pfc, "HTTP_CONTENT_TYPE") + "'");
}

inline void CISAPIFirewall::LogRequestInfo(CHttpFilterContext* pfc)
{
	logger.Append(GetServerVariable(pfc,"REQUEST_METHOD"));
	logger.Append(GetServerVariable(pfc,"SCRIPT_NAME"));
	logger.Append(GetServerVariable(pfc,"PATH_INFO"));
	logger.Append(GetServerVariable(pfc,"QUERY_STRING"));
}

bool CISAPIFirewall::ScanResponseStatus(CHttpFilterContext* pfc, LPCTSTR Status,bool& ResetFilterContext, bool& LogExtendedResponse)
{
	if(Status!=NULL){
		switch(Status[1]){
		case '1':
			//if this is not a class 1 response (100 Continue,101), reset filtercontext pointer
			//otherwise with a simple refresh in browser one might circumvent header checking (filtercontext = void* 1)
			//because filtercontext = per connection (with keep-alives -> same filtercontext)
			ResetFilterContext=false;
			return false;
			break;
		case '4':
			if(Settings.HTTPLogging.LogClientErrors){
				logger.Append("HTTP Client Error");
				LogRequestInfo(pfc);
				LogClientInfoExtra(pfc);
				return true;
			}
			break;
		case '5':
			if(Settings.HTTPLogging.LogServerErrors){
				logger.Append("HTTP Server Error");
				LogRequestInfo(pfc);
				LogClientInfoExtra(pfc);
				LogExtendedResponse=true; //also log subsequent data sending
				return true;
			}
			break;
		}
	}
	return false;
}

void CISAPIFirewall::SetFilterVersion(PHTTP_FILTER_VERSION pVer)
{
	//if ( pVer->dwServerFilterVersion >= MAKELONG( 0, 6 ) )
	//{
	//// The server is IIS6 and later - should not use SF_NOTIFY_READ_RAW_DATA
	//}

	// Set the filter priority
	if(Settings.Engine.AllowLateScanning){
		pVer->dwFlags |= SF_NOTIFY_ORDER_LOW;
		logger.NewEntryFlush("WARNING: Firewall is installed as low priority (possible risk)");
	}else{
		pVer->dwFlags |= SF_NOTIFY_ORDER_HIGH;
		logger.NewEntryFlush("INFO: Firewall is installed as high priority (very secure)");
	}
	
	//are we a global filter? if yes, we can use the OnReadRawData event
#ifdef DO_READRAWDATA_EVENT
	if(pVer->dwServerFilterVersion >= MAKELONG( 0, 7 )){ //no OnReadRawData in IIS7 and later
		logger.NewEntryFlush("INFO: Firewall will not process raw data because this ISAPI version does not support it (OnReadRawData event is not supported in IIS7 and later)");
	}else{
		if(Settings.Global.IsInstalledAsGlobalFilter){
			pVer->dwFlags |= SF_NOTIFY_READ_RAW_DATA;
			logger.NewEntryFlush("INFO: Firewall will process raw data because it is a global filter (OnReadRawData event)");
			logger.NewEntryFlush("WARNING: If the firewall fails to load (red arrow down), make sure it is installed as a global filter or uncheck 'Is Installed As Global Filter' in Global Filter Capabilities (needs a restart)");
		}else{
			logger.NewEntryFlush("WARNING: Firewall will not process raw data because it is not a global filter (no OnReadRawData event, all other capabilities of the firewall will still be used)");
			logger.NewEntryFlush("INFO: If you want to use the additional raw data scanning capabilities of the firewall, you need to install it as a global filter and make sure the 'Is Installed As Global Filter' under Global Filter Capabilities is checked (needs a restart)");
		}
	}
#endif
	
	//do we need to scan non-secure port?
	if(Settings.Engine.ScanNonSecurePort){
		pVer->dwFlags |= SF_NOTIFY_NONSECURE_PORT;
		logger.NewEntryFlush("INFO: HTTP traffic will be monitored");
	}else{
		logger.NewEntryFlush("WARNING: HTTP traffic will NOT be monitored");
	}
	
	//do we need to scan secure port?
	if(Settings.Engine.ScanSecurePort){
		pVer->dwFlags |= SF_NOTIFY_SECURE_PORT;
		logger.NewEntryFlush("INFO: HTTPS traffic will be monitored");
	}else{
		logger.NewEntryFlush("WARNING: HTTPS traffic will NOT be monitored");
	}

}

bool CISAPIFirewall::OnReadRawDataDebug(CHttpFilterContext *pCtxt, PHTTP_FILTER_RAW_DATA pRawData)
{
	try{

		logger.NewEntry("GLOBAL");
		logger.Append("OnReadRawDataDebug");
		LogClientInfo(pCtxt);
		logger.Append("pRawData->cbInBuffer");
		logger.Append(CStringHelper::Safe_IntToAscii((int)pRawData->cbInBuffer));
		logger.Append("pRawData->cbInData");
		logger.Append(CStringHelper::Safe_IntToAscii((int)pRawData->cbInData));
		logger.Append("pFilterContext");
		logger.Append(CStringHelper::Safe_IntToAscii((int)pCtxt->m_pFC->pFilterContext));
		logger.Flush();
		return true;

	}catch(...){
		return false;
	}
}

bool CISAPIFirewall::ScanHeaders(CHttpFilterContext *pfc, PHTTP_FILTER_PREPROC_HEADERS pHeaders)
{
	bool hack(false);
	POSITION pos;
	CString str;

	//HEADERS: Deny certain headers (list is case insensitive by RFC - GetHeader does this for us)
	if(Settings.Headers.DenyHeaders.Enabled){
		pos = Settings.Headers.DenyHeaders.List.GetHeadPosition();
		while (pos != NULL){
			str=Settings.Headers.DenyHeaders.List.GetNext(pos);
			CString h = GetHeader(pfc,pHeaders,str.GetBuffer(0));
			if(h!=""){
				hack = true;
				logger.Append("BLOCKED: Header '" + str + "' not allowed");
				logger.Append(h.Left(1024));
			}
		}
	}

	//HEADERS: Check length of certain headers
	if(Settings.Limits.MaxHeaders.Enabled){
		pos = Settings.Limits.MaxHeaders.Map.GetStartPosition();
		CString hsize; CString h;
		while(pos!=NULL){
			Settings.Limits.MaxHeaders.Map.GetNextAssoc(pos, h, hsize);
			str = GetHeader(pfc,pHeaders,h.GetBuffer(0));
			if(str.GetLength()>atol(hsize)){
				hack = true;
				logger.Append("BLOCKED: Header '" + h + "' too long");
				logger.Append(str.Left(1024));
			}
		}
	}

	return hack;
}

bool CISAPIFirewall::ScanConnection(CHttpFilterContext *pfc, CString &ip)
{
	bool hack(false);

	//CONNECTION - IP blocking
	if(Settings.Connection.DenyAddresses.Enabled){
		if(DenyIPRangeList.IsInList(ip)){
			hack = true;
			logger.Append("BLOCKED: IP address '" + ip + "' not allowed");
		}
	}

	//CONNECTION - Request Limits
	if(Settings.Connection.RequestsLimit.Enabled){
		//add request to cache
		IP_RequestsLimit.Add(ip);

		//block if number of request exceeds certain treshold
		if(IP_RequestsLimit.Count(ip)>Settings.Connection.RequestsLimit.MaxCount){
			hack = true;
			logger.Append("BLOCKED: Reached connection request limit");
		}
	}

	//CONNECTION - Hacking IP Blocking
	if(Settings.HTTPResponse.BlockIP){
		if(IP_HACK_Blocked.Exists(ip)){
			hack = true;
			logger.Append("BLOCKED: Blocked IP address (previous alert)");
		}
	}

	bool flush(false);
	//CONNECTION - Hacking IP Monitoring
	if(Settings.HTTPResponse.MonitorIP){
		if(IP_HACK_Monitored.Exists(ip)){
			flush = true;
			logger.Append("MONITORED: IP address (previous alert)");
		}
	}

	//CONNECTION - IP Monitoring
	if(Settings.Connection.MonitorAddresses.Enabled){
		if(MonitorIPRangeList.IsInList(ip)){
			flush = true;
			logger.Append("MONITORED: IP address '" + ip + "'");
		}
	}

	if(flush){ //monitored connection
		LogClientInfoExtra(pfc);
		logger.Flush();
	}
	//no code here: logger.Flush() can already be called

	return hack;
}

bool CISAPIFirewall::CustomHeaderScanning(CHttpFilterContext *pfc, PHTTP_FILTER_PREPROC_HEADERS pHeaders, CString& ip, CURL& RawUrl)
{
	CString ua(GetHeader(pfc,pHeaders,"User-Agent:"));
	logger.Append(ua.Left(2048));

	//HEADERS: User Agent
	return ScanUserAgent(ua);
}

bool CISAPIFirewall::CustomSendRawData(CHttpFilterContext *pfc, PHTTP_FILTER_RAW_DATA pRawData, CString& Response)
{
	return false;
}
