/*
    AQTRONIX C++ Library
    Copyright 2002-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// Logger.h: interface for the CLogger class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LOGGER_H__081BF6DC_4410_406A_A3A5_4F61505BBD9F__INCLUDED_)
#define AFX_LOGGER_H__081BF6DC_4410_406A_A3A5_4F61505BBD9F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "LogFile.h"

//#define ENABLE_ODBC
#ifdef ENABLE_ODBC
#include "ODBCConnection.h"
#endif

#define SECS_IN_DAY 86400					//number of seconds in a day
#define CLOGGER_LOG_FORMAT "%y.%m.%d.log"	//format of the filename
#define CLOGGER_FIELD_SEPARATOR " ; "

#define CLOGGER_DATEFORMAT "%Y-%m-%d"
#define CLOGGER_TIMEFORMAT "%H:%M:%S"

class CLogger : private CObject  
{
public:
#ifdef ENABLE_ODBC
	bool SetODBC(LPCTSTR ConnectionString, LPCTSTR Table);
	bool LogODBC(CString Msg);
#endif
	bool GMTLogging;
	void SetHeaderFields(CStringList& FieldList);

	void NewEntryFlushNoTimestamp(const CString& msg="");
	void NewEntryNoTimestamp(const CString& msg="");
	void SetMaxLogLine(int size);
	void SetMaxLogItem(int size);
	void SetHeader(LPCTSTR pSoftware = NULL, LPCTSTR pOptional = NULL);
	void Cancel();
	bool Enabled;
	void NewEntryFlush(const CString& msg="");
	void NewEntryFlush(const CStringList& List);
	void NewEntry(const CString& msg="");
	void NewEntry(const CStringList& List);
	void Flush();
	void SetRetention(const int ret);
	void SetLogDir(const CString& d, const bool EnableLogging = true);
	void Append(const CString&);
	void Append(const int i);
	void Append(const double d);
	void Append(const CStringList& List);

	CLogger();
	CLogger(LPCTSTR ld,int ret=28);
	virtual ~CLogger();

protected:
	CString GetHeader();

	//ODBC
#ifdef ENABLE_ODBC
	CString LogTable;
	CODBCConnection ODBC;
	bool UseODBC;
#endif

	//header
	bool UseHeader;
	CString DirectiveSoftware;
	CString DirectiveOptional;
	CString DirectiveFields;

	int MaxLogLine;
	int MaxLogItem;
	void DeleteOldLogs();
	void UpdateClock();
	CLogFile CurrentLog;	//the logfile itself
	CString LogDate;		//the date in filename format for the logfile
	CString LogDir;			//the folder to put the log files
	int Retention;			//period to keep old log files (in days)
	CString Buffer;			//a single line buffer

	CString GetLocalDifference();
	CString GetGMTDifference();

};

#endif // !defined(AFX_LOGGER_H__081BF6DC_4410_406A_A3A5_4F61505BBD9F__INCLUDED_)
