/*
	AQTRONIX WebKnight - ISAPI Filter for securing IIS
	Copyright 2002-2008 Parcifal Aertssen

	This file is part of AQTRONIX WebKnight.

    AQTRONIX WebKnight is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

    AQTRONIX WebKnight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX WebKnight; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
// WEBKNIGHT.CPP - Implementation file for your Internet Server
//    WebKnight Filter


#include "stdafx.h"
#include "WebKnight.h"

#define STRING2(x) #x
#define STRING(x) STRING2(x)

#ifdef _MSC_VER
#pragma message("***** _MSC_VER:" STRING(_MSC_VER) "*****")
#if _MSC_VER > 1200
						//VS 6.0 = MSC 1200
						//VS 2005 = MSC 1400
#else
#pragma message("***** To avoid errors, make sure you add the project Library folder to your include sources (Tools->Options->Directories). *****")
#endif
#endif

#ifdef IISBUILD
#define SITEINSTANCE "W3SVC"		//IIS site labelling
#else
#define SITEINSTANCE "Instance "	//default site labelling
#endif

/* AQTRONIX WebKnight
 *
 * See README.TXT for information about the source code
 *
 * 13 August    2002: Project started
 * 26 October   2002: Started coding a GUI for settings
 *  4 November  2002: Beta 1.0 released (without GUI)
 * 12 January   2003: First version of GUI finished
 * 25 February  2003: Windows Installer package finished
 * 24 March     2003: Final 1.0 released (+config 1.0, +installer 1.0)
 * 14 July      2003: Final 1.1 released (+config 1.1, +installer 1.1)
 *  8 September 2003: Final 1.2 released (+config 1.1, +installer 1.2)
 * 10 November  2003: Final 1.3 released (+config 1.2, +installer 1.3)
 * 24 December  2006: Final 2.0 released (+config 1.3, +installer 2.0 + loganalysis 1.0)
 *  8 October   2007: Final 2.1 released (+config 1.3, +installer 2.1 + loganalysis 1.1)
 *  2 September 2008: Final 2.2 released (+config 1.4, +installer 2.2 + loganalysys 1.2)
 *
 * You can not define _UNICODE because this program is not made for it and the
 * ISAPI framework does not (yet) support Unicode (I use Visual C++ 6.0).
 * If you still want to enable this you have to review every piece of code and
 * adjust all char types, "str" functions,...
 *
 * For IIS6 (and up) you cannot register for the OnReadRawData event if IIS is
 * running in Worker Process mode (only works in IIS 5.0 Isolation mode).
 *
 * Making default settings (pre WebKnight 2.0 Final):
 * - remove logdir
 * - remove denied ip addresses
 * - uncheck UseDenyIPList
 * - remove denied user agents
 * - uncheck UseDenyHeaderUserAgent
 * - remove denied user agent sequences
 * - uncheck UseDenyHeaderUserAgentSequences
 */

/* TODO
 * -----
 * WEBKNIGHT
 *  <2.3>
 * - IPv6 problem on W2008
		MonitorIPRangeList.Enumerate(Settings.Connection.MonitorAddresses.List);
		DenyIPRangeList.Enumerate(Settings.Connection.DenyAddresses.List);
		ExcludedIPRangeList.Enumerate(Settings.Engine.ExcludedIPAddresses.List);
		127.0.0.1 -> ::1
 * - IIS 6 isapi wildcard alternative to global filtering
 * - log file per web instance / single file logging like urlscan 3.0 beta
 * - fully multithreaded scanning engine (not just thread safe)
 *
 *  <2.?>
 * - captcha alternative / data analysis to detect bots and spamming POST messages
		- http://www.stopforumspam.com/apis (90.152.30.61 zit er in en heeft gespamd)
		- http://isc.sans.org/diary.html?storyid=1836
		- session not initialized or no jpg/gif/css file requested
		- http://www.usenix.org/events/nsdi05/tech/kandula/kandula_html/
 * - log backup
 * - WebKnight ISA Server installation + test this
 * - Exchange2003 compatibility (functions)
 * - Robots database auto update
 * - Denied unix commands in postdata/headers?
 * - ripping protection (image ripping or other file extension)
 * - ODBC logging
 *   - connection timeout
 *   - threading problem (CoInitializeEx())
 * - generic header length
 * - generic variable length (query, postdata)
 * - Unicode ready
 * - set permissions on WebKnight dir & logdir (createpath function)?
 * - headers in request (set value/block winhttp.dll headers)
 * - application/x-www-UTF8-encoded & UTF (url,query,header,data) + check invalid UTF sequences
 *
 * - screenshots of settings/possibilities of webknight on web site
 * - changelog: - Added ODBC Logging
 *
 *  <3.0>
 *									    	Function / Settings / Testing
 * - compatible with SOAP					   Ok		  OK	   TO DO
 * - compatible with IPP					   OK		  OK	   TO DO
 * - compatible with PHP					   OK		 TO DO     TO DO
 * - compatible with BITS					   OK		  OK	   TO DO
 * - compatible with Outlook Web Access		   OK		  OK	    OK
 * - compatible with Outlook Mobile Access	   OK		  OK	   TO DO
 * - compatible with ActiveSync				   OK		  OK	   TO DO
 * - compatible with RPC over HTTP			   OK		  OK	   TO DO
 * - compatible with Frontpage Extensions	   OK         OK       TO DO
 * - compatible with Coldfusion				   OK        TO DO     TO DO
 * - compatible with Flash MX				   OK        TO DO     TO DO
 * - compatible with WebDAV					   OK         OK        OK
 * - compatible with IISADMPWD				   OK		  OK	   TO DO
 * - compatible with Sharepoint Portal Server  OK	      OK       TO DO
 * - compatible with Sharepoint Team Services  OK		  OK       TO DO
 * - compatible with Certificate Services	   OK         OK       TO DO
 * - compatible with BizTalk				   OK         OK       TO DO
 * - compatible with Commerce Server		   OK         OK       TO DO
 * - compatible with Small Business Server	   OK         OK       TO DO
 * - compatible with ASP.NET				   OK         OK       TO DO
 * - compatible with ASP					   OK         OK        OK
 * - compatible with Static HTML			   OK         OK        OK
 * - compatible with Unicode				   OK         OK       TO DO
 * - compatible with File Uploads			   OK		  OK	   TO DO
 *
 *
 * FEATURES UNDER CONSIDERATION
 * - fake user agents:  - cloak robots.txt Disallow: /
 *						- first request is for robots.txt -> block it
 *
 * WANTED FEATURES
 * - Central Policy Management: Settings in local file or Active Directory (+schedules, +organizational unit, +multiple policies)
 * - Real-Time Statistics
 * - Logging to ODBC Database
 * - MMC or tabsheet plugin for IIS: using own GUI XML protocol for future compatibility with Linux
 *
 * REJECTED FEATURES
 * - scan user agent anomalies (unusual http header sequence,...)
 * - print loaded settings (only options) to the log file (when a load/reload of settings happens) REJECTED: too much settings, this is not urlscan!
 * - compare settings with previously loaded settings when printing settings to log file
 * - validHostport (see kodeit), this is stupid because of HTTP 0.9 and HTTP 1.0 requests still possible
 * - Logging: W3C extended log format: too limited!
 * - Settings in Registry: why use that when you have XML or LDAP?
 *
 * POSSIBLE EXPLOITS IN IIS:
 * - %00 exploit?
 * - Ms-Echo-Request header??
 * - VERBS (from winhttp.dll & SEARCH exploit?)
 * - headers (from winhttp.dll)
 *
 * BUGS IN WEBKNIGHT
 * - Settings: INI file: writing ';' in SQL Keywords cannot be read by ReadFrom function -> WILL NOT BE FIXED (Reason: INI settings not used by default)
 *
 * BUGS IN ISAPI FRAMEWORK:
 * 1) if attack detected in OnReadRawData -> no OnSendRawData event (server header not
 *    changed or removed and filtercontext not reset). SOLUTION: added code in HackResponse().
 *    Only works with ResponseDirectly=1! If you redirect to a url or only send back a
 *    ResponseStatus then server header is not changed.
 */

///////////////////////////////////////////////////////////////////////
// The one and only CWebKnightFilter object

CWebKnightFilter theFilter;

///////////////////////////////////////////////////////////////////////
// CWebKnightFilter implementation

CWebKnightFilter::CWebKnightFilter():CISAPIFirewall(Settings){

	CString strinit; CString strinit2; CString strinit3;

	//get the directory of the .dll file & windows directory
	CFileName dll = GetProcessPath("WebKnight.dll");
	CString windir = GetWindowsPath();

	CSingleLock lock(&crit, TRUE);

	strinit = LoadSettingFile(dll.Path,"WebKnight.xml","WebKnight.ini","urlscan.ini");
	strinit2 = SaveDefaultSettingFile(dll.Path,"WebKnight.xml");

	CString logdir;
#ifdef SYSTEM_LOG_BUILD
	logdir = windir + "\\System32\\LogFiles\\WebKnight\\"; //for global filter
	#pragma message("***** This build will log under the LogFiles folder of your Windows NT installation! *****")
#else
	logdir = dll.Path  + "LogFiles\\";				   //<dir of dll>\LogFiles
#endif
	strinit3 = InitializeLogger(logdir,APP_FULLNAME);

	logger.NewEntryFlush("AQTRONIX WebKnight loaded");
	logger.NewEntryFlush(strinit);

	if(strinit2!="")
		logger.NewEntryFlush(strinit2);
	if(strinit3!="")
		logger.NewEntryFlush(strinit3);

	//are we in logging only mode? if so, warning!
	WarnLogonlyMode();

	Settings.ReadAgentsFile(dll.Path + "Robots.xml");

	LoadCache();

	SaveRunningValues(dll.Path,"Loaded.xml");

	lock.Unlock();
}

CWebKnightFilter::~CWebKnightFilter()
{
	CSingleLock lock(&crit, TRUE);
	logger.NewEntryFlush("AQTRONIX WebKnight unloaded");
	lock.Unlock();
}

BOOL CWebKnightFilter::GetFilterVersion(PHTTP_FILTER_VERSION pVer)
{
	// Call default implementation for initialization
	CHttpFilter::GetFilterVersion(pVer);
	// Clear the flags set by base class
	pVer->dwFlags &= ~SF_NOTIFY_ORDER_MASK;

	CSingleLock lock(&crit, TRUE);

	SetFilterVersion(pVer);

	pVer->dwFlags	|= SF_NOTIFY_SEND_RAW_DATA | SF_NOTIFY_URL_MAP
					| SF_NOTIFY_PREPROC_HEADERS | SF_NOTIFY_AUTHENTICATION;

	// Load description string
	TCHAR sz[SF_MAX_FILTER_DESC_LEN+1];
	ISAPIVERIFY(::LoadString(AfxGetResourceHandle(),IDS_FILTER, sz, SF_MAX_FILTER_DESC_LEN));
	_tcscpy(pVer->lpszFilterDesc, sz);

	lock.Unlock();
	return TRUE;
}

DWORD CWebKnightFilter::OnReadRawData(CHttpFilterContext* pCtxt,PHTTP_FILTER_RAW_DATA pRawData)
{
	return CISAPIFirewall::OnReadRawData(pCtxt,pRawData,APP_SERVER_HEADER);
}

DWORD CWebKnightFilter::OnPreprocHeaders(CHttpFilterContext* pfc, PHTTP_FILTER_PREPROC_HEADERS pHeaders) 
{
	return CISAPIFirewall::OnPreprocHeaders(pfc,pHeaders,SITEINSTANCE,APP_SERVER_HEADER);
}

DWORD CWebKnightFilter::OnUrlMap(CHttpFilterContext* pfc, PHTTP_FILTER_URL_MAP pUrlMap) 
{
	return CISAPIFirewall::OnUrlMap(pfc,pUrlMap,SITEINSTANCE,APP_SERVER_HEADER);
}

DWORD CWebKnightFilter::OnAuthentication(CHttpFilterContext* pfc, PHTTP_FILTER_AUTHENT pAuthent) 
{
	return CISAPIFirewall::OnAuthentication(pfc,pAuthent,SITEINSTANCE,APP_SERVER_HEADER);
}

DWORD CWebKnightFilter::OnSendRawData(CHttpFilterContext* pfc, PHTTP_FILTER_RAW_DATA pRawData)
{
	return CISAPIFirewall::OnSendRawData(pfc,pRawData,SITEINSTANCE);
}

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CWebKnightFilter, CHttpFilter)
	//{{AFX_MSG_MAP(CWebKnightFilter)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

///////////////////////////////////////////////////////////////////////
// If your extension will not use MFC, you'll need this code to make
// sure the extension objects can find the resource handle for the
// module.  If you convert your extension to not be dependent on MFC,
// remove the comments arounn the following AfxGetResourceHandle()
// and DllMain() functions, as well as the g_hInstance global.

/****

static HINSTANCE g_hInstance;

HINSTANCE AFXISAPI AfxGetResourceHandle()
{
	return g_hInstance;
}

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ulReason,
					LPVOID lpReserved)
{
	if (ulReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hInst;
	}

	return TRUE;
}

****/


void CWebKnightFilter::LoadCache()
{
	CHTTPFirewall::LoadCache();

	BOT_Blocked.SetMaxAge(CTimeSpan(0,Settings.Robots.BotTimeout>1?Settings.Robots.BotTimeout:1,0,0));
	BOT_Aggressive.SetMaxAge(CTimeSpan(0,0,Settings.Robots.DenyAggressive.MaxTime>1?Settings.Robots.DenyAggressive.MaxTime:1,0));
}

bool CWebKnightFilter::CustomHeaderScanning(CHttpFilterContext *pfc, PHTTP_FILTER_PREPROC_HEADERS pHeaders, CString& ip, CURL& RawUrl)
{
	bool hack(false);
	CFileName fn(CURL::URLEncodedToASCII(RawUrl.Url));

	//BOTS
	CString ua(GetHeader(pfc,pHeaders,"User-Agent:"));
	logger.Append(ua.Left(2048));

	bool bRobotFile(false);

	//BOTS
	if(fn.FileNameLCase=="robots.txt"){
		//BOTS - Robots.txt
		if(Settings.Robots.AllowRobotsFile){
			logger.Append("File access 'robots.txt'");
			bRobotFile = true;
		}
		//BOTS - Block All
		if(Settings.Robots.DenyAll){
			BOT_Blocked.Add(ip,ua);
			logger.Append("BLOCKED: Robots not allowed");
		}
		//BOTS - Deny aggressive - start monitoring
		if(Settings.Robots.DenyAggressive.Enabled){
			//add request to cache to start monitoring
			BOT_Aggressive.Add(ip,ua);
		}
	}else{
		//BOTS - Block Aggressive Bots - monitor subsequent requests after robots.txt
		if(Settings.Robots.DenyAggressive.Enabled){
			if(BOT_Aggressive.Exists(ip,ua)){
				//add request to cache
				BOT_Aggressive.Add(ip,ua);

				//block if number of request exceeds certain treshold
				if(BOT_Aggressive.Count(ip)>Settings.Robots.DenyAggressive.MaxCount){
					BOT_Blocked.Add(ip,ua);
					logger.Append("BLOCKED: Aggressive robot not allowed");
				}
			}
		}
	}

	//BOTS - Block Bad Bots
	if(Settings.Robots.DenyBad){
		if( DenySequences(RawUrl.Url,Settings.Robots.BadBotTraps,"BLOCKED: Bad robot fell for trap: '","'") ){
			BOT_Blocked.Add(ip,ua);
		}
	}

	if(!bRobotFile){
		//BOTS - Block if in list
		if(BOT_Blocked.Exists(ip,ua)){
			hack = true;
			logger.Append("BLOCKED: Robot not allowed until timeout expires");
		}

		//HEADERS: User Agent
		hack |= ScanUserAgent(ua);
	}

	return hack;
}

bool CWebKnightFilter::CustomSendRawData(CHttpFilterContext *pfc, PHTTP_FILTER_RAW_DATA pRawData, CString& Response)
{
	/*
	 * Change server header or remove it.
	 * Log only if an error occured, but deny nothing!
	 */ 
	bool doflush(false);
	//if we have to do something with server header
	if (Settings.Headers.RemoveServerHeader||Settings.Headers.ChangeServerHeader){

		if(ChangeRawDataHeader(Response,"Server:",Settings.Headers.ServerHeader,Settings.Headers.RemoveServerHeader)){
			char* p = (char*) pfc->AllocMem(Response.GetLength()+1);
			if(p!=NULL){
				strcpy(p,(LPCTSTR)Response);
				pRawData->pvInData = p;
				pRawData->cbInData = Response.GetLength();
				pRawData->cbInBuffer = Response.GetLength()+1;
				logger.Append("INFO: Changed/removed 'Server:' header");
			}else{
				logger.Append("WARNING: Error changing/removing 'Server:' header (could not allocate memory!)");
				doflush = true;
			}
		}else{
			logger.Append(Response); //Response contains error message if false returned
			doflush = true;
		}
	}
	return doflush;
}
