/*
	AQTRONIX WebKnight - ISAPI Filter for securing IIS
	Copyright 2002-2006 Parcifal Aertssen

	This file is part of AQTRONIX WebKnight.

    AQTRONIX WebKnight is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    AQTRONIX WebKnight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX WebKnight; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
// WebKnightSettings.cpp: implementation of the CWebKnightSettings class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Globals.h"
//#include "Logger.h"

#define MAX_INI_FILE_SIZE 50000

#include "WebKnightSettings.h"
#include "FileName.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWebKnightSettings::CWebKnightSettings()
{
	LoadDefaults(); //always load defaults (failover redundancy)
}

CWebKnightSettings::~CWebKnightSettings()
{

}

void CWebKnightSettings::LoadDefaults()
{
#ifdef IISBUILD
	LoadIISDefaults();
#endif
}

inline void CWebKnightSettings::LoadIISDefaults()
{
	/*
	 * Loads the default settings into memory
	 */

#ifdef PRIVATEBUILD
#pragma message("***** PRIVATE BUILD: This is a private personal build *****")
#endif

	//Scanning engine
	Engine.AllowLateScanning = false;			//high or low priority (true=low)
	Engine.ScanNonSecurePort = true;			//scan http requests
	Engine.ScanSecurePort = true;				//scan https requests

	//Scanning engine: excluded web instances. These are the web sites you
	//don't want to be scanned by this filter, because they are not normal
	//websites and use their own filter/extension to process requests.
	//Examples are Outlook Web Access, MS Proxy 2.0 (not ISA Server), SQL XML (only blocked if URL queries are enabled(SQL injection))
	Engine.ExcludedInstances.Enabled = true;
	Engine.ExcludedInstances.List.RemoveAll();
	Engine.ExcludedInstances.List.AddTail("100");	//Outlook Web Access - virtual HTTP server (custom)
	Engine.ExcludedInstances.List.AddTail("101");	//Outlook Web Access - virtual HTTP server (custom)
	Engine.ExcludedInstances.List.AddTail("102");	//Outlook Web Access - virtual HTTP server (custom)
	Engine.ExcludedInstances.List.AddTail("103");	//Outlook Web Access - virtual HTTP server (custom)
	Engine.ExcludedInstances.List.AddTail("104");	//Outlook Web Access - virtual HTTP server (custom)
	Engine.ExcludedInstances.List.AddTail("105");	//Outlook Web Access - virtual HTTP server (custom)
	//Engine.ExcludedInstances.List.AddTail("1");	//Default web site (ONLY if you are running MS Proxy 2.0 on the default instance and still want to protect other web instances)

	Engine.ExcludedIPAddresses.Enabled = false;
	Engine.ExcludedIPAddresses.List.RemoveAll();
#ifdef PRIVATEBUILD
	//Neos/Ogone payment system
	Engine.ExcludedIPAddresses.Enabled = true;
	Engine.ExcludedIPAddresses.List.AddTail("212.23.45.96-212.23.45.111");
	Engine.ExcludedIPAddresses.List.AddTail("213.254.248.96-213.254.248.127");
	Engine.ExcludedIPAddresses.List.AddTail("212.35.124.160-212.35.124.175");
#else
	Engine.ExcludedIPAddresses.List.AddTail("127.0.0.0/8");
	Engine.ExcludedIPAddresses.List.AddTail("10.0.0.0/8");
	Engine.ExcludedIPAddresses.List.AddTail("172.16.0.0/12");
	Engine.ExcludedIPAddresses.List.AddTail("192.168.0.0/16");
#endif

	//get the directory of the .dll file
	CFileName dll = GetProcessPath("WebKnight.dll");

	//Response handling
	HTTPResponse.Directly = true;			//write contents of denied.htm to client directly (without redirect)
	HTTPResponse.UseStatus = true;			//if ResponseRedirect=true, then this has no effect
	//ResponseStatus = "200 OK";		//to see the next step of the script kiddie (for testing only)
	HTTPResponse.Status = "999 No Hacking";	//"31337 No Hacking";//"404 Object Not Found"; //maybe an invalid response will crash script kiddiots tools
	HTTPResponse.Redirect = false;			//redirect to a custom url
	HTTPResponse.RedirectURL = "/denied.htm";
	HTTPResponse.DropConnection = true;
	Response.LogOnly = false;			//if true, log only (debug function)

	HTTPResponse.MonitorIP = true;
	HTTPResponse.MonitorIPTimeout = 6;
	HTTPResponse.BlockIP = false;
	HTTPResponse.BlockIPTimeout = 36;

	//Reponse contents
	HTTPResponse.Contents = "<HTML><HEAD><TITLE>WebKnight Application Firewall Alert</TITLE></HEAD><BODY><H1>WebKnight Application Firewall Alert</H1></BODY></HTML>";
	if(!CFileName::ReadFileToString(dll.Path + "denied.htm",HTTPResponse.Contents)){
		CFileName::ReadFileToString(dll.Path = "nohack.htm",HTTPResponse.Contents);	//backward compatibility
	}

	//logging
	Logging.Enabled = true;

#ifdef SYSTEM_LOG_BUILD
	LogDirDefault = "%WINDIR%\\System32\\LogFiles\\WebKnight\\";
#else
	LogDirDefault = "<Path of WebKnight>\\LogFiles\\";
#endif

#ifdef ENABLE_ODBC
	Logging.LogODBC = false;
	Logging.ODBCTable = "LogWebKnight";
	Logging.ODBCDSN = "DSN=mydsn;DATABASE=mydatabase;UID=;PWD=;";
#endif

	Logging.UseGMT = true;
	Logging.PerProcess = false;			//unique logdir per process (for hosting with multiple instances of the webserver)
	Logging.Retention = 28;				//number of days to keep log files
#ifdef PRIVATEBUILD
	Logging.Retention = 3000;
#endif
	Logging.LogAllowed = false;				//log requests which are allowed
	Logging.LogClientIP = true;
	Logging.LogUserName = true;
	HTTPLogging.Log_HTTP_VIA = false;			//HTTP VIA headers (used in TRACE)
	HTTPLogging.Log_HTTP_X_FORWARDED_FOR = false;//certain ISPs use a proxy which add this header (one of those ISPs is telenet.be)
	HTTPLogging.LogHostHeader = true;
	HTTPLogging.LogUserAgent = false;			//not essential information except maybe to identify script kiddie tools
	HTTPLogging.LogClientErrors = true;			//http 404 errors, etc.
	HTTPLogging.LogServerErrors = true;			//http 500 errors...

	//Connection control
	Connection.MonitorAddresses.Enabled = false;
	Connection.DenyAddresses.Enabled = false;
	Connection.MonitorAddresses.List.RemoveAll();
	Connection.DenyAddresses.List.RemoveAll();
	Connection.RequestsLimit.Enabled = false;
	Connection.RequestsLimit.MaxCount = 400;
	Connection.RequestsLimit.MaxTime = 2;
#ifdef PRIVATEBUILD
	Connection.RequestsLimit.Enabled = true;
	Connection.MonitorAddresses.Enabled = true;
	Connection.MonitorAddresses.List.AddTail("87.64.236.231");//bad requests
	Connection.MonitorAddresses.List.AddTail("194.150.224.102");//CoWorks
	Connection.MonitorAddresses.List.AddTail("82.146.99.119");//CoWorks
	//Connection.MonitorAddresses.List.AddTail("38.*.*.*");		//Spammers (with 'Referer' header)
	Connection.MonitorAddresses.List.AddTail("69.28.128.0/18");	//QuepasaCreep crawler (spammers)
	//Connection.MonitorAddresses.List.AddTail("72.14.192.0/18");	//Google research with FireFox/IE - fell for bad bot trap!
	//Connection.MonitorAddresses.List.AddTail("66.249.84.*");	//Google AdSense team 66.249.84.15 researched case about invalid clicks due to bad referrer
	Connection.MonitorAddresses.List.AddTail("207.46.*.*");		//Microsoft Global (207.46.228.16 surfed to www.aqtronix.com on 15 Jan 2004)
	Connection.MonitorAddresses.List.AddTail("131.107.0.0/16");	//Microsoft (e-mail)
	Connection.MonitorAddresses.List.AddTail("157.54.0.0/15");	//Microsoft (e-mail)
	Connection.MonitorAddresses.List.AddTail("157.56.0.0/14");	//Microsoft (e-mail)
	Connection.MonitorAddresses.List.AddTail("157.60.0.0/16");	//Microsoft (e-mail)
	Connection.MonitorAddresses.List.AddTail("72.44.32.0/19");	//Amazon.com
#endif
	
	//Authentication
	Authentication.ScanExcludedInstances = true;
	Authentication.DenyBlankPasswords = true;
	Authentication.DenySamePasswordAsUsername = true;
	Authentication.DenySystemAccounts = true;
	Authentication.DenyBruteForceAttack.Enabled = true;
	Authentication.DenyBruteForceAttack.MaxCount = 5;
	Authentication.DenyBruteForceAttack.MaxTime = 30;
	Authentication.ScanAccountAllEvents = true;

	//AUTH - Default passwords
	Authentication.DenyDefaultPasswords.Enabled = true;
	LoadDefaultsAuthPasswords(Authentication.DenyDefaultPasswords.List);
	
	//AUTH - Allowed accounts
	Authentication.AllowAccounts.Enabled = false;
	Authentication.AllowAccounts.List.RemoveAll();
	Authentication.AllowAccounts.List.AddTail("Administrateur");
	Authentication.AllowAccounts.List.AddTail("Administrator");
	Authentication.AllowAccounts.List.AddTail("Beheerder");

	//AUTH - Denied accounts
#ifdef PRIVATEBUILD
	Authentication.DenyAccounts.Enabled = true;
#else
	Authentication.DenyAccounts.Enabled = false;
#endif
	LoadDefaultsAuthAccounts(Authentication.DenyAccounts.List);

	//Limits
	Limits.LimitContentLength = true;
	Limits.LimitURL = true;
	Limits.LimitQueryString = true;
	Limits.LimitHTTPVersion = true;
	Limits.MaxContentLength="5300642";//(see bugtraq: Content-Length DoS 5300643)//"30000000";//max value of content length header (to prevent overflow, it cannot be longer than 9 characters)
	Limits.MaxURL = 1024; //16384;		//max length of url (MAX_PATH=260)
	Limits.MaxQueryString=1024;//4096;	//max length of querystring
	Limits.MaxHTTPVersion=22;				//max length of version string, in the form of HTTP/<major>.<minor> (e.g. HTTP/1.1) (HTTP/0.9 bug? buffer overflow? TODO)
	Limits.AllowHTTPVersions.Enabled = true;
	LoadDefaultsHTTPVersions(Limits.AllowHTTPVersions.List);

	Limits.MaxHeaders.Enabled = true;
	Limits.MaxHeaders.Map.SetAt("Accept:","1024");				//rfc & filters REQUEST
	Limits.MaxHeaders.Map.SetAt("Accept-Charset:","1024");		//rfc			REQUEST
	Limits.MaxHeaders.Map.SetAt("Accept-Encoding:","256");		//rfc & filters REQUEST
	Limits.MaxHeaders.Map.SetAt("Accept-Language:","356");		//rfc & filters REQUEST (Accept language of Netscape/mozilla on Mac is 356 long)
	Limits.MaxHeaders.Map.SetAt("Accept-Ranges:","128");		//rfc			REQUEST/RESPONSE
	//Limits.MaxHeaders.Map.SetAt("Age:","32");				//rfc					RESPONSE
	Limits.MaxHeaders.Map.SetAt("Allow:","512");				//rfc			REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Authorization:","4000");		//rfc & filters REQUEST
	//Limits.MaxHeaders.Map.SetAt("Authentication-Info:","4000");//rfc2069				RESPONSE
	Limits.MaxHeaders.Map.SetAt("BITS-Packet-Type:","64");				//BITS upload REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("BITS-Protocol:","1024");				//BITS upload		  RESPONSE
	Limits.MaxHeaders.Map.SetAt("BITS-Session-Id:","1024");				//BITS upload REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("BITS-Supported-Protocols:","4095");	//BITS upload REQUEST
	//Limits.MaxHeaders.Map.SetAt("BITS-Host-Id:","256");				//BITS upload		  RESPONSE
	//Limits.MaxHeaders.Map.SetAt("BITS-Host-Id-Fallback-Timeout:","20");//BITS upload		  RESPONSE
	//Limits.MaxHeaders.Map.SetAt("BITS-Received-Content-Range:","20");	//BITS upload		  RESPONSE
	//Limits.MaxHeaders.Map.SetAt("BITS-Reply-URL:","1024");				//BITS upload		  RESPONSE
	//Limits.MaxHeaders.Map.SetAt("BITS-Error-Code:","64");				//BITS upload		  RESPONSE
	//Limits.MaxHeaders.Map.SetAt("BITS-Error-Context:","64");			//BITS upload		  RESPONSE
	Limits.MaxHeaders.Map.SetAt("BITS-Original-Request-URL:","1024");		//BITS Notification REQUEST
	Limits.MaxHeaders.Map.SetAt("BITS-Request-DataFile-Name:","1024");	//BITS Notification REQUEST
	Limits.MaxHeaders.Map.SetAt("BITS-Response-DataFile-Name:","1024");	//BITS Notification REQUEST
	//Limits.MaxHeaders.Map.SetAt("BITS-Static-Response-URL:","1024");		//BITS Notification			RESPONSE
	//Limits.MaxHeaders.Map.SetAt("BITS-Copy-File-To-Destination:","64");	//BITS Notification			RESPONSE
	Limits.MaxHeaders.Map.SetAt("Cache-Control:","256");		//rfc			REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Charge-To:","512");			//w3c			REQUEST
	Limits.MaxHeaders.Map.SetAt("Connection:","256");			//rfc & filters REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Content-Base:","1024");		//rfc MIME		REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Content-Description:","1000");//rfc MIME		REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Content-Disposition:","256");	//rfc MIME		REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Content-Encoding:","128");	//rfc			REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Content-Id:","1024");			//rfc MIME		REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Content-Language:","256");	//rfc			REQUEST/RESPONSE
	//Limits.MaxHeaders.Map.SetAt("Content-Length:","9");		//rfc - is checked elsewhere
	Limits.MaxHeaders.Map.SetAt("Content-Location:","1024");	//rfc			REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Content-MD5:","4000");		//rfc			REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Content-Name:","512");					//BITS REQUEST
	Limits.MaxHeaders.Map.SetAt("Content-Range:","256");		//rfc			REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Content-Transfer-Encoding:","128");//winhttp.dll	NOT ALLOWED IN HTTP
	Limits.MaxHeaders.Map.SetAt("Content-Type:","256");		//rfc			REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Cookie:","4095");				//winhttp & filters
	//Limits.MaxHeaders.Map.SetAt("Cost:","256");				//winhttp.dll			RESPONSE
	Limits.MaxHeaders.Map.SetAt("Date:","64");					//rfc			REQUEST/RESPONSE
	//Limits.MaxHeaders.Map.SetAt("DAV:","128");				//rfc 2518(DAV)			RESPONSE	(not in winhttp.dll)
	Limits.MaxHeaders.Map.SetAt("Depth:","64");				//rfc 2518(DAV)	REQUEST				(not in winhttp.dll)
	Limits.MaxHeaders.Map.SetAt("Derived-From:","9000");		//winhttp.dll	REQUEST/RESPONSE?
	Limits.MaxHeaders.Map.SetAt("Destination:","1024");		//rfc 2518(DAV)	REQUEST				(not in winhttp.dll)
	//Limits.MaxHeaders.Map.SetAt("Etag:","4000");				//rfc					RESPONSE
	Limits.MaxHeaders.Map.SetAt("Expect:","256");				//rfc			REQUEST
	Limits.MaxHeaders.Map.SetAt("Expires:","64");				//rfc			REQUEST?/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Forwarded:","9000");			//winhttp.dll	REQUEST/RESPONSE?
	Limits.MaxHeaders.Map.SetAt("From:","321");				//rfc			REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Host:","256");				//rfc & filters REQUEST
	Limits.MaxHeaders.Map.SetAt("If:","4095");					//rfc 2518(DAV)	REQUEST
	Limits.MaxHeaders.Map.SetAt("If-Match:","256");			//rfc			REQUEST
	Limits.MaxHeaders.Map.SetAt("If-Modified-Since:","64");	//rfc & filters REQUEST
	Limits.MaxHeaders.Map.SetAt("If-None-Match:","256");		//rfc & filters REQUEST
	Limits.MaxHeaders.Map.SetAt("If-Range:","4000");			//rfc			REQUEST
	Limits.MaxHeaders.Map.SetAt("If-Unmodified-Since:","64");	//rfc			REQUEST
	Limits.MaxHeaders.Map.SetAt("Last-Modified:","64");		//rfc			REQUEST
	//Limits.MaxHeaders.Map.SetAt("Link:","1024");				//winhttp.dll	        RESPONSE
	//Limits.MaxHeaders.Map.SetAt("Location:","1024");			//rfc					RESPONSE
	Limits.MaxHeaders.Map.SetAt("Lock-Token:","1024");			//rfc 2518(DAV)	REQUEST
	Limits.MaxHeaders.Map.SetAt("Max-Forwards:","32");			//rfc			REQUEST
	Limits.MaxHeaders.Map.SetAt("Message-id:","1024");			//winhttp.dll	REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Mime-Version:","128");		//winhttp.dll	REQUEST?/RESPONSE? (winhttp.dll contains 1 header more than wininet.dll: "Mime-Version:")
	Limits.MaxHeaders.Map.SetAt("Ms-Echo-Reply:","128");		//winhttp.dll	REQUEST?/RESPONSE?
	Limits.MaxHeaders.Map.SetAt("Ms-Echo-Request:","128");		//winhttp.dll	REQUEST?/RESPONSE?
	Limits.MaxHeaders.Map.SetAt("Orig-Uri:","1024");			//winhttp.dll	REQUEST
	Limits.MaxHeaders.Map.SetAt("Overwrite:","64");			//rfc 2518(DAV)	REQUEST			(not in winhttp.dll)
	Limits.MaxHeaders.Map.SetAt("PassportConfig:","4000");		//winhttp.dll		TO DO: in request & value?
	Limits.MaxHeaders.Map.SetAt("PassportURLs:","4000");		//winhttp.dll		TO DO: in request & value?
	Limits.MaxHeaders.Map.SetAt("Pragma:","256");				//rfc			REQUEST/RESPONSE
	//Limits.MaxHeaders.Map.SetAt("Proxy-Authenticate:","256");//rfc					RESPONSE
	Limits.MaxHeaders.Map.SetAt("Proxy-Authorization:","4000");//rfc			REQUEST
	Limits.MaxHeaders.Map.SetAt("Proxy-Connection:","256");	//winhttp.dll	REQUEST/RESPONSE?
	Limits.MaxHeaders.Map.SetAt("Proxy-Support:","4000");		//winhttp.dll		TO DO: in request & value?
	Limits.MaxHeaders.Map.SetAt("Public:","4000");				//winhttp.dll		TO DO: in request & value?
	Limits.MaxHeaders.Map.SetAt("Range:","256");				//rfc			REQUEST/RESPONSE?
	Limits.MaxHeaders.Map.SetAt("Referer:","1024");			//rfc & filters REQUEST
	//Limits.MaxHeaders.Map.SetAt("Refresh:","64");			//winhttp.dll			RESPONSE
	//Limits.MaxHeaders.Map.SetAt("Retry-After:","64");		//rfc					RESPONSE
	//Limits.MaxHeaders.Map.SetAt("Server:","256");			//rfc					RESPONSE
	//Limits.MaxHeaders.Map.SetAt("Set-Cookie:","4095");		//winhttp.dll			RESPONSE
	Limits.MaxHeaders.Map.SetAt("SOAPAction:","4095");		//SOAP				REQUEST
	//Limits.MaxHeaders.Map.SetAt("Status-URI:","1024");		//rfc 2518(DAV)			RESPONSE (not in winhttp.dll)
	Limits.MaxHeaders.Map.SetAt("Timeout:","1024");			//rfc 2518(DAV)	REQUEST			 (not in winhttp.dll)
	Limits.MaxHeaders.Map.SetAt("Title:","9000");				//winhttp.dll	REQUEST?/RESPONSE
	Limits.MaxHeaders.Map.SetAt("TE:","128");					//rfc only!		REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Trailer:","128");				//rfc only!		REQUEST?/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Transfer-Encoding:","128");	//rfc & filters REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Translate:","256");			//winhttp.dll	REQUEST
	Limits.MaxHeaders.Map.SetAt("Unless-Modified-Since:","64");//winhttp.dll	REQUEST
	Limits.MaxHeaders.Map.SetAt("Upgrade:","256");				//rfc			REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Uri:","1024");				//winhttp.dll	REQUEST/RESPONSE
	Limits.MaxHeaders.Map.SetAt("User-Agent:","256");			//rfc & filters	REQUEST
	Limits.MaxHeaders.Map.SetAt("Vary:","128");				//rfc			REQUEST?/RESPONSE
	Limits.MaxHeaders.Map.SetAt("Via:","9000");				//rfc			REQUEST/RESPONSE
	//Limits.MaxHeaders.Map.SetAt("Warning:","4000");			//rfc					RESPONSE
	Limits.MaxHeaders.Map.SetAt("Version:","128");				//www.w3.org	REQUEST/RESPONSE
	//Limits.MaxHeaders.Map.SetAt("WWW-Authenticate:","256");	//rfc					RESPONSE

	//URL
	URL.DenyEncodingExploits = true;
	URL.DenyParentPath = true;			//no ".." in url
	URL.DenyTrailingDotInDir = true;	//no "./" in url
	URL.DenyBackslash = true;			//no '\' in url
	URL.DenyAlternateStream = false;	//you should allow ':' in url (see rfc 2616), urlscan blocks it, which is wrong!!!
	URL.DenyEscaping = true;			//no '%' in url (after url normalization)
	URL.DenyMultipleCGI = true;		//no '&' in url
	URL.DenyCharacters = "?#";			//other characters to block
	URL.DenyHighBitShellcode = true;	//deny high bit shellcode (ascii>127)

	//URL: Deny certain sequences (must be lowercase)
	URL.DenySequences.Enabled = true;
	URL.DenySequences.List.RemoveAll();
	URL.DenySequences.List.AddTail("/scripts");		//scripts (needed by MS Proxy 2.0, frontpage - not by ISA Server)
	URL.DenySequences.List.AddTail("/iishelp");		//iishelp
	URL.DenySequences.List.AddTail("/iisadmin");	//iisadmin (remote administration)
	URL.DenySequences.List.AddTail("/msadc");		//Microsoft Advanced Data Connector = RDS (Remote Data Service)
	URL.DenySequences.List.AddTail("/printers");	//installed printers
	URL.DenySequences.List.AddTail("/samples");		//just samples
	URL.DenySequences.List.AddTail("/iisadmpwd");	//change windows password
	URL.DenySequences.List.AddTail("/_vti_aut");	//frontpage extensions
	URL.DenySequences.List.AddTail("/_vti_bin");	//frontpage extensions
	URL.DenySequences.List.AddTail("/_vti_rpc");	//frontpage extensions
	URL.DenySequences.List.AddTail("/_vti_pvt");	//frontpage extensions
	URL.DenySequences.List.AddTail("/admcgi");		//frontpage extensions
	URL.DenySequences.List.AddTail("/admisapi");	//frontpage extensions
	URL.DenySequences.List.AddTail("/_private");	//frontpage extensions
	URL.DenySequences.List.AddTail("/_vti_cnf");	//frontpage extensions
	URL.DenySequences.List.AddTail("/_vti_log");	//frontpage extensions
	URL.DenySequences.List.AddTail("/_vti_script");//frontpage extensions
	URL.DenySequences.List.AddTail("/_vti_txt");	//frontpage extensions
	URL.DenySequences.List.AddTail("/_mem_bin");	//frontpage extensions
	URL.DenySequences.List.AddTail("/ftp only");	//ftp only :)
	URL.DenySequences.List.AddTail("/ftp_only");	//ftp only :)
	URL.DenySequences.List.AddTail("/ftp-only");	//ftp only :)
	//URL.DenySequences.List.AddTail("/winnt");		//operating system sensitive dirs
	//URL.DenySequences.List.AddTail("/system");
	URL.DenySequences.List.AddTail("/system32");
	URL.DenySequences.List.AddTail("/adsamples");
	URL.DenySequences.List.AddTail("/pbserver");
	URL.DenySequences.List.AddTail("/rpc");
	URL.DenySequences.List.AddTail("/cfdocs");				//coldfusion docs
	URL.DenySequences.List.AddTail("/cfdocs/exampleapp");	//coldfusion samples
	URL.DenySequences.List.AddTail("/cfdocs/snippets");		//coldfusion samples
	URL.DenySequences.List.AddTail("/cfdocs/expeval");		//coldfusion
	URL.DenySequences.List.AddTail("/cfdocs/examples");		//coldfusion samples
	URL.DenySequences.List.AddTail("/cfappman");			//coldfusion samples
	URL.DenySequences.List.AddTail("/cfide/administrator");//coldfusion
	URL.DenySequences.List.AddTail("/siteserver");
	URL.DenySequences.List.AddTail("/advworks");
	URL.DenySequences.List.AddTail("/ssi/envout.bat");
	URL.DenySequences.List.AddTail("/cgi-bin");		//CGI
	URL.DenySequences.List.AddTail("/cgi-local");	//CGI
	URL.DenySequences.List.AddTail("/cgi-win");		//CGI
	URL.DenySequences.List.AddTail("/htbin");		//CGI
	URL.DenySequences.List.AddTail("/cgibin");		//CGI
	URL.DenySequences.List.AddTail("/cgis");		//CGI
	URL.DenySequences.List.AddTail("/cgi");			//CGI
	URL.DenySequences.List.AddTail("/test-cgi");	//CGI
	URL.DenySequences.List.AddTail("/ows-bin/");
	URL.DenySequences.List.AddTail("/bin/");
	URL.DenySequences.List.AddTail("/sbin/");		
	URL.DenySequences.List.AddTail("/etc/");
	URL.DenySequences.List.AddTail("/database/");	//database
	URL.DenySequences.List.AddTail("/databases/");	//database
	URL.DenySequences.List.AddTail("/dbase/");		//database
	URL.DenySequences.List.AddTail("/db/");			//database
	URL.DenySequences.List.AddTail("/storedb/");	//database
	URL.DenySequences.List.AddTail("/fpdb/");		//frontpage extensions (database)
	URL.DenySequences.List.AddTail("/log/");		//logs
	URL.DenySequences.List.AddTail("/logs/");
	URL.DenySequences.List.AddTail("/logfile/");
	URL.DenySequences.List.AddTail("/logfiles/");
	URL.DenySequences.List.AddTail("/logger/");
	URL.DenySequences.List.AddTail("/server_stats/");
	URL.DenySequences.List.AddTail("/trafficlog/");
	URL.DenySequences.List.AddTail("/weblog/");
	URL.DenySequences.List.AddTail("/weblogs/");
	URL.DenySequences.List.AddTail("/webstats/");
	URL.DenySequences.List.AddTail("/wstats/");
	URL.DenySequences.List.AddTail("/wusage/");
	URL.DenySequences.List.AddTail("/wwwlog/");
	URL.DenySequences.List.AddTail("/wwwstats/");
	URL.DenySequences.List.AddTail("/mall_log_files/");
	URL.DenySequences.List.AddTail("/admin");		//admin stuff
	URL.DenySequences.List.AddTail("/admin_/");
	URL.DenySequences.List.AddTail("/_admin/");
	URL.DenySequences.List.AddTail("/srchadm/");
	URL.DenySequences.List.AddTail("/admnlogin");
	URL.DenySequences.List.AddTail("/adminlogin");
	URL.DenySequences.List.AddTail("/siteadmin");
	URL.DenySequences.List.AddTail("/w3perl/admin");
	URL.DenySequences.List.AddTail("/webaccess/");
	URL.DenySequences.List.AddTail("/account/");
	URL.DenySequences.List.AddTail("/administrator/");
	URL.DenySequences.List.AddTail("/config/");
	URL.DenySequences.List.AddTail("/fpadmin/");
	URL.DenySequences.List.AddTail("/srchadm/");
	URL.DenySequences.List.AddTail("/admin_files");
	URL.DenySequences.List.AddTail("/passwords/");
	URL.DenySequences.List.AddTail("/etc/passwd");
	URL.DenySequences.List.AddTail("/exchange");	//Outlook Web Access on Default web site
	URL.DenySequences.List.AddTail("/exchweb");		//Outlook Web Access on Default web site
	URL.DenySequences.List.AddTail("/public");		//Outlook Web Access (Public folders)
	URL.DenySequences.List.AddTail("/exadmin");		//Outlook Web Access Administration
	URL.DenySequences.List.AddTail("/hit_tracker/");//counter stuff
	URL.DenySequences.List.AddTail("/hitmatic/");
	URL.DenySequences.List.AddTail("/counter/");
	URL.DenySequences.List.AddTail("/c/");			//other things
	URL.DenySequences.List.AddTail("/d/");
	URL.DenySequences.List.AddTail("/doc-html/");
	URL.DenySequences.List.AddTail("/ftp/");
	URL.DenySequences.List.AddTail("/htdocs/");
	URL.DenySequences.List.AddTail("/install/");
	URL.DenySequences.List.AddTail("/intranet/");
	URL.DenySequences.List.AddTail("/jdbc/");
	URL.DenySequences.List.AddTail("/msql/");
	URL.DenySequences.List.AddTail("/odbc/");
	URL.DenySequences.List.AddTail("/oracle/");
	URL.DenySequences.List.AddTail("/private/");
	URL.DenySequences.List.AddTail("/sql/");
	URL.DenySequences.List.AddTail("/temp/");		//temp & test stuff
	URL.DenySequences.List.AddTail("/tmp/");
	URL.DenySequences.List.AddTail("/test/");
	URL.DenySequences.List.AddTail("/webdriver");
	URL.DenySequences.List.AddTail(" /http/1.");
	URL.DenySequences.List.AddTail("/glimpse");
	URL.DenySequences.List.AddTail("/aglimpse");
	URL.DenySequences.List.AddTail("/htmlscript");
	URL.DenySequences.List.AddTail("/info2www");
	URL.DenySequences.List.AddTail("/nph-test-cgi");
	URL.DenySequences.List.AddTail("/nph-publish");
	URL.DenySequences.List.AddTail("/view-source");
	URL.DenySequences.List.AddTail("/w3-msql");
	URL.DenySequences.List.AddTail("/www-sql");
	URL.DenySequences.List.AddTail("/level/*/exec/");
	URL.DenySequences.List.AddTail("/nessus_is_probing_you_");
	URL.DenySequences.List.AddTail("/~root");
	URL.DenySequences.List.AddTail("/handler");
	URL.DenySequences.List.AddTail("/backup");
	URL.DenySequences.List.AddTail("/filemail");
	URL.DenySequences.List.AddTail("/plusmail");
	URL.DenySequences.List.AddTail("/ultraboard");
	URL.DenySequences.List.AddTail("/empower");
	URL.DenySequences.List.AddTail("/pals-cgi");
	URL.DenySequences.List.AddTail("/htgrep");
	URL.DenySequences.List.AddTail("/.nsconfig");
	URL.DenySequences.List.AddTail("/catinfo");
	URL.DenySequences.List.AddTail("/sweditservlet");
	URL.DenySequences.List.AddTail("/cybercop");
	URL.DenySequences.List.AddTail("/webcart/");
	URL.DenySequences.List.AddTail("/rmp_query");
	URL.DenySequences.List.AddTail("/rpm_query");
	URL.DenySequences.List.AddTail("/servlet/servletexec");
	URL.DenySequences.List.AddTail("/admin-serv/config/admpw");
	URL.DenySequences.List.AddTail("/c32web.exe/changeadminpassword");
	URL.DenySequences.List.AddTail("/graphics/sml3com");
	URL.DenySequences.List.AddTail("/jsp/snp/*.snp");
	URL.DenySequences.List.AddTail("_authchangeurl");
	URL.DenySequences.List.AddTail("/dbadmin");
	URL.DenySequences.List.AddTail("/myadmin");
	URL.DenySequences.List.AddTail("/mysqladmin");
	URL.DenySequences.List.AddTail("/mysql-admin");
	URL.DenySequences.List.AddTail("/phpadmin");
	URL.DenySequences.List.AddTail("/phpmyadmin");
	URL.DenySequences.List.AddTail("slurpconfirm404");
	URL.DenySequences.List.AddTail("/thisdoesnotexistahaha.");

	//URL: Only allow certain first characters of a URL (must be lowercase)
	URL.AllowURL.Enabled = true;
	URL.AllowURL.List.RemoveAll();
	URL.AllowURL.List.AddTail("/");		//allow all relative URLs (http 1.0)
	URL.AllowURL.List.AddTail("http://");	//absolute URLs (http 1.1 compliant webserver or proxy server)
	URL.AllowURL.List.AddTail("https://"); //https absolute URL
	//URL.AllowURL.List.AddTail("ftp://");		//ftp request on http port (allow proxy? network request is not supported? can be exploited? TODO: check this)
	//URL.AllowURL.List.AddTail("gopher://");	//gopher request on http port (allow proxy?)
	
	URL.RFCCompliantURL = true;
	URL.RFCCompliantHTTPURL = true;
	URL.UseRawScan = true;			//Use URL raw scanning
	
	//FULLPATH
	Path.DenyParentPath = true;
	Path.DenyBackspace = true;			//new hack?
	Path.DenyCarriageReturn = true;	//new hack?
	Path.DenyNewLine = true;			//new hack?
	Path.DenyEscaping = true;			//if true, you cannot use '%' in a filename
	Path.DenyCharacters = "*?\"<>|$^#+=";	//a windows filename cannot contain "\/:*?"<>|" and block "$^#+=" as well

	//PATH
	Path.DenyDot = false; //deny dot in path

	//PATH: allowed paths (must be lowercase)
	//      NO ending '\' otherwise problems with virtual dirs (if those virtual dirs
	//      are mapped to a physical directory with no ending '\'). For best practices
	//      I suggest you add a trailing '\' for each virtual dir in IIS (this tightens
	//      security). Once you have done that, you can add a trailing '\' in here too.
	//      I wonder if you can exploit this? (TODO)
	Path.AllowPaths.Enabled = true;
	Path.AllowPaths.List.RemoveAll();
	Path.AllowPaths.List.AddTail("c:\\inetpub\\wwwroot");			//default web site & wwwroot2 ...
	Path.AllowPaths.List.AddTail("c:\\inetpub\\scripts");			//Scripts (/scripts)
	Path.AllowPaths.List.AddTail("c:\\winnt\\help\\iishelp");		//IISHelp (/iishelp)
	Path.AllowPaths.List.AddTail("c:\\inetpub\\iissamples");			//IIS samples
	Path.AllowPaths.List.AddTail("c:\\winnt\\web\\printers");		//Printers (/printers)
	Path.AllowPaths.List.AddTail("c:\\winnt\\system32\\inetsrv\\iisadmin");//IIS Administration Website & /iisadmin (individual site administration)
	//Path.AllowPaths.List.AddTail("c:\\winnt\\system32\\inetsrv\\iisadmpwd");//IIS change password interface (/iisadmpwd)
	Path.AllowPaths.List.AddTail("c:\\winnt\\system32\\certsrv");	//Certificate Services
	Path.AllowPaths.List.AddTail("c:\\program files\\common files\\system\\msadc"); //MSADC (/msadc) & 
	Path.AllowPaths.List.AddTail("c:\\program files\\common files\\microsoft shared\\web server extensions"); //Web Server Extensions (FrontPage) (/_vti_bin)
	Path.AllowPaths.List.AddTail("m:\\");							//Outlook Web Access
	Path.AllowPaths.List.AddTail("c:\\program files\\exchsrvr\\exchweb");//Outlook Web Access
	Path.AllowPaths.List.AddTail("\\\\.\\backofficestorage");		//Exchange admin ("/ExAdmin")
	Path.AllowPaths.List.AddTail("c:\\program files");				//allow all applications to add functionality via the web server
	Path.AllowPaths.List.AddTail("d:\\www");							//custom web site
	Path.AllowPaths.List.AddTail("d:\\web");							//custom web site
	Path.AllowPaths.List.AddTail("e:\\www");							//custom web site
	Path.AllowPaths.List.AddTail("e:\\web");							//custom web site
	Path.AllowPaths.List.AddTail("f:\\www");							//custom web site
	Path.AllowPaths.List.AddTail("f:\\web");							//custom web site

	//FILENAME
	Filename.UseRawScan = true;
	Filename.DenyCharacters = "\\:/*?\"<>|$^#+="; //a windows filename cannot contain "\/:*?"<>|" and block "$^#+=" as well
	Filename.DenyDefaultDocument = false;

	//FILENAME: Deny almost all Windows 2000 commands (must be lowercase)
	Filename.DenyFiles.Enabled = true;
	Filename.DenyFiles.List.RemoveAll();
	Filename.DenyFiles.List.AddTail("arp.exe");
	Filename.DenyFiles.List.AddTail("at.exe");
	Filename.DenyFiles.List.AddTail("atmadm.exe");
	Filename.DenyFiles.List.AddTail("attrib.exe");
	Filename.DenyFiles.List.AddTail("cacls.exe");
	Filename.DenyFiles.List.AddTail("chkdsk.exe");
	Filename.DenyFiles.List.AddTail("chkntfs.exe");
	Filename.DenyFiles.List.AddTail("cipher.exe");
	Filename.DenyFiles.List.AddTail("cluster.exe");
	Filename.DenyFiles.List.AddTail("cmd.exe");
	Filename.DenyFiles.List.AddTail("comp.exe");
	Filename.DenyFiles.List.AddTail("date.exe");
	Filename.DenyFiles.List.AddTail("debug.exe");
	Filename.DenyFiles.List.AddTail("diskcomp.com");
	Filename.DenyFiles.List.AddTail("diskcopy.com");
	Filename.DenyFiles.List.AddTail("diskperf.exe");
	Filename.DenyFiles.List.AddTail("doskey.exe");
	Filename.DenyFiles.List.AddTail("edlin.exe");
	Filename.DenyFiles.List.AddTail("exe2bin.exe");
	Filename.DenyFiles.List.AddTail("expand.exe");
	Filename.DenyFiles.List.AddTail("fc.exe");
	Filename.DenyFiles.List.AddTail("find.exe");
	Filename.DenyFiles.List.AddTail("findstr.exe");
	Filename.DenyFiles.List.AddTail("forcedos.exe");
	Filename.DenyFiles.List.AddTail("format.exe");
	Filename.DenyFiles.List.AddTail("ftp.exe");
	Filename.DenyFiles.List.AddTail("graphics.com");
	Filename.DenyFiles.List.AddTail("hostname.exe");
	Filename.DenyFiles.List.AddTail("ipconfig.exe");
	Filename.DenyFiles.List.AddTail("ipxroute.exe");
	Filename.DenyFiles.List.AddTail("label.exe");
	Filename.DenyFiles.List.AddTail("loadfix.exe");
	Filename.DenyFiles.List.AddTail("mem.exe");
	Filename.DenyFiles.List.AddTail("mode.com");
	Filename.DenyFiles.List.AddTail("mountvol.exe");
	Filename.DenyFiles.List.AddTail("nbtstat.exe");
	Filename.DenyFiles.List.AddTail("net.exe");
	Filename.DenyFiles.List.AddTail("netsh.exe");
	Filename.DenyFiles.List.AddTail("netstat.exe");
	Filename.DenyFiles.List.AddTail("nslookup.exe");
	Filename.DenyFiles.List.AddTail("pathping.exe");
	Filename.DenyFiles.List.AddTail("ping.exe");
	Filename.DenyFiles.List.AddTail("rcp.exe");
	Filename.DenyFiles.List.AddTail("replace.exe");
	Filename.DenyFiles.List.AddTail("rexec.exe");
	Filename.DenyFiles.List.AddTail("route.exe");
	Filename.DenyFiles.List.AddTail("rsh.exe");
	Filename.DenyFiles.List.AddTail("runas.exe");
	Filename.DenyFiles.List.AddTail("rundll.exe");	//deny running dlls
	Filename.DenyFiles.List.AddTail("rundll32.exe");	//deny running dlls
	Filename.DenyFiles.List.AddTail("runonce.exe");
	Filename.DenyFiles.List.AddTail("setver.exe");
	Filename.DenyFiles.List.AddTail("subst.exe");
	Filename.DenyFiles.List.AddTail("tcmsetup.exe");
	Filename.DenyFiles.List.AddTail("telnet.exe"); //not in win2k help -> command reference!
	Filename.DenyFiles.List.AddTail("tftp.exe"); //is also blocked by scanning for ftp.exe
	Filename.DenyFiles.List.AddTail("time.exe");
	Filename.DenyFiles.List.AddTail("winnt.exe");
	Filename.DenyFiles.List.AddTail("winnt32.exe");
	Filename.DenyFiles.List.AddTail("xcopy.exe");
	//FILENAME: Deny common applications used in hacking/scanning
	Filename.DenyFiles.List.AddTail("command.com");
	Filename.DenyFiles.List.AddTail("cmd1.exe");
	Filename.DenyFiles.List.AddTail("root.exe");
	Filename.DenyFiles.List.AddTail("shell.exe");
	Filename.DenyFiles.List.AddTail("iisreset.exe");
	Filename.DenyFiles.List.AddTail("formmail.");		//.cgi & .pl
	Filename.DenyFiles.List.AddTail("mailform.");		//.cgi & .pl
	Filename.DenyFiles.List.AddTail("nc.exe");		//netcat
	Filename.DenyFiles.List.AddTail("netcat.exe");	//netcat
	Filename.DenyFiles.List.AddTail("sumthin");		//found in some script kiddie stuff
	//Filename.DenyFiles.List.AddTail("default.ida");
	Filename.DenyFiles.List.AddTail("whisker.ida");
	Filename.DenyFiles.List.AddTail("whisker.idc");
	Filename.DenyFiles.List.AddTail("whisker.idq");
	Filename.DenyFiles.List.AddTail("whisker.htw");
	Filename.DenyFiles.List.AddTail("whisker.htr");
	Filename.DenyFiles.List.AddTail("carbo.dll");
	Filename.DenyFiles.List.AddTail("ctguestb.idc");
	Filename.DenyFiles.List.AddTail("details.idc");
	Filename.DenyFiles.List.AddTail("w3proxy.dll");	//ms proxy access
	Filename.DenyFiles.List.AddTail("sam._");
	Filename.DenyFiles.List.AddTail("sensepost.exe");
	Filename.DenyFiles.List.AddTail("achg.htr");
	Filename.DenyFiles.List.AddTail("anot.htr");
	Filename.DenyFiles.List.AddTail("adctest.asp");
	Filename.DenyFiles.List.AddTail("ism.dll");
	Filename.DenyFiles.List.AddTail("bdir.htr");
	Filename.DenyFiles.List.AddTail("codebrws.asp");
	Filename.DenyFiles.List.AddTail("form_jscript.asp");
	Filename.DenyFiles.List.AddTail("form_vbscript.asp");
	Filename.DenyFiles.List.AddTail("servervariables_jscript.asp");
	Filename.DenyFiles.List.AddTail("fpcount.exe");
	Filename.DenyFiles.List.AddTail("_vti_inf.html");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("postinfo.html"); //frontpage extensions
	Filename.DenyFiles.List.AddTail("fp30reg.dll");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("fp4areg.dll");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("shtml.dll");		//frontpage extensions
	Filename.DenyFiles.List.AddTail("shtml.exe");		//frontpage extensions
	Filename.DenyFiles.List.AddTail("fpsrvadm.exe");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("fpremadm.exe");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("fpadmin.htm");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("fpadmcgi.exe");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("cfgwiz.exe");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("authors.pwd");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("author.exe");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("author.dll");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("administrators.pwd");//frontpage extensions
	Filename.DenyFiles.List.AddTail("access.cnf");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("service.cnf");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("service.pwd");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("service.stp");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("services.cnf");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("svcacl.cnf");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("users.pwd");		//frontpage extensions
	Filename.DenyFiles.List.AddTail("writeto.cnf");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("dvwssr.dll");	//frontpage extensions
	Filename.DenyFiles.List.AddTail("getdrvs.exe");
	Filename.DenyFiles.List.AddTail("global.asa");
	Filename.DenyFiles.List.AddTail("$data");
	Filename.DenyFiles.List.AddTail("msadcs.dll");
	Filename.DenyFiles.List.AddTail("newdsn.exe");
	Filename.DenyFiles.List.AddTail("search97.vts");
	Filename.DenyFiles.List.AddTail("viewcode.");
	Filename.DenyFiles.List.AddTail("showcode.");
	Filename.DenyFiles.List.AddTail("site.csc");
	Filename.DenyFiles.List.AddTail("srch.htm");
	Filename.DenyFiles.List.AddTail("uploadn.asp");
	Filename.DenyFiles.List.AddTail("logonfrm.asp");	//owa dos
	Filename.DenyFiles.List.AddTail("cgimail.exe");
	Filename.DenyFiles.List.AddTail("quickstore.cfg");
	Filename.DenyFiles.List.AddTail("bigconf.cgi");
	Filename.DenyFiles.List.AddTail("storemgr.pw");
	Filename.DenyFiles.List.AddTail("admin.pw");
	Filename.DenyFiles.List.AddTail("test.");
	Filename.DenyFiles.List.AddTail("password.php3");
	Filename.DenyFiles.List.AddTail("password.txt");
	Filename.DenyFiles.List.AddTail("passwd.txt");
	Filename.DenyFiles.List.AddTail("passwd.php");
	Filename.DenyFiles.List.AddTail("passwd.php3");
	Filename.DenyFiles.List.AddTail("submit.cgi");
	Filename.DenyFiles.List.AddTail("ss.cfg");
	Filename.DenyFiles.List.AddTail("ncl_items.html");
	Filename.DenyFiles.List.AddTail("stat_what.log");
	Filename.DenyFiles.List.AddTail("easylog.html");
	Filename.DenyFiles.List.AddTail("analyse.cgi");
	Filename.DenyFiles.List.AddTail("admin.cgi");
	Filename.DenyFiles.List.AddTail("admin.php");
	Filename.DenyFiles.List.AddTail("admin.pl");
	Filename.DenyFiles.List.AddTail("access-options.txt");
	Filename.DenyFiles.List.AddTail("access.log");
	Filename.DenyFiles.List.AddTail("access-log");
	Filename.DenyFiles.List.AddTail("awebvisit.stat");
	Filename.DenyFiles.List.AddTail("dan_o.dat");
	Filename.DenyFiles.List.AddTail("hits.txt");
	Filename.DenyFiles.List.AddTail("log.htm");
	Filename.DenyFiles.List.AddTail("log.html");
	Filename.DenyFiles.List.AddTail("logfile");
	Filename.DenyFiles.List.AddTail("logfile.htm");
	Filename.DenyFiles.List.AddTail("logfile.html");
	Filename.DenyFiles.List.AddTail("logfile.txt");
	Filename.DenyFiles.List.AddTail("logger.html");
	Filename.DenyFiles.List.AddTail("stat.htm");
	Filename.DenyFiles.List.AddTail("stats.htm");
	Filename.DenyFiles.List.AddTail("stats.html");
	Filename.DenyFiles.List.AddTail("stats.txt");
	Filename.DenyFiles.List.AddTail("webaccess.htm");
	Filename.DenyFiles.List.AddTail("whois_raw.cgi");
	Filename.DenyFiles.List.AddTail("localstart.asp");	//local start document
	Filename.DenyFiles.List.AddTail(".asa.");				//.asp. bug
	Filename.DenyFiles.List.AddTail(".asp.");				//.asp. bug
	Filename.DenyFiles.List.AddTail(".asax.");				//code behind access
	Filename.DenyFiles.List.AddTail(".aspx.");
	Filename.DenyFiles.List.AddTail("trace.axd");			//ASP.NET tracing
	Filename.DenyFiles.List.AddTail("webplus");
	Filename.DenyFiles.List.AddTail("websendmail");
	Filename.DenyFiles.List.AddTail("dcboard.cgi");
	Filename.DenyFiles.List.AddTail("dcforum.cgi");
	Filename.DenyFiles.List.AddTail("mmstdod.cgi");
	Filename.DenyFiles.List.AddTail("cvsweb.cgi");
	Filename.DenyFiles.List.AddTail("php.cgi");
	Filename.DenyFiles.List.AddTail("maillist.pl");
	Filename.DenyFiles.List.AddTail("perl.exe");
	Filename.DenyFiles.List.AddTail("rguest.exe");
	Filename.DenyFiles.List.AddTail("rwwwshell.pl");
	Filename.DenyFiles.List.AddTail("textcounter.pl");
	Filename.DenyFiles.List.AddTail("uploader.exe");
	Filename.DenyFiles.List.AddTail("webhits.exe");
	Filename.DenyFiles.List.AddTail("webgais");
	Filename.DenyFiles.List.AddTail("finger");
	Filename.DenyFiles.List.AddTail("perlshop.cgi");
	Filename.DenyFiles.List.AddTail("pfdisplay.cgi");
	Filename.DenyFiles.List.AddTail("args.bat");
	Filename.DenyFiles.List.AddTail("at-admin.cgi");
	Filename.DenyFiles.List.AddTail("bnbform.cgi");
	Filename.DenyFiles.List.AddTail("wais.pl");
	Filename.DenyFiles.List.AddTail("wguest.exe");
	Filename.DenyFiles.List.AddTail("classifieds.cgi");
	Filename.DenyFiles.List.AddTail("environ.pl");
	Filename.DenyFiles.List.AddTail("filemail.pl");
	Filename.DenyFiles.List.AddTail("man.sh");
	Filename.DenyFiles.List.AddTail("snork.bat");
	Filename.DenyFiles.List.AddTail("blat.exe");
	Filename.DenyFiles.List.AddTail("day5datacopier.cgi");
	Filename.DenyFiles.List.AddTail("day5datanotifier.cgi");
	Filename.DenyFiles.List.AddTail("hsx.cgi");
	Filename.DenyFiles.List.AddTail("s.cgi");
	Filename.DenyFiles.List.AddTail("yabb.cgi");
	Filename.DenyFiles.List.AddTail("post-query");
	Filename.DenyFiles.List.AddTail("visadmin.exe");
	Filename.DenyFiles.List.AddTail("dumpenv.pl");
	Filename.DenyFiles.List.AddTail("snorkerz.cmd");
	Filename.DenyFiles.List.AddTail("win-c-sample.exe");
	Filename.DenyFiles.List.AddTail("w3tvars.pm");
	Filename.DenyFiles.List.AddTail("lwgate");
	Filename.DenyFiles.List.AddTail("flexform");
	Filename.DenyFiles.List.AddTail("www-admin.pl");
	Filename.DenyFiles.List.AddTail("sendform.cgi");
	Filename.DenyFiles.List.AddTail("ppdscgi.exe");
	Filename.DenyFiles.List.AddTail("upload.pl");
	Filename.DenyFiles.List.AddTail("anyform2");
	Filename.DenyFiles.List.AddTail("machineinfo");
	Filename.DenyFiles.List.AddTail("bb-hist.sh");
	Filename.DenyFiles.List.AddTail("pals-cgi");
	Filename.DenyFiles.List.AddTail("webspirs.cgi");
	Filename.DenyFiles.List.AddTail("tstisapi.dll");
	Filename.DenyFiles.List.AddTail("testisapi.dll");
	Filename.DenyFiles.List.AddTail("sendmessage.cgi");
	Filename.DenyFiles.List.AddTail("lastlines.cgi");
	Filename.DenyFiles.List.AddTail("zml.cgi");
	Filename.DenyFiles.List.AddTail("ads.cgi");
	Filename.DenyFiles.List.AddTail("%00.jsp");
	Filename.DenyFiles.List.AddTail("postinfo.asp");
	Filename.DenyFiles.List.AddTail("repost.asp");
	Filename.DenyFiles.List.AddTail("queryhit.htm");
	Filename.DenyFiles.List.AddTail("counter.exe");
	Filename.DenyFiles.List.AddTail("cached_feed.cgi");
	Filename.DenyFiles.List.AddTail("shopping_cart.mdb");
	Filename.DenyFiles.List.AddTail("password.mdb");
	Filename.DenyFiles.List.AddTail("check.txt");
	Filename.DenyFiles.List.AddTail("checks.txt");
	Filename.DenyFiles.List.AddTail("mylog.phtml");
	Filename.DenyFiles.List.AddTail("mlog.phtml");
	Filename.DenyFiles.List.AddTail("convert.bas");
	Filename.DenyFiles.List.AddTail("cpshost.dll");
	Filename.DenyFiles.List.AddTail("smssend.php");
	Filename.DenyFiles.List.AddTail("txt2html.cgi");
	Filename.DenyFiles.List.AddTail("console.exe");
	Filename.DenyFiles.List.AddTail("sojourn.cgi");
	Filename.DenyFiles.List.AddTail("ping.exe");	//"ping" generates too much false positives: "shopping"
	Filename.DenyFiles.List.AddTail("ftp.pl");
	Filename.DenyFiles.List.AddTail("poll_it_ssi_v2.0.cgi");
	Filename.DenyFiles.List.AddTail("source.asp");
	Filename.DenyFiles.List.AddTail("guestbook.pl");
	Filename.DenyFiles.List.AddTail("import.txt");
	Filename.DenyFiles.List.AddTail("count.cgi");
	Filename.DenyFiles.List.AddTail("catalog.nsf");
	Filename.DenyFiles.List.AddTail("domcfg.nsf");
	Filename.DenyFiles.List.AddTail("domlog.nsf");
	Filename.DenyFiles.List.AddTail("log.nsf");
	Filename.DenyFiles.List.AddTail("names.nsf");
	Filename.DenyFiles.List.AddTail("windmail.exe");
	Filename.DenyFiles.List.AddTail("quikstore.cfg");
	Filename.DenyFiles.List.AddTail("order.log");
	Filename.DenyFiles.List.AddTail("webdist.cgi");
	Filename.DenyFiles.List.AddTail("ws_ftp.ini");
	Filename.DenyFiles.List.AddTail("jdkrqnotify.exe");
	Filename.DenyFiles.List.AddTail("infosrch.cgi");
	Filename.DenyFiles.List.AddTail("code.php3");
	Filename.DenyFiles.List.AddTail("search.vts");
	Filename.DenyFiles.List.AddTail("ax-admin.cgi");
	Filename.DenyFiles.List.AddTail("axs.cgi");
	Filename.DenyFiles.List.AddTail("cachemgr.cgi");
	Filename.DenyFiles.List.AddTail("dfire.cgi");
	Filename.DenyFiles.List.AddTail("web-map.cgi");
	Filename.DenyFiles.List.AddTail("responder.cgi");
	Filename.DenyFiles.List.AddTail("read.php3");
	Filename.DenyFiles.List.AddTail("violation.php3");
	Filename.DenyFiles.List.AddTail("get32.exe");
	Filename.DenyFiles.List.AddTail("cgitest.exe");
	Filename.DenyFiles.List.AddTail("ftpsavecsp.dll");
	Filename.DenyFiles.List.AddTail("ftpsavecvp.dll");
	Filename.DenyFiles.List.AddTail("ftpsave.dll");
	Filename.DenyFiles.List.AddTail("contextadmin.html");
	Filename.DenyFiles.List.AddTail("architext_query.pl");
	Filename.DenyFiles.List.AddTail("wwwboard.pl");
	Filename.DenyFiles.List.AddTail("db_mysql.inc");
	Filename.DenyFiles.List.AddTail("cs.exe");
	Filename.DenyFiles.List.AddTail("bizdb1-search.cgi");
	Filename.DenyFiles.List.AddTail("bb-hostsvc.sh");
	Filename.DenyFiles.List.AddTail("pscoerrpage.htm");
	Filename.DenyFiles.List.AddTail("gwweb.exe");
	Filename.DenyFiles.List.AddTail("openview5.exe");
	Filename.DenyFiles.List.AddTail("rpcproxy.dll");		//Windows 2003 (RPC over HTTP Proxy)
	Filename.DenyFiles.List.AddTail("read_dump.php");
	Filename.DenyFiles.List.AddTail("backdoor");

	//coldfusion
	Filename.DenyFiles.List.AddTail("cfcache.map");		//coldfusion exploit
	Filename.DenyFiles.List.AddTail("exprcalc.cfm");		//coldfusion exploit
	Filename.DenyFiles.List.AddTail("beaninfo.cfm");		//coldfusion exploit
	Filename.DenyFiles.List.AddTail("application.cfm");	//coldfusion exploit
	Filename.DenyFiles.List.AddTail("getfile.cfm");		//coldfusion exploit
	Filename.DenyFiles.List.AddTail("addcontent.cfm");	//coldfusion exploit
	Filename.DenyFiles.List.AddTail("fileexists.cfm");	//coldfusion exploit
	Filename.DenyFiles.List.AddTail("evaluate.cfm");		//coldfusion exploit
	Filename.DenyFiles.List.AddTail("displayopenedfile.cfm");//coldfusion exploit
	Filename.DenyFiles.List.AddTail("mainframeset.cfm");	//coldfusion exploit
	Filename.DenyFiles.List.AddTail("cfmlsyntaxcheck.cfm");//coldfusion exploit
	Filename.DenyFiles.List.AddTail("onrequestend.cfm");	//coldfusion exploit
	Filename.DenyFiles.List.AddTail("startstop.html");	//coldfusion exploit
	Filename.DenyFiles.List.AddTail("gettempdirectory.cfm");//coldfusion exploit
	//FILENAME: Deny DOS devices
	Filename.DenyFiles.List.AddTail("nul");
	Filename.DenyFiles.List.AddTail("lpt1");
	Filename.DenyFiles.List.AddTail("lpt2");
	Filename.DenyFiles.List.AddTail("lpt3");
	Filename.DenyFiles.List.AddTail("lpt4");
	Filename.DenyFiles.List.AddTail("aux");
	Filename.DenyFiles.List.AddTail("prn");
	Filename.DenyFiles.List.AddTail("com1");
	Filename.DenyFiles.List.AddTail("com2");
	Filename.DenyFiles.List.AddTail("com3");
	Filename.DenyFiles.List.AddTail("com4");
	//FILENAME: Deny certain *nix commands
	LoadDefaultsUnixCommands(Filename.DenyFiles.List);

	Filename.MonitorFiles.Enabled = false;
	Filename.MonitorFiles.List.RemoveAll(); //lowercase!
#ifdef PRIVATEBUILD
	Filename.MonitorFiles.Enabled = true;
	Filename.MonitorFiles.List.AddTail("webknight.zip");
	//Filename.MonitorFiles.List.AddTail("robots.xml");
	//Filename.MonitorFiles.List.AddTail("bot.asp");
	//Filename.MonitorFiles.List.AddTail("luckys.src");
	//Filename.MonitorFiles.List.AddTail("duckys.jpg");
	//Filename.MonitorFiles.List.AddTail("luckys.jph");
	Filename.MonitorFiles.List.AddTail("verjaardagnico.zip");
#else
	Filename.MonitorFiles.List.AddTail("robots.txt");
#endif

	//EXTENSION: Allowed extensions (Common MIME types) (lowercase!)
	Extensions.AllowExtensions.Enabled = false;
	Extensions.AllowExtensions.List.RemoveAll();
	Extensions.AllowExtensions.List.AddTail("");			//always allow default document & no extension!!!
	Extensions.AllowExtensions.List.AddTail(".htm");		//text/html
	Extensions.AllowExtensions.List.AddTail(".html");		//text/html
	Extensions.AllowExtensions.List.AddTail(".mdl");		//text/html
	Extensions.AllowExtensions.List.AddTail(".htt");		//text/webviewhtml
	Extensions.AllowExtensions.List.AddTail(".htc");		//text/x-component
	Extensions.AllowExtensions.List.AddTail(".xml");		//text/xml
	Extensions.AllowExtensions.List.AddTail(".wml");		//wml
	Extensions.AllowExtensions.List.AddTail(".dtd");		//text/html
	Extensions.AllowExtensions.List.AddTail(".css");		//text/css (style sheet)
	Extensions.AllowExtensions.List.AddTail(".uls");		//text/iuls (netmeeting link)
	Extensions.AllowExtensions.List.AddTail(".wsc");		//text/scriptlet
	Extensions.AllowExtensions.List.AddTail(".vcf");		//text/x-vcard
	Extensions.AllowExtensions.List.AddTail(".sgm");		//sgml
	Extensions.AllowExtensions.List.AddTail(".sgml");		//sgml
	Extensions.AllowExtensions.List.AddTail(".vrml");		//vrml
	Extensions.AllowExtensions.List.AddTail(".wrl");		//vrml
	Extensions.AllowExtensions.List.AddTail(".cur");
	Extensions.AllowExtensions.List.AddTail(".ani");
	Extensions.AllowExtensions.List.AddTail(".js");		//javascript download
	Extensions.AllowExtensions.List.AddTail(".class");		//java applet
	Extensions.AllowExtensions.List.AddTail(".sfw");		//shockwave flash
	Extensions.AllowExtensions.List.AddTail(".cfm");		//ColdFusion
	Extensions.AllowExtensions.List.AddTail(".cfml");		//ColdFusion
	Extensions.AllowExtensions.List.AddTail(".txt");		//text/plain
	Extensions.AllowExtensions.List.AddTail(".asc");		//ascii file
	Extensions.AllowExtensions.List.AddTail(".doc");		//doc
	Extensions.AllowExtensions.List.AddTail(".ai");		//postscript
	Extensions.AllowExtensions.List.AddTail(".eps");		//postscript
	Extensions.AllowExtensions.List.AddTail(".ps");		//postscript
	Extensions.AllowExtensions.List.AddTail(".vsd");		//visio
	Extensions.AllowExtensions.List.AddTail(".mpp");		//ms project
	Extensions.AllowExtensions.List.AddTail(".pdf");		//pdf
	Extensions.AllowExtensions.List.AddTail(".wk4");		
	Extensions.AllowExtensions.List.AddTail(".rtf");		//rich text format
	Extensions.AllowExtensions.List.AddTail(".wmf");		//windows media file
	Extensions.AllowExtensions.List.AddTail(".mcw");
	Extensions.AllowExtensions.List.AddTail(".wps");
	Extensions.AllowExtensions.List.AddTail(".wpg");
	Extensions.AllowExtensions.List.AddTail(".xls");		//excel
	Extensions.AllowExtensions.List.AddTail(".csv");		//comma separated
	Extensions.AllowExtensions.List.AddTail(".xlw");		//excel
	Extensions.AllowExtensions.List.AddTail(".ppt");		//powerpoint
	Extensions.AllowExtensions.List.AddTail(".pot");		//powerpoint template
	Extensions.AllowExtensions.List.AddTail(".png");		//image
	Extensions.AllowExtensions.List.AddTail(".jpe");		//image
	Extensions.AllowExtensions.List.AddTail(".jpg");		//image
	Extensions.AllowExtensions.List.AddTail(".jpeg");		//image
	Extensions.AllowExtensions.List.AddTail(".gif");		//image
	Extensions.AllowExtensions.List.AddTail(".tif");		//image
	Extensions.AllowExtensions.List.AddTail(".tiff");		//image
	Extensions.AllowExtensions.List.AddTail(".bmp");		//image
	Extensions.AllowExtensions.List.AddTail(".xbm");		//image
	Extensions.AllowExtensions.List.AddTail(".ico");		//image
	Extensions.AllowExtensions.List.AddTail(".pcx");		//image
	Extensions.AllowExtensions.List.AddTail(".ief");		//image
	Extensions.AllowExtensions.List.AddTail(".rgb");		//image
	Extensions.AllowExtensions.List.AddTail(".ppm");		//image
	Extensions.AllowExtensions.List.AddTail(".pbm");		//image
	Extensions.AllowExtensions.List.AddTail(".pnm");		//image
	Extensions.AllowExtensions.List.AddTail(".mpg");		//video
	Extensions.AllowExtensions.List.AddTail(".mpeg");		//video
	Extensions.AllowExtensions.List.AddTail(".mpe");		//video
	Extensions.AllowExtensions.List.AddTail(".mp2");		//video
	Extensions.AllowExtensions.List.AddTail(".avi");		//video
	Extensions.AllowExtensions.List.AddTail(".mov");		//video
	Extensions.AllowExtensions.List.AddTail(".qt");		//video
	Extensions.AllowExtensions.List.AddTail(".asf");		//video
	Extensions.AllowExtensions.List.AddTail(".ivf");		//video
	Extensions.AllowExtensions.List.AddTail(".lsx");		//video
	Extensions.AllowExtensions.List.AddTail(".wm");		//video/x-ms-wm
	Extensions.AllowExtensions.List.AddTail(".wmv");		//video/x-ms-wmv
	Extensions.AllowExtensions.List.AddTail(".wmx");		//video/x-ms-wmx
	Extensions.AllowExtensions.List.AddTail(".wvx");		//video/x-ms-wvx
	Extensions.AllowExtensions.List.AddTail(".asd");		//asf description
	Extensions.AllowExtensions.List.AddTail(".asx");		//video
	Extensions.AllowExtensions.List.AddTail(".divx");		//video
	Extensions.AllowExtensions.List.AddTail(".wma");		//audio/x-ms-wma
	Extensions.AllowExtensions.List.AddTail(".mp3");		//audio
	Extensions.AllowExtensions.List.AddTail(".m3u");		//audio
	Extensions.AllowExtensions.List.AddTail(".aif");		//audio
	Extensions.AllowExtensions.List.AddTail(".aiff");		//audio
	Extensions.AllowExtensions.List.AddTail(".aifc");		//audio
	Extensions.AllowExtensions.List.AddTail(".au");		//audio
	Extensions.AllowExtensions.List.AddTail(".snd");		//audio
	Extensions.AllowExtensions.List.AddTail(".ra");		//realaudio
	Extensions.AllowExtensions.List.AddTail(".ram");		//realaudio
	Extensions.AllowExtensions.List.AddTail(".wav");		//audio
	Extensions.AllowExtensions.List.AddTail(".mod");		//audio
	Extensions.AllowExtensions.List.AddTail(".mid");		//audio
	Extensions.AllowExtensions.List.AddTail(".midi");		//audio
	Extensions.AllowExtensions.List.AddTail(".cdf");		//channel definition format (w3c)
	Extensions.AllowExtensions.List.AddTail(".pac");		//proxy auto configuration
	Extensions.AllowExtensions.List.AddTail(".zip");		//compress
	Extensions.AllowExtensions.List.AddTail(".rar");		//compress
	Extensions.AllowExtensions.List.AddTail(".tar");		//compress
	Extensions.AllowExtensions.List.AddTail(".gtar");		//compress
	Extensions.AllowExtensions.List.AddTail(".arj");		//compress
	Extensions.AllowExtensions.List.AddTail(".jar");		//compress
	Extensions.AllowExtensions.List.AddTail(".gz");		//compress
	Extensions.AllowExtensions.List.AddTail(".z");			//compress
	Extensions.AllowExtensions.List.AddTail(".tgz");		//compress
	Extensions.AllowExtensions.List.AddTail(".cab");		//potentially dangerous?
	Extensions.AllowExtensions.List.AddTail(".exe");		//download
	Extensions.AllowExtensions.List.AddTail(".hqx");		//mac binhex
	Extensions.AllowExtensions.List.AddTail(".msi");/*
	Extensions.AllowExtensions.List.AddTail(".idq");
	Extensions.AllowExtensions.List.AddTail(".htw");
	Extensions.AllowExtensions.List.AddTail(".ida");
	Extensions.AllowExtensions.List.AddTail(".idc");
	Extensions.AllowExtensions.List.AddTail(".shtm");
	Extensions.AllowExtensions.List.AddTail(".shtml");
	Extensions.AllowExtensions.List.AddTail(".stm");
	Extensions.AllowExtensions.List.AddTail(".htr");*/
	Extensions.AllowExtensions.List.AddTail(".jsp");	//java server pages
	Extensions.AllowExtensions.List.AddTail(".asp");	//asp
	//Extensions.AllowExtensions.List.AddTail(".asa");	//asp global.asa
	Extensions.AllowExtensions.List.AddTail(".aspx");	//asp.NET
	Extensions.AllowExtensions.List.AddTail(".ashx");	//asp.net web handler
	Extensions.AllowExtensions.List.AddTail(".asmx");	//web service
	Extensions.AllowExtensions.List.AddTail(".cer");	//certificates
	Extensions.AllowExtensions.List.AddTail(".p7b");	//Certificate Services
	Extensions.AllowExtensions.List.AddTail(".crl");	//certificate revocation list
	//Extensions.AllowExtensions.List.AddTail(".cdx");
	//Extensions.AllowExtensions.List.AddTail(".printer");

	//EXTENSION: Denied extensions (lowercase!)
	Extensions.DenyExtensions.Enabled = true;
	Extensions.DenyExtensions.List.RemoveAll();/*
	//EXTENSION: Deny ASP requests
	Extensions.DenyExtensions.List.AddTail(".asp");
	Extensions.DenyExtensions.List.AddTail(".aspx");
	Extensions.DenyExtensions.List.AddTail(".ashx");
	Extensions.DenyExtensions.List.AddTail(".cer");
	Extensions.DenyExtensions.List.AddTail(".cdx");
	Extensions.DenyExtensions.List.AddTail(".crl");*/
	Extensions.DenyExtensions.List.AddTail(".asa");//asp global file, should not be downloaded
	//Deny ASP.NET critical files
	Extensions.DenyExtensions.List.AddTail(".ascx");		//forbidden by asp.net
#ifdef PRIVATEBUILD
	//allow axd for personal use
#else
	Extensions.DenyExtensions.List.AddTail(".axd");			//WebResources
#endif
	Extensions.DenyExtensions.List.AddTail(".config");		//forbidden by asp.net (config)
	//Extensions.DenyExtensions.List.AddTail(".cs");		//forbidden by asp.net (c#) (also blocks ".css" style sheet!)
	Extensions.DenyExtensions.List.AddTail(".csproj");		//forbidden by asp.net (c# project)
	Extensions.DenyExtensions.List.AddTail(".licx");		//forbidden by asp.net (licenses)
	Extensions.DenyExtensions.List.AddTail(".rem");		//remoting ASP.NET
	Extensions.DenyExtensions.List.AddTail(".resources");	//forbidden by asp.net (resources)
	Extensions.DenyExtensions.List.AddTail(".resx");		//forbidden by asp.net (resources - xml)
	Extensions.DenyExtensions.List.AddTail(".soap");		//remoting & handling soap requests
	Extensions.DenyExtensions.List.AddTail(".vb");			//forbidden by asp.net (vb code)
	Extensions.DenyExtensions.List.AddTail(".vbproj");		//forbidden by asp.net (vb project)
	Extensions.DenyExtensions.List.AddTail(".vsdisco");	//dynamic discovery: security issue!!!! http://msdn.microsoft.com/msdnmag/issues/02/08/XMLFiles/default.aspx
	Extensions.DenyExtensions.List.AddTail(".webinfo");	//forbidden by asp.net
	//EXTENSION: Deny executables that could run on the server
	//Extensions.DenyExtensions.List.AddTail(".exe");		//this file should be allowed (download)
	//Extensions.DenyExtensions.List.AddTail(".dll");		//deny isapi extension
	//Extensions.DenyExtensions.List.AddTail(".ocx");		//(TODO: check this)
	//Extensions.DenyExtensions.List.AddTail(".drv");
	//Extensions.DenyExtensions.List.AddTail(".sys");
	//Extensions.DenyExtensions.List.AddTail(".vxd");
	Extensions.DenyExtensions.List.AddTail(".scr");
	Extensions.DenyExtensions.List.AddTail(".vbs");		//(TODO: check this)
	Extensions.DenyExtensions.List.AddTail(".bat");
	Extensions.DenyExtensions.List.AddTail(".btr");		//Frontpage dependency files
	Extensions.DenyExtensions.List.AddTail(".cmd");
	//Extensions.DenyExtensions.List.AddTail(".cgi");
	Extensions.DenyExtensions.List.AddTail(".com");
	Extensions.DenyExtensions.List.AddTail(".cpl");
	Extensions.DenyExtensions.List.AddTail(".pif");		//??? can be run??? need to check this out */
	//EXTENSION: Deny infrequently used scripts
	Extensions.DenyExtensions.List.AddTail(".htw");		//Maps to webhits.dll - Index Server
	Extensions.DenyExtensions.List.AddTail(".ida");		//Maps to idq.dll - Index Server
	Extensions.DenyExtensions.List.AddTail(".idq");		//Maps to idq.dll - Index Server
	Extensions.DenyExtensions.List.AddTail(".htr");		//Maps to ism.dll - Legacy administrative tool
	Extensions.DenyExtensions.List.AddTail(".idc");		//Maps to httpodbc.dll - Legacy database access tool
	Extensions.DenyExtensions.List.AddTail(".shtm");		//Maps to ssinc.dll - Server Side Includes
	Extensions.DenyExtensions.List.AddTail(".shtml");		//Maps to ssinc.dll - Server Side Includes
	Extensions.DenyExtensions.List.AddTail(".stm");		//Maps to ssinc.dll - Server Side Includes
	Extensions.DenyExtensions.List.AddTail(".printer");	//Maps to msw3prt.dll - Internet Printing Services
	//EXTENSION: Deny various static files
	Extensions.DenyExtensions.List.AddTail(".bin");		//IIS MetaBase
	Extensions.DenyExtensions.List.AddTail(".dmp");		//dumps & sql server logs
	Extensions.DenyExtensions.List.AddTail(".dns");		//dns files
	Extensions.DenyExtensions.List.AddTail(".evt");		//Event viewer log files
	Extensions.DenyExtensions.List.AddTail(".ini");
	Extensions.DenyExtensions.List.AddTail(".mdb");
	Extensions.DenyExtensions.List.AddTail(".mde");
	Extensions.DenyExtensions.List.AddTail(".ldb");		//access locking file
	Extensions.DenyExtensions.List.AddTail(".sav");		//backup registry files
	Extensions.DenyExtensions.List.AddTail(".adp");
	Extensions.DenyExtensions.List.AddTail(".db");		//db & dbf & dbx
	Extensions.DenyExtensions.List.AddTail(".cfg");
	Extensions.DenyExtensions.List.AddTail(".cnf");
	Extensions.DenyExtensions.List.AddTail(".conf");
	//Extensions.DenyExtensions.List.AddTail(".config");	//already denied for ASP.NET
	Extensions.DenyExtensions.List.AddTail(".ids");		//snort log
	Extensions.DenyExtensions.List.AddTail(".rules");		//snort rules
	Extensions.DenyExtensions.List.AddTail(".log");
	Extensions.DenyExtensions.List.AddTail(".pol");
	Extensions.DenyExtensions.List.AddTail(".dom");		//policy template
	Extensions.DenyExtensions.List.AddTail(".sec");
	Extensions.DenyExtensions.List.AddTail(".bak");
	Extensions.DenyExtensions.List.AddTail(".backup");
	Extensions.DenyExtensions.List.AddTail(".old");
	Extensions.DenyExtensions.List.AddTail(".000");
	Extensions.DenyExtensions.List.AddTail(".asp~");
	Extensions.DenyExtensions.List.AddTail(".tmp");
	Extensions.DenyExtensions.List.AddTail(".acl");
	Extensions.DenyExtensions.List.AddTail(".sch");
	Extensions.DenyExtensions.List.AddTail(".dat");
	Extensions.DenyExtensions.List.AddTail(".mmc");
	Extensions.DenyExtensions.List.AddTail(".msc");
	Extensions.DenyExtensions.List.AddTail(".sql");
	Extensions.DenyExtensions.List.AddTail(".tql");
	Extensions.DenyExtensions.List.AddTail(".cns");		//terminal client settings
	Extensions.DenyExtensions.List.AddTail(".inc");		//ASP include file
	Extensions.DenyExtensions.List.AddTail(".sam");		//password file
	Extensions.DenyExtensions.List.AddTail(".htgroup");	//found in snort rules (*nix only)
	Extensions.DenyExtensions.List.AddTail(".htpasswd");
	Extensions.DenyExtensions.List.AddTail(".htaccess");
	Extensions.DenyExtensions.List.AddTail(".wwwacl");
	Extensions.DenyExtensions.List.AddTail(".www_acl");
    Extensions.DenyExtensions.List.AddTail(".ewl");
	

	//HEADERS
	Headers.RemoveServerHeader = false;
	Headers.ChangeServerHeader = true;
	Headers.ServerHeader = "WWW Server/1.1";//"Apache/0.8.4"; //in the form of <application>/<version_major>.<version_minor>
	Headers.RFCCompliantHostHeader = true;
	Headers.DenyHeaders.Enabled = true;
	Headers.DenyHeaders.List.RemoveAll();
	Headers.DenyHeaders.List.AddTail("Translate:");			//needed by MS-WebDAV & Outlook Web Access
	Headers.DenyHeaders.List.AddTail("If:");				//WebDAV
	Headers.DenyHeaders.List.AddTail("Lock-Token:");		//WebDAV
	Headers.DenyHeaders.List.AddTail("Timeout:");			//WebDAV
	Headers.DenyHeaders.List.AddTail("Transfer-Encoding:"); //disables message encoding (chunked encoding,...) (TODO: needed by ASP?)
	Headers.DenyHeaders.List.AddTail("Content-Encoding:");	//disables entity encoding
	Headers.DenyHeaders.List.AddTail("Content-Transfer-Encoding:");	//not allowed in HTTP (rfc 2616)
	Headers.DenyHeaders.List.AddTail("Proxy-Authenticate:");
	Headers.DenyHeaders.List.AddTail("WWW-Authenticate:");
	Headers.DenyHeaders.List.AddTail("Ms-Echo-Request:");	//found this in asp.dll, don't know what it means, but looks very disturbing, so block it! (TODO: check this) (also found in winhttp.dll,wininet.dll,msxml3.dll)
	Headers.DenyHeaders.List.AddTail("Ms-Echo-Reply:");		//found in asp.dll: HTTP_MS_ECHO_REPLY ??? (TODO)
	Headers.DenyHeaders.List.AddTail("SOAPAction:");					//SOAP
	Headers.DenyHeaders.List.AddTail("BITS-Packet-Type:");				//BITS upload REQUEST/RESPONSE
	//Headers.DenyHeaders.List.AddTail("BITS-Protocol:");					//BITS upload		  RESPONSE
	Headers.DenyHeaders.List.AddTail("BITS-Session-Id:");				//BITS upload REQUEST/RESPONSE
	Headers.DenyHeaders.List.AddTail("BITS-Supported-Protocols:");		//BITS upload REQUEST
	//Headers.DenyHeaders.List.AddTail("BITS-Host-Id:");					//BITS upload		  RESPONSE
	//Headers.DenyHeaders.List.AddTail("BITS-Host-Id-Fallback-Timeout:");	//BITS upload		  RESPONSE
	//Headers.DenyHeaders.List.AddTail("BITS-Received-Content-Range:");	//BITS upload		  RESPONSE
	//Headers.DenyHeaders.List.AddTail("BITS-Reply-URL:");				//BITS upload		  RESPONSE
	//Headers.DenyHeaders.List.AddTail("BITS-Error-Code:");				//BITS upload		  RESPONSE
	//Headers.DenyHeaders.List.AddTail("BITS-Error-Context:");			//BITS upload		  RESPONSE

	Headers.DenyHeaders.List.AddTail("BITS-Original-Request-URL:");		//BITS Notification REQUEST
	Headers.DenyHeaders.List.AddTail("BITS-Request-DataFile-Name:");	//BITS Notification REQUEST
	Headers.DenyHeaders.List.AddTail("BITS-Response-DataFile-Name:");	//BITS Notification REQUEST
	//Headers.DenyHeaders.List.AddTail("BITS-Static-Response-URL:");		//BITS Notification			RESPONSE
	//Headers.DenyHeaders.List.AddTail("BITS-Copy-File-To-Destination:");	//BITS Notification			RESPONSE
	Headers.DenyHeaders.List.AddTail("Content-Name:");		//BITS REQUEST

	//COOKIE
	Cookie.DenySQLInjection = true;
	Cookie.DenyEncodingExploits = true;
	Cookie.DenyDirectoryTraversal = false;
	Cookie.DenyHighBitShellcode = false;
	Cookie.DenySequences.Enabled = false;
	Cookie.DenySequences.List.RemoveAll();

	//USER AGENTS
	UserAgent.DenyEmpty = true;
#ifdef PRIVATEBUILD
	UserAgent.RequireCharacter = "/-._";
#else
	//UserAgent.RequireCharacter = "/ -._";
	UserAgent.RequireCharacter = "";
#endif
	UserAgent.DenyNonRFC = true;
	UserAgent.DenyUserAgents.Enabled = false;
	UserAgent.DenyUserAgents.List.RemoveAll();
	//UserAgent.DenyUserAgents.List.AddTail("");			//empty line blocks no user-agents
	//AddNotInList(UserAgent.DenyUserAgents.List,"NPBot-1/2.0 (http://www.nameprotect.com/botinfo.html)");				//nameprotect.com (before july 2003)
	//AddNotInList(UserAgent.DenyUserAgents.List,"NPBot (http://www.nameprotect.com/botinfo.html)");					//nameprotect.com (after july 2003)

	UserAgent.DenySequences.Enabled = false;
	UserAgent.DenySequences.List.RemoveAll();	//lowercase!

	ContentType.AllowContentTypes.Enabled = true;
	ContentType.AllowContentTypes.List.RemoveAll();
	ContentType.AllowContentTypes.List.AddTail("");									//always allow no content-type
	ContentType.AllowContentTypes.List.AddTail("application/x-www-form-urlencoded");	//simple post
	//ContentType.AllowContentTypes.List.AddTail("multipart/form-data");				//posts like fileuploads
	//ContentType.AllowContentTypes.List.AddTail("text/xml");							//allow SOAP/XMLRPC posts
	ContentType.DenyContentTypes.Enabled = false;
	ContentType.DenyContentTypes.List.RemoveAll();

	Headers.DenySQLInjection = false;
	Headers.DenyEncodingExploits = false;				//needed for 'Referer:' (like with Google cache)
	Headers.DenyDirectoryTraversal = true;
	Headers.DenyHighBitShellcode = false;				//deny high bit characters (only allow US-ASCII)
	Headers.DenySequences.Enabled = true;
	Headers.DenySequences.List.RemoveAll();					//list should be lowercase
	Headers.DenySequences.List.AddTail("cmd.exe");
	Headers.DenySequences.List.AddTail("root.exe");
	Headers.DenySequences.List.AddTail("system32");
	Headers.DenySequences.List.AddTail("xp_cmdshell");
	Headers.DenySequences.List.AddTail("cirestriction=none");
	Headers.DenySequences.List.AddTail("cihilitetype=full");
	Headers.DenySequences.List.AddTail("ciwebhitsfile");
	Headers.DenySequences.List.AddTail("%u");				//unicode encoding
	Headers.DenySequences.List.AddTail("<object");			//deny malicious html (some people log e.g. 'Referer:' header and show it in html page)
	Headers.DenySequences.List.AddTail("<iframe");
	Headers.DenySequences.List.AddTail("<link");
	Headers.DenySequences.List.AddTail("<applet");
	Headers.DenySequences.List.AddTail("<embed");
	Headers.DenySequences.List.AddTail("<script");
	Headers.DenySequences.List.AddTail("<form");
	Headers.DenySequences.List.AddTail("javascript:");
	Headers.DenySequences.List.AddTail("urn:schemas-microsoft-com:time");
	Headers.DenySequences.List.AddTail("onabort=");
	Headers.DenySequences.List.AddTail("onblur=");
	Headers.DenySequences.List.AddTail("onchange=");
	Headers.DenySequences.List.AddTail("onclick=");
	Headers.DenySequences.List.AddTail("ondblclick=");
	Headers.DenySequences.List.AddTail("ondragdrop=");
	Headers.DenySequences.List.AddTail("onerror=");
	Headers.DenySequences.List.AddTail("onfocus=");
	Headers.DenySequences.List.AddTail("onkeydown=");
	Headers.DenySequences.List.AddTail("onkeypress=");
	Headers.DenySequences.List.AddTail("onkeyup=");
	Headers.DenySequences.List.AddTail("onload=");
	Headers.DenySequences.List.AddTail("onmousedown=");
	Headers.DenySequences.List.AddTail("onmouseout=");
	Headers.DenySequences.List.AddTail("onmouseover=");
	Headers.DenySequences.List.AddTail("onmouseup=");
	Headers.DenySequences.List.AddTail("onmove=");
	Headers.DenySequences.List.AddTail("onreset=");
	Headers.DenySequences.List.AddTail("onresize=");
	Headers.DenySequences.List.AddTail("onselect=");
	Headers.DenySequences.List.AddTail("onsubmit=");
	Headers.DenySequences.List.AddTail("onunload=");
	Headers.DenySequences.List.AddTail("cf_setdatasourceusername()");	//coldfusion app exploit
	Headers.DenySequences.List.AddTail("cf_setdatasourcepassword()");	//coldfusion exploit
	Headers.DenySequences.List.AddTail("cf_iscoldfusiondatasource()");	//coldfusion exploit
	Headers.DenySequences.List.AddTail("cfusion_getodbcdsn()");			//coldfusion exploit
	Headers.DenySequences.List.AddTail("cfusion_dbconnections_flush()"); //coldfusion exploit
	Headers.DenySequences.List.AddTail("cfusion_encrypt()");				//coldfusion exploit
	Headers.DenySequences.List.AddTail("cfusion_setodbcini()");			//coldfusion exploit
	Headers.DenySequences.List.AddTail("cfusion_settings_refresh()");	//coldfusion exploit
	Headers.DenySequences.List.AddTail("cfusion_verifymail()");			//coldfusion exploit
	Headers.DenySequences.List.AddTail("file=http\\://");			//PHP-Nuke remote file attempt
	//Headers.DenySequences.List.AddTail("user-agent: webtrends security analyzer");	//Webtrends Security Analyzer user agent string
	Headers.DenySequences.List.AddTail("includedir=");
	Headers.DenySequences.List.AddTail("_phplib[libdir]");
	Headers.DenySequences.List.AddTail("cookie: =");					//Cookie ASP exploit

	//Referrer
	Referrer.UseScan = true;
	Referrer.RFCCompliantURL = true;
	Referrer.RFCCompliantHTTPURL = true;
	Referrer.DenyEncodingExploits = false;
	Referrer.DenyHotLinking = false;
	Referrer.HotLinkingFileExtensions.RemoveAll();
	Referrer.HotLinkingFileExtensions.AddTail(".bmp");
	Referrer.HotLinkingFileExtensions.AddTail(".jpg");
	Referrer.HotLinkingFileExtensions.AddTail(".gif");
	Referrer.HotLinkingFileExtensions.AddTail(".png");
	Referrer.HotLinkingFileExtensions.AddTail(".exe");
	Referrer.HotLinkingFileExtensions.AddTail(".zip");
	Referrer.HotLinkingAllowDomains.Enabled = true;
	Referrer.HotLinkingAllowDomains.List.RemoveAll();
	Referrer.HotLinkingAllowDomains.List.AddTail("localhost");
	Referrer.HotLinkingAllowDomains.List.AddTail("127.0.0.1");
	Referrer.HotLinkingDenyDomains.Enabled = false;
	Referrer.HotLinkingDenyDomains.List.RemoveAll();
	Referrer.HotLinkingUseHostHeader = true;
	Referrer.HotLinkingDenyBlankReferrer = false;
	Referrer.DenyCharacters = "";
	Referrer.DenyHighBitShellcode = false;
	Referrer.DenySQLInjection = true;
	Referrer.DenySequences.Enabled = false;
	Referrer.DenySequences.List.RemoveAll();
#ifdef PRIVATEBUILD
	Referrer.DenyHotLinking = true;
	//Referrer.HotLinkingDenyBlankReferrer = true;
#endif

	//VERBS: Allowed verbs list (case sensitive: see RFC 2616!)
	Verbs.AllowVerbs.Enabled = true;
	Verbs.AllowVerbs.List.RemoveAll();
	//Verbs.AllowVerbs.List.AddTail("OPTIONS");	//rfc & biztalk & frontpage & sharepoint
	Verbs.AllowVerbs.List.AddTail("GET");			//rfc - simple request (must allow!)
	Verbs.AllowVerbs.List.AddTail("HEAD");		//rfc - simple request (must allow!)
	Verbs.AllowVerbs.List.AddTail("POST");		//rfc - post data
	//Verbs.AllowVerbs.List.AddTail("PUT");		//rfc & biztalk & commerce
	//Verbs.AllowVerbs.List.AddTail("DELETE");	//rfc & biztalk
	//Verbs.AllowVerbs.List.AddTail("TRACE");		//rfc
	//Verbs.AllowVerbs.List.AddTail("CONNECT");	//rfc
	//Verbs.AllowVerbs.List.AddTail("PROPFIND");	//biztalk & webdav
	//Verbs.AllowVerbs.List.AddTail("PROPPATCH");	//OWA & webdav
	//Verbs.AllowVerbs.List.AddTail("MKCOL");		//biztalk & webdav
	//Verbs.AllowVerbs.List.AddTail("POLL");		//exchange 2000
	//Verbs.AllowVerbs.List.AddTail("DEBUG");		//ASP.NET debugging
	//Verbs.AllowVerbs.List.AddTail("BITS_POST");	//BITS

	//VERBS: Denied verbs list (must be uppercase! lowercase & variants will also be blocked)
	Verbs.DenyVerbs.Enabled = false;
	Verbs.DenyVerbs.List.RemoveAll();
	Verbs.DenyVerbs.List.AddTail("PROPFIND");  //these verbs will block WebDAV (warning: WebDAV
	Verbs.DenyVerbs.List.AddTail("PROPPATCH"); //is required for Outlook Web Access and "web folders")
	Verbs.DenyVerbs.List.AddTail("MKCOL");
	Verbs.DenyVerbs.List.AddTail("DELETE");
	Verbs.DenyVerbs.List.AddTail("PUT");
	Verbs.DenyVerbs.List.AddTail("COPY");
	Verbs.DenyVerbs.List.AddTail("MOVE");
	Verbs.DenyVerbs.List.AddTail("LOCK");
	Verbs.DenyVerbs.List.AddTail("UNLOCK");
	Verbs.DenyVerbs.List.AddTail("OPTIONS");
	Verbs.DenyVerbs.List.AddTail("POLL");
	Verbs.DenyVerbs.List.AddTail("SEARCH");	//used by MS & rfc proposal (didn't make it)
	Verbs.DenyVerbs.List.AddTail("CONNECT");	//I added this one, the ones above are from urlscan
	Verbs.DenyVerbs.List.AddTail("TRACE");	//I added this one too (rfc)
	//found these in winhttp.dll (TODO: what are these & exploit? IIS says "not implemented")
	Verbs.DenyVerbs.List.AddTail("M-POST");		//winhttp.dll SOAP, xmlrpc,...
	Verbs.DenyVerbs.List.AddTail("PIN");			//winhttp.dll TODO: what is it & exploit?
	Verbs.DenyVerbs.List.AddTail("INVOKE");		//winhttp.dll used by sharepoint portal server TODO: what is it & exploit?
	Verbs.DenyVerbs.List.AddTail("CHECKOUT");		//winhttp.dll & rfc proposal (didn't make it) TODO: what is it & exploit?
	Verbs.DenyVerbs.List.AddTail("CHECKIN");		//winhttp.dll & rfc proposal (didn't make it) TODO: what is it & exploit?
	Verbs.DenyVerbs.List.AddTail("NOTIFY");		//winhttp.dll
	Verbs.DenyVerbs.List.AddTail("UNSUBSCRIBE");	//winhttp.dll
	Verbs.DenyVerbs.List.AddTail("SUBSCRIBE");	//winhttp.dll
	Verbs.DenyVerbs.List.AddTail("BDELETE");		//winhttp.dll
	Verbs.DenyVerbs.List.AddTail("BPROPPATCH");	//winhttp.dll
	Verbs.DenyVerbs.List.AddTail("BPROPFIND");	//winhttp.dll
	Verbs.DenyVerbs.List.AddTail("BCOPY");		//winhttp.dll
	Verbs.DenyVerbs.List.AddTail("BMOVE");		//winhttp.dll
	Verbs.DenyVerbs.List.AddTail("UNLINK");		//winhttp.dll
	Verbs.DenyVerbs.List.AddTail("LINK");			//winhttp.dll
	Verbs.DenyVerbs.List.AddTail("TRACK");		//found in MSDN library: returns the original request (like TRACE)
	Verbs.DenyVerbs.List.AddTail("SHOWMETHOD");	//http method proposal that didn't make it into the RFC
	Verbs.DenyVerbs.List.AddTail("TEXTSEARCH");	//http method proposal that didn't make it into the RFC
	Verbs.DenyVerbs.List.AddTail("SPACEJUMP");	//http method proposal that didn't make it into the RFC
	Verbs.DenyVerbs.List.AddTail("SUBSCRIPTIONS");//urlscan (urlscan_exchange2000.ini)
	Verbs.DenyVerbs.List.AddTail("ACL");			//urlscan (urlscan_exchange2000.ini)
	Verbs.DenyVerbs.List.AddTail("NOTIFY");		//urlscan (urlscan_exchange2000.ini)
	Verbs.DenyVerbs.List.AddTail("DEBUG");		//ASP.NET debugging
	Verbs.DenyVerbs.List.AddTail("X-MS-ENUMATTS");//Found in Exchange 2000 SDK http://msdn.microsoft.com/library/default.asp?url=/library/en-us/wss/wss/_webdav_x-ms-enumatts.asp
	Verbs.DenyVerbs.List.AddTail("RPC_IN_DATA");	//RPC-over-HTTP (new in Windows2003)
	Verbs.DenyVerbs.List.AddTail("RPC_OUT_DATA");	//RPC-over-HTTP (new in Windows2003)
	Verbs.DenyVerbs.List.AddTail("BITS_POST");		//BITS

	LoadDefaultsSQLInjection(SQLKeyWords);
	
	//QUERYSTRING
	QueryString.DenySQLInjection = true;			//Deny SQL injection in querystring
	QueryString.DenyEncodingExploits = true;		//Deny embedded encoding exploits
	QueryString.DenyDirectoryTraversal = true;
	QueryString.DenyHighBitShellcode = false;		//deny high bit characters (only allow US-ASCII)
	QueryString.UseRawScan = true;
	QueryString.DenySequences.Enabled = true;			//Deny certain sequences
	QueryString.DenySequences.List.RemoveAll();			//list has to be lowercase
	QueryString.DenySequences.List.AddTail("<object");	//deny malicious html
	QueryString.DenySequences.List.AddTail("<iframe");
	QueryString.DenySequences.List.AddTail("<link");
	QueryString.DenySequences.List.AddTail("<applet");
	QueryString.DenySequences.List.AddTail("<embed");
	QueryString.DenySequences.List.AddTail("<script");
	QueryString.DenySequences.List.AddTail("<form");
	QueryString.DenySequences.List.AddTail("javascript:");
	QueryString.DenySequences.List.AddTail("urn:schemas-microsoft-com:time");
	QueryString.DenySequences.List.AddTail("onabort=");
	QueryString.DenySequences.List.AddTail("onblur=");
	QueryString.DenySequences.List.AddTail("onchange=");
	QueryString.DenySequences.List.AddTail("onclick=");
	QueryString.DenySequences.List.AddTail("ondblclick=");
	QueryString.DenySequences.List.AddTail("ondragdrop=");
	QueryString.DenySequences.List.AddTail("onerror=");
	QueryString.DenySequences.List.AddTail("onfocus=");
	QueryString.DenySequences.List.AddTail("onkeydown=");
	QueryString.DenySequences.List.AddTail("onkeypress=");
	QueryString.DenySequences.List.AddTail("onkeyup=");
	QueryString.DenySequences.List.AddTail("onload=");
	QueryString.DenySequences.List.AddTail("onmousedown=");
	QueryString.DenySequences.List.AddTail("onmouseout=");
	QueryString.DenySequences.List.AddTail("onmouseover=");
	QueryString.DenySequences.List.AddTail("onmouseup=");
	QueryString.DenySequences.List.AddTail("onmove=");
	QueryString.DenySequences.List.AddTail("onreset=");
	QueryString.DenySequences.List.AddTail("onresize=");
	QueryString.DenySequences.List.AddTail("onselect=");
	QueryString.DenySequences.List.AddTail("onsubmit=");
	QueryString.DenySequences.List.AddTail("onunload=");
	QueryString.DenySequences.List.AddTail("administrators /add");
	QueryString.DenySequences.List.AddTail("cirestriction=none");
	QueryString.DenySequences.List.AddTail("cihilitetype=full");
	QueryString.DenySequences.List.AddTail("ciwebhitsfile");
	QueryString.DenySequences.List.AddTail("cmd.exe");
	QueryString.DenySequences.List.AddTail("system32");
	QueryString.DenySequences.List.AddTail("root.exe");
	QueryString.DenySequences.List.AddTail("xp_cmdshell");
	QueryString.DenySequences.List.AddTail("%u");
	QueryString.DenySequences.List.AddTail("cf_setdatasourceusername()");	//coldfusion app exploit
	QueryString.DenySequences.List.AddTail("cf_setdatasourcepassword()");	//coldfusion exploit
	QueryString.DenySequences.List.AddTail("cf_iscoldfusiondatasource()");	//coldfusion exploit
	QueryString.DenySequences.List.AddTail("cfusion_getodbcdsn()");			//coldfusion exploit
	QueryString.DenySequences.List.AddTail("cfusion_dbconnections_flush()");//coldfusion exploit
	QueryString.DenySequences.List.AddTail("cfusion_encrypt()");			//coldfusion exploit
	QueryString.DenySequences.List.AddTail("cfusion_setodbcini()");			//coldfusion exploit
	QueryString.DenySequences.List.AddTail("cfusion_settings_refresh()");	//coldfusion exploit
	QueryString.DenySequences.List.AddTail("cfusion_verifymail()");			//coldfusion exploit
	QueryString.DenySequences.List.AddTail("includedir=");
	QueryString.DenySequences.List.AddTail("_phplib[libdir]");
	//Netscape Enterprise Server directory view exploit (only via querystring)
	QueryString.DenySequences.List.AddTail("wp-verify-link");
	QueryString.DenySequences.List.AddTail("wp-cs-dump");
	QueryString.DenySequences.List.AddTail("wp-ver-info");
	QueryString.DenySequences.List.AddTail("wp-ver-diff");
	QueryString.DenySequences.List.AddTail("wp-start-ver");
	QueryString.DenySequences.List.AddTail("wp-stop-ver");
	QueryString.DenySequences.List.AddTail("wp-uncheckout");
	QueryString.DenySequences.List.AddTail("wp-html-rend");
	QueryString.DenySequences.List.AddTail("wp-usr-prop");
	//Querystring deny *nix commands
	//LoadDefaultsUnixCommands(QueryString.DenySequences.List);	//too much collateral damage
	//XSS injection
	QueryString.DenySequences.List.AddTail("=http://");
	QueryString.DenySequences.List.AddTail("=https://");
	QueryString.DenySequences.List.AddTail("=[http://");
	QueryString.DenySequences.List.AddTail("=[https://");
	QueryString.DenySequences.List.AddTail("dir=http://");
	QueryString.DenySequences.List.AddTail("dir]=http://");
	QueryString.DenySequences.List.AddTail("file=http://");
	QueryString.DenySequences.List.AddTail("file]=http://");
	QueryString.DenySequences.List.AddTail("root=http://");
	QueryString.DenySequences.List.AddTail("root]=http://");
	QueryString.DenySequences.List.AddTail("path=http://");
	QueryString.DenySequences.List.AddTail("path]=http://");
	QueryString.DenySequences.List.AddTail("include=http://");
	QueryString.DenySequences.List.AddTail("name=http://");
	QueryString.DenySequences.List.AddTail("page=http://");
	//PHP injection
	QueryString.DenySequences.List.AddTail("<?php");
	//file access
	QueryString.DenySequences.List.AddTail("c:\\");
	QueryString.DenySequences.List.AddTail("\r\n");

	//raw data (headers sequences & POST data)
	Global.IsInstalledAsGlobalFilter = true;		//Raw data can only be scanned if filter is a global filter
	Global.DenyPostSQLInjection = false;			//no SQL injection in POST data
	Global.DenyPostEncodingExploits = false;		//embedded encoding needed by OWA
	Global.DenyPostDirectoryTraversal = false;
	Global.DenyPostHighBitShellcode = false;		//deny high bit characters (only allow US-ASCII)
#ifdef PRIVATEBUILD
	Global.DenyPostSequences.Enabled = false;
#else
	Global.DenyPostSequences.Enabled = true;
#endif
	Global.DenyPostSequences.List.RemoveAll();		//list has to be lowercase
	Global.DenyPostSequences.List.AddTail("<object");	//deny malicious html
	Global.DenyPostSequences.List.AddTail("<iframe");
	Global.DenyPostSequences.List.AddTail("<link");
	Global.DenyPostSequences.List.AddTail("<applet");
	Global.DenyPostSequences.List.AddTail("<embed");
	Global.DenyPostSequences.List.AddTail("<script");
	Global.DenyPostSequences.List.AddTail("<form");
	Global.DenyPostSequences.List.AddTail("javascript:");
	Global.DenyPostSequences.List.AddTail("urn:schemas-microsoft-com:time");
	Global.DenyPostSequences.List.AddTail("onabort=");
	Global.DenyPostSequences.List.AddTail("onblur=");
	Global.DenyPostSequences.List.AddTail("onchange=");
	Global.DenyPostSequences.List.AddTail("onclick=");
	Global.DenyPostSequences.List.AddTail("ondblclick=");
	Global.DenyPostSequences.List.AddTail("ondragdrop=");
	Global.DenyPostSequences.List.AddTail("onerror=");
	Global.DenyPostSequences.List.AddTail("onfocus=");
	Global.DenyPostSequences.List.AddTail("onkeydown=");
	Global.DenyPostSequences.List.AddTail("onkeypress=");
	Global.DenyPostSequences.List.AddTail("onkeyup=");
	Global.DenyPostSequences.List.AddTail("onload=");
	Global.DenyPostSequences.List.AddTail("onmousedown=");
	Global.DenyPostSequences.List.AddTail("onmouseout=");
	Global.DenyPostSequences.List.AddTail("onmouseover=");
	Global.DenyPostSequences.List.AddTail("onmouseup=");
	Global.DenyPostSequences.List.AddTail("onmove=");
	Global.DenyPostSequences.List.AddTail("onreset=");
	Global.DenyPostSequences.List.AddTail("onresize=");
	Global.DenyPostSequences.List.AddTail("onselect=");
	Global.DenyPostSequences.List.AddTail("onsubmit=");
	Global.DenyPostSequences.List.AddTail("onunload=");
	Global.DenyPostSequences.List.AddTail("cmd.exe");
	Global.DenyPostSequences.List.AddTail("system32");
	Global.DenyPostSequences.List.AddTail("root.exe");
	Global.DenyPostSequences.List.AddTail("xp_cmdshell");
	Global.DenyPostSequences.List.AddTail("%u");					//unicode encoding
	Global.DenyPostSequences.List.AddTail("content-encoding:");	//disables entity encoding (bodyparts)
	Global.DenyPostSequences.List.AddTail("content-transfer-encoding:"); //not allowed in HTTP (rfc 2616)
	Global.DenyPostSequences.List.AddTail("cf_setdatasourceusername()");	//coldfusion app exploit
	Global.DenyPostSequences.List.AddTail("cf_setdatasourcepassword()");	//coldfusion exploit
	Global.DenyPostSequences.List.AddTail("cf_iscoldfusiondatasource()");	//coldfusion exploit
	Global.DenyPostSequences.List.AddTail("cfusion_getodbcdsn()");			//coldfusion exploit
	Global.DenyPostSequences.List.AddTail("cfusion_dbconnections_flush()");//coldfusion exploit
	Global.DenyPostSequences.List.AddTail("cfusion_encrypt()");			//coldfusion exploit
	Global.DenyPostSequences.List.AddTail("cfusion_setodbcini()");			//coldfusion exploit
	Global.DenyPostSequences.List.AddTail("cfusion_settings_refresh()");	//coldfusion exploit
	Global.DenyPostSequences.List.AddTail("cfusion_verifymail()");			//coldfusion exploit
	Global.DenyPostSequences.List.AddTail("file=http\\://");			//PHP-Nuke remote file attempt
	Global.DenyPostSequences.List.AddTail("includedir=");
	Global.DenyPostSequences.List.AddTail("_phplib[libdir]");
		//XSS injection
	Global.DenyPostSequences.List.AddTail("=http://");
	Global.DenyPostSequences.List.AddTail("=https://");
	Global.DenyPostSequences.List.AddTail("=[http://");
	Global.DenyPostSequences.List.AddTail("=[https://");
	Global.DenyPostSequences.List.AddTail("dir=http://");
	Global.DenyPostSequences.List.AddTail("dir]=http://");
	Global.DenyPostSequences.List.AddTail("file=http://");
	Global.DenyPostSequences.List.AddTail("file]=http://");
	Global.DenyPostSequences.List.AddTail("root=http://");
	Global.DenyPostSequences.List.AddTail("root]=http://");
	Global.DenyPostSequences.List.AddTail("path=http://");
	Global.DenyPostSequences.List.AddTail("path]=http://");
	Global.DenyPostSequences.List.AddTail("include=http://");
	Global.DenyPostSequences.List.AddTail("name=http://");
	Global.DenyPostSequences.List.AddTail("page=http://");
	//PHP injection
	Global.DenyPostSequences.List.AddTail("<?php");

	//Allow compatible web applications
	WebApp.Allow_FileUploads = false;
	WebApp.Allow_Unicode = false;
	WebApp.Allow_OutlookWebAccess = false;
	WebApp.Allow_OutlookMobileAccess = false;
	WebApp.Allow_FrontpageExtensions = false;
	WebApp.Allow_Coldfusion = false;
	WebApp.Allow_FlashMX = false;
	WebApp.Allow_WebDAV = false;
	WebApp.Allow_IISADMPWD = false;
	WebApp.Allow_SharePointPortalServer = false;
	WebApp.Allow_SharePointTeamServices = false;
	WebApp.Allow_CertificateServices = false;
	WebApp.Allow_BizTalkServer = false;
	WebApp.Allow_CommerceServer = false;
	WebApp.Allow_SmallBusinessServer = false;
	WebApp.Allow_ASPNET = false;
	WebApp.Allow_ASP = false;
	WebApp.Allow_RPCoverHTTP = false;
	WebApp.Allow_ActiveSync = false;
	WebApp.Allow_BITS = false;
	WebApp.Allow_PHP = false;
	WebApp.Allow_SOAP = false;

	//BOTS
	Robots.AllowRobotsFile = true;
	Robots.DenyAll = false;
	Robots.DenyBad = true;
#ifdef PRIVATEBUILD
	Robots.DenyAggressive.Enabled = true;
#else
	Robots.DenyAggressive.Enabled = false;
#endif
	Robots.DenyAggressive.MaxTime = 3;		//minutes
	Robots.DenyAggressive.MaxCount = 180;	//number of requests
	Robots.BadBotTraps.RemoveAll();
	Robots.BadBotTraps.AddTail("/badbottrap");//badbot trap (case sensitive) and no ending slash (for cyveillance bot!)
	Robots.BadBotTraps.AddTail("/guestbookspamtrap");//for those bots going to "guestbook" urls
	Robots.BadBotTraps.AddTail("/robotsxx.txt");
	Robots.BotTimeout = 36;	//24 (hours)

	Agents.BlockDataMiningCommercial = false;
	Agents.BlockDataMiningPublic = false;
	Agents.BlockDownloadManagers = false;
	Agents.BlockEmailHarvesting = true;
	Agents.BlockGuestbookSpammers = true;
	Agents.BlockHackTools = false;
	Agents.BlockImageDownloaders = false;
	Agents.BlockIndexing = false;
	Agents.BlockMonitoring = false;
	Agents.BlockOfflineBrowsers = false;
	Agents.BlockOtherBad = true;
	Agents.BlockTrademark = false;
	Agents.BlockValidationTools = false;
	Agents.BlockLinkChecking = false;
	Agents.BlockBrowsers = false;
	Agents.BlockMediaPlayers = false;
	Agents.BlockProxies = false;
	Agents.BlockAdware = false;
	Agents.BlockBrowserExtensions = false;
	Agents.BlockSpyware = false;
	Agents.BlockEditing = false;
	Agents.BlockDevice = false;
	Agents.BlockNewsFeed = false;
	Agents.BlockSearchEngines = false;
	Agents.BlockFilteringSoftware = false;
	Agents.BlockSoftwareComponent = false;

#ifdef PRIVATEBUILD
	//Agents.BlockDataMiningCommercial = true; //Allow Google AdSense
	Agents.BlockEmailHarvesting = true;
	Agents.BlockGuestbookSpammers = true;
	Agents.BlockHackTools = true;
	Agents.BlockImageDownloaders = true;
	Agents.BlockMonitoring = true;
	Agents.BlockOfflineBrowsers = true;
	Agents.BlockOtherBad = true;
	Agents.BlockTrademark = true;
	Agents.BlockValidationTools = true;
#endif

}

bool CWebKnightSettings::WriteToFileXML(LPCTSTR fn)
{
	CString xml("");

	//xml += "<!DOCTYPE WebKnight[\r\n";			//start dtd
	//xml += "<!ELEMENT WebKnightSettings ANY >\r\n";
	//xml += " ]>\r\n";							//end dtd

	xml += "<WebKnightSettings>\r\n";	//start entity
	xml += "<AQTRONIX_WebKnight_Configuration App='Title'/>\r\n";
	xml += "<WebKnight_Configuration App='Document'/>\r\n";
	xml += ToXML();
	xml += "</WebKnightSettings>\r\n";		//end entity

	return WriteXMLEntityToFile(fn,xml);
}

bool CWebKnightSettings::WriteToFileINI(LPCTSTR fn)
{
	/* 
	 * Write all settings to a simple INI file
	 * All checks on the file should have been
	 * done before this function is called
	 */

	if(!CHTTPFirewallSettings::WriteToFileINI(fn))
		return false;

	try{

	//BOTS
	WRITE_INI_BOOL(Robots,AllowRobotsFile)
	WRITE_INI_BOOL(Robots,DenyAll)
	WRITE_INI_BOOL(Robots,DenyBad)
	WRITE_INI_CACHE(Robots,DenyAggressive)
	WRITE_INI_INT(Robots,BotTimeout)
	WRITE_INI_CSTRINGLIST(Robots,BadBotTraps)

	//BOTS DATABASE
#define CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(name) \
		WritePrivateProfileString("Robots","BlockBots" #name,Agents.Block##name?"1":"0",fn);
	
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(DataMiningCommercial)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(DataMiningPublic)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(DownloadManagers)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(EmailHarvesting)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(GuestbookSpammers)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(HackTools)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(ImageDownloaders)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(Indexing)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(Monitoring)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(OfflineBrowsers)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(OtherBad)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(Trademark)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(ValidationTools)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(LinkChecking)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(Browsers)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(MediaPlayers)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(Proxies)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(Adware)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(BrowserExtensions)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(Spyware)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(Editing)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(Device)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(NewsFeed)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(SearchEngines)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(FilteringSoftware)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_BOTS(SoftwareComponent)

	//Web applications
#define CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(name) \
		WritePrivateProfileString("Web Applications","Allow" #name,WebApp.Allow_##name?"1":"0",fn);

	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(FileUploads)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(Unicode)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(OutlookWebAccess)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(OutlookMobileAccess)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(ActiveSync)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(RPCoverHTTP)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(FrontpageExtensions)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(Coldfusion)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(FlashMX)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(WebDAV)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(IISADMPWD)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(SharePointPortalServer)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(SharePointTeamServices)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(CertificateServices)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(BizTalkServer)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(CommerceServer)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(SmallBusinessServer)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(ASPNET)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(ASP)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(PHP)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(BITS)
	CWEBKNIGHTSETTINGS_WRITETOFILEINI_WEBAPP(SOAP)
	
		return true;
	}catch(...){
		return false;
	}
}


bool CWebKnightSettings::ReadFromFileINI(LPCTSTR fn)
{
	if(!CHTTPFirewallSettings::ReadFromFileINI(fn))
		return false;

	try{

		const int sz = MAX_INI_FILE_SIZE;
		char buf[sz];
		DWORD l(0);

#define WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(name)\
	Agents.Block##name = GetPrivateProfileInt("Robots","BlockBots"#name,Agents.Block##name,fn)>0;

		//BOTS
		READ_INI_BOOL(Robots,AllowRobotsFile)
		READ_INI_BOOL(Robots,DenyAll)
		READ_INI_BOOL(Robots,DenyBad)
		READ_INI_CACHE(Robots,DenyAggressive)
		READ_INI_INT(Robots,BotTimeout)
		READ_INI_CSTRINGLIST(Robots,BadBotTraps,CleanupList)

		//BOTS DATABASE
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(DataMiningCommercial)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(DataMiningPublic)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(DownloadManagers)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(EmailHarvesting)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(GuestbookSpammers)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(HackTools)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(ImageDownloaders)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(Indexing)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(Monitoring)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(OfflineBrowsers)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(OtherBad)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(Trademark)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(ValidationTools)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(LinkChecking)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(Browsers)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(MediaPlayers)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(Proxies)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(Adware)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(BrowserExtensions)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(Spyware)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(Editing)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(Device)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(NewsFeed)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(SearchEngines)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(FilteringSoftware)
		WEBKNIGHTSETTINGS_READFROMFILEINI_BOTS(SoftwareComponent)

		//WEB APPLICATIONS
#define WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(name)\
		WebApp.Allow_##name = GetPrivateProfileInt("Web Applications","Allow"#name,WebApp.Allow_##name,fn)>0;\
		if(WebApp.Allow_##name) Enable##name();

		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(FileUploads)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(Unicode)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(OutlookWebAccess)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(OutlookMobileAccess)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(ActiveSync)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(RPCoverHTTP)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(FrontpageExtensions)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(Coldfusion)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(FlashMX)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(WebDAV)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(IISADMPWD)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(SharePointPortalServer)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(SharePointTeamServices)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(CertificateServices)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(BizTalkServer)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(CommerceServer)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(SmallBusinessServer)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(ASPNET)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(ASP)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(PHP)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(BITS)
		WEBKNIGHTSETTINGS_READFROMFILEINI_WEBAPPS(SOAP)
		
		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

bool CWebKnightSettings::ParseXML(CString& key, CString& val)
{
	//if not a valid key
	if (key=="")
		return false;

#define CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(var,key)\
	else if(name.CompareNoCase("Block_Bots_" #key)==0){	Agents.Block##var = val=="1"?true:false; }

#define CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(var,key)\
	else if(name.CompareNoCase("Allow_" #key)==0){	WebApp.Allow_##var = val=="1"?true:false; if(WebApp.Allow_##var) Enable##var(); }

	if(!CHTTPFirewallSettings::ParseXML(key,val)){

		CString name = XMLDecode(XMLKeyParameter(key,"name"));

		//BOTS
		PARSE_XML_BOOL(Robots,AllowRobotsFile,"Allow_Bots_Robots_File")
		else PARSE_XML_BOOL(Robots,DenyAll,"Deny_Bots_All")
		else PARSE_XML_BOOL(Robots,DenyBad,"Deny_Bots_Bad")
		else PARSE_XML_CSTRINGLIST(Robots,BadBotTraps,"Deny_Bots_BotTraps")
		else PARSE_XML_CACHE(Robots,DenyAggressive,"Deny_Bots_Aggressive")
		else PARSE_XML_INT(Robots,BotTimeout,"Deny_Bots_Timeout",(unsigned int))

		//BOTS DATABASE
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(DataMiningCommercial,Data_Mining_Commercial)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(DataMiningPublic,Data_Mining_Public)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(DownloadManagers,Download_Managers)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(EmailHarvesting,Email_Harvesting)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(GuestbookSpammers,Guestbook_Spammers)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(HackTools,Hack_Tools)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(ImageDownloaders,Image_Downloaders)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(Indexing,Indexing)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(Monitoring,Monitoring)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(OfflineBrowsers,Offline_Browsers)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(OtherBad,Other_Bad)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(Trademark,Trademark)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(ValidationTools,Validation_Tools)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(LinkChecking,Link_Checking)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(Browsers,Browsers)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(MediaPlayers,Media_Players)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(Proxies,Proxies)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(Adware,Adware)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(BrowserExtensions,Browser_Extensions)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(Spyware,Spyware)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(Editing,Editing)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(Device,Device)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(NewsFeed,News_Feed)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(SearchEngines,Search_Engines)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(FilteringSoftware,Filtering_Software)
		CWEBKNIGHTSSETTINGS_PARSEXML_BOTS(SoftwareComponent,Software_Component)

		//Web applications
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(FileUploads,File_Uploads)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(Unicode,Unicode)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(OutlookWebAccess,Outlook_Web_Access)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(OutlookMobileAccess,Outlook_Mobile_Access)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(ActiveSync,ActiveSync)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(RPCoverHTTP,RPC_over_HTTP)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(FrontpageExtensions,Frontpage_Extensions)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(Coldfusion,Coldfusion)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(FlashMX,Flash_MX)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(WebDAV,WebDAV)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(IISADMPWD,IISADMPWD)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(SharePointPortalServer,SharePoint_Portal_Server)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(SharePointTeamServices,SharePoint_Team_Services)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(CertificateServices,Certificate_Services)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(BizTalkServer,BizTalk_Server)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(CommerceServer,Commerce_Server)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(SmallBusinessServer,Small_Business_Server)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(ASPNET,ASP_NET)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(ASP,ASP)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(PHP,PHP)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(BITS,BITS)
		CWEBKNIGHTSSETTINGS_PARSEXML_WEBAPPS(SOAP,SOAP)
		//else
		//	{
		//		CLogger test("C:\\Debug\\WebKnightSettings\\");
		//		test.NewEntry(name);
		//		test.Append(val);
		//		test.Flush();
		//		return false;
		//	}
	}
	return true;
}

bool CWebKnightSettings::EnableOutlookWebAccess()
{
	try{
		//From urlscan and http://support.microsoft.com/default.aspx?scid=kb;EN-US;309508
		EnableWebDAV(); //WebDAV required for OWA

		//Allow all POST sequences
		Global.DenyPostEncodingExploits = false;
		Global.DenyPostSequences.Enabled = false;

		//Allow virtual dirs
		RemoveFrom(URL.DenySequences.List,"/exchange");
		RemoveFrom(URL.DenySequences.List,"/public");
		RemoveFrom(URL.DenySequences.List,"/exadmin");
		RemoveFrom(URL.DenySequences.List,"/exchweb");
		RemoveFrom(URL.DenySequences.List,"/bin/");

		//Allow logon form (Exchange 5.5)
		RemoveFrom(Filename.DenyFiles.List,"logonfrm.asp");
		RemoveFrom(Filename.DenyFiles.List,"prn");

		//Allow certain paths
		AddNotInList(Path.AllowPaths.List,"m:\\");
		AddNotInList(Path.AllowPaths.List,"c:\\program files\\exchsrvr\\exchweb");
		AddNotInList(Path.AllowPaths.List,"\\.\backofficestorage");

		//Allow certain content-types (from testing)
		AddNotInList(ContentType.AllowContentTypes.List,"");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-UTF8-encoded");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-form-urlencoded");
		AddNotInList(ContentType.AllowContentTypes.List,"multipart/form-data");

	//EXCHANGE 2000
		//Allow certain settings (from urlscan)
		Path.DenyDot = false;				//allow dot in path
		URL.DenyHighBitShellcode = false;	//allow high bit characters in url
		URL.DenyAlternateStream = false;	//allow ':' in url
		Referrer.DenyHighBitShellcode = false;

		//Allow these verbs not in WebDAV (from urlscan)
		AddNotInList(Verbs.AllowVerbs.List,"POLL");			RemoveFrom(Verbs.DenyVerbs.List,"POLL");
		AddNotInList(Verbs.AllowVerbs.List,"BMOVE");		RemoveFrom(Verbs.DenyVerbs.List,"BMOVE");
		AddNotInList(Verbs.AllowVerbs.List,"BCOPY");		RemoveFrom(Verbs.DenyVerbs.List,"BCOPY");
		AddNotInList(Verbs.AllowVerbs.List,"BPROPPATCH");	RemoveFrom(Verbs.DenyVerbs.List,"BPROPPATCH");
		AddNotInList(Verbs.AllowVerbs.List,"BDELETE");		RemoveFrom(Verbs.DenyVerbs.List,"BDELETE");
		AddNotInList(Verbs.AllowVerbs.List,"SUBSCRIBE");	RemoveFrom(Verbs.DenyVerbs.List,"SUBSCRIBE");
		AddNotInList(Verbs.AllowVerbs.List,"UNSUBSCRIBE");	RemoveFrom(Verbs.DenyVerbs.List,"UNSUBSCRIBE");
		AddNotInList(Verbs.AllowVerbs.List,"SUBSCRIPTIONS");RemoveFrom(Verbs.DenyVerbs.List,"SUBSCRIPTIONS");
		AddNotInList(Verbs.AllowVerbs.List,"ACL");			RemoveFrom(Verbs.DenyVerbs.List,"ACL");
		AddNotInList(Verbs.AllowVerbs.List,"NOTIFY");		RemoveFrom(Verbs.DenyVerbs.List,"NOTIFY");

		//Allow these filename extensions (from URLScan)
		EnableASP();
		EnableStaticHTML();

		//Allow these filename extensions (from testing)
		AddNotInList(Extensions.AllowExtensions.List,".css"); RemoveFrom(Extensions.DenyExtensions.List,".css");
		AddNotInList(Extensions.AllowExtensions.List,".js");  RemoveFrom(Extensions.DenyExtensions.List,".js");
		AddNotInList(Extensions.AllowExtensions.List,".htc"); RemoveFrom(Extensions.DenyExtensions.List,".htc");
		AddNotInList(Extensions.AllowExtensions.List,".xsl"); RemoveFrom(Extensions.DenyExtensions.List,".xsl");
		AddNotInList(Extensions.AllowExtensions.List,".eml"); RemoveFrom(Extensions.DenyExtensions.List,".eml");
		AddNotInList(Extensions.AllowExtensions.List,".wav"); RemoveFrom(Extensions.DenyExtensions.List,".wav");

		//Allow # in url
		URL.DenyCharacters.Remove('#');

		//Allow certain characters in path
		Path.DenyCharacters.Remove(':');
		Path.DenyCharacters.Remove('=');
		Path.DenyCharacters.Remove('+');
		Path.DenyCharacters.Remove('$');
		Path.DenyCharacters.Remove('<');
		Path.DenyCharacters.Remove('>');
		Path.DenyCharacters.Remove('"');
		Path.DenyCharacters.Remove('*');
		Path.DenyCharacters.Remove('|');
		Path.DenyCharacters.Remove('#');
		Path.DenyCharacters.Remove('^');

		//Allow certain characters in filename
		Filename.DenyCharacters.Remove(':');
		Filename.DenyCharacters.Remove('=');
		Filename.DenyCharacters.Remove('+');
		Filename.DenyCharacters.Remove('$');
		Filename.DenyCharacters.Remove('<');
		Filename.DenyCharacters.Remove('>');
		Filename.DenyCharacters.Remove('"');
		Filename.DenyCharacters.Remove('*');
		Filename.DenyCharacters.Remove('|');
		Filename.DenyCharacters.Remove('#');
		Filename.DenyCharacters.Remove('^');

	//EXCHANGE 5.5
		//already in exchange 2000 settings
	
		//EnableIISADMPWD(); (for OWA change password in options - not recommended)

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableOutlookMobileAccess()
{
	/* 
	 * Outlook Mobile Access (Exchange 2003), previous called
	 * Microsoft Mobile Information Server 2002
	 * From: http://support.microsoft.com/default.aspx?scid=kb;en-us;823175
	 */
	try{

		//Allow static html
		EnableStaticHTML();

		//Allow embedded encoding (from EnableOWA()) (TODO: check this)
		Global.DenyPostEncodingExploits = false;
		Global.DenyPostSequences.Enabled = false;

		//Allow certain verbs
		AddNotInList(Verbs.AllowVerbs.List,"POST");RemoveFrom(Verbs.DenyVerbs.List,"POST");
		AddNotInList(Verbs.AllowVerbs.List,"PROPFIND");RemoveFrom(Verbs.DenyVerbs.List,"PROPFIND");
		AddNotInList(Verbs.AllowVerbs.List,"PROPPATCH");RemoveFrom(Verbs.DenyVerbs.List,"PROPPATCH");
		AddNotInList(Verbs.AllowVerbs.List,"DELETE");RemoveFrom(Verbs.DenyVerbs.List,"DELETE");
		AddNotInList(Verbs.AllowVerbs.List,"MOVE");RemoveFrom(Verbs.DenyVerbs.List,"MOVE");
		AddNotInList(Verbs.AllowVerbs.List,"SEARCH");RemoveFrom(Verbs.DenyVerbs.List,"SEARCH");
		AddNotInList(Verbs.AllowVerbs.List,"X-MS-ENUMATTS");RemoveFrom(Verbs.DenyVerbs.List,"X-MS-ENUMATTS");	//Found this also in Exchange 2000 SDK on 04-01-04 (new in Exchange 2000 SP2)

		//Allow certain paths
		AddNotInList(Path.AllowPaths.List,"m:\\");
		AddNotInList(Path.AllowPaths.List,"\\.\backofficestorage");
		AddNotInList(Path.AllowPaths.List,"c:\\program files\\exchsrvr\\oma");

		//from urlscan
		Path.DenyDot = false;				//allow dot in path
		URL.DenyHighBitShellcode = false;	//allow high bit characters in url
		URL.DenyAlternateStream = false;	//allow ':' in url
		Referrer.DenyHighBitShellcode = false;

		//Allow certain content-types (TODO: check this)
		AddNotInList(ContentType.AllowContentTypes.List,"");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-UTF8-encoded");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-form-urlencoded");
		AddNotInList(ContentType.AllowContentTypes.List,"multipart/form-data");

		/*
		TODO
		; Deny extensions for Outlook Mobile Access.
		.asax
		.ascs
		.config
		.cs
		.csproj
		.licx
		.pdb
		.resx
		.resources
		.vb
		.vbproj
		.vsdisco
		.webinfo
		.xsd
		.xsx
		//*/

		//Allow # in url
		URL.DenyCharacters.Remove('#');

		//Allow certain characters in path
		Path.DenyCharacters.Remove(':');
		Path.DenyCharacters.Remove('=');
		Path.DenyCharacters.Remove('+');
		Path.DenyCharacters.Remove('$');
		Path.DenyCharacters.Remove('<');
		Path.DenyCharacters.Remove('>');
		Path.DenyCharacters.Remove('"');
		Path.DenyCharacters.Remove('*');
		Path.DenyCharacters.Remove('|');
		Path.DenyCharacters.Remove('#');
		Path.DenyCharacters.Remove('^');

		//Allow certain characters in filename
		Filename.DenyCharacters.Remove(':');
		Filename.DenyCharacters.Remove('=');
		Filename.DenyCharacters.Remove('+');
		Filename.DenyCharacters.Remove('$');
		Filename.DenyCharacters.Remove('<');
		Filename.DenyCharacters.Remove('>');
		Filename.DenyCharacters.Remove('"');
		Filename.DenyCharacters.Remove('*');
		Filename.DenyCharacters.Remove('|');
		Filename.DenyCharacters.Remove('#');
		Filename.DenyCharacters.Remove('^');

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableActiveSync()
{
	try{
		/* 
		 * Enables Exchange 2003 ActiveSync
		 * From: http://support.microsoft.com/default.aspx?scid=kb;en-us;823175#1
		 */

		//Allow static html
		EnableStaticHTML();

		//Allow embedded encoding (from EnableOWA()) (TODO: check this)
		Global.DenyPostEncodingExploits = false;
		Global.DenyPostSequences.Enabled = false;

		//ActiveSync
		AddNotInList(Verbs.AllowVerbs.List,"POST");RemoveFrom(Verbs.DenyVerbs.List,"POST");
		AddNotInList(Verbs.AllowVerbs.List,"PROPFIND");RemoveFrom(Verbs.DenyVerbs.List,"PROPFIND");
		AddNotInList(Verbs.AllowVerbs.List,"PROPPATCH");RemoveFrom(Verbs.DenyVerbs.List,"PROPPATCH");
		AddNotInList(Verbs.AllowVerbs.List,"MKCOL");RemoveFrom(Verbs.DenyVerbs.List,"MKCOL");
		AddNotInList(Verbs.AllowVerbs.List,"DELETE");RemoveFrom(Verbs.DenyVerbs.List,"DELETE");
		AddNotInList(Verbs.AllowVerbs.List,"MOVE");RemoveFrom(Verbs.DenyVerbs.List,"MOVE");
		AddNotInList(Verbs.AllowVerbs.List,"BMOVE");RemoveFrom(Verbs.DenyVerbs.List,"BMOVE");
		AddNotInList(Verbs.AllowVerbs.List,"SEARCH");RemoveFrom(Verbs.DenyVerbs.List,"SEARCH");
		AddNotInList(Verbs.AllowVerbs.List,"PUT");RemoveFrom(Verbs.DenyVerbs.List,"PUT");
		AddNotInList(Verbs.AllowVerbs.List,"OPTIONS");RemoveFrom(Verbs.DenyVerbs.List,"OPTIONS");
		AddNotInList(Verbs.AllowVerbs.List,"X-MS-ENUMATTS");RemoveFrom(Verbs.DenyVerbs.List,"X-MS-ENUMATTS");	//Found this also in Exchange 2000 SDK on 04-01-04 (new in Exchange 2000 SP2)

		//Allow certain paths
		AddNotInList(Path.AllowPaths.List,"m:\\");
		AddNotInList(Path.AllowPaths.List,"\\.\backofficestorage");
		AddNotInList(Path.AllowPaths.List,"c:\\program files\\exchsrvr\\oma\\sync");

		//from urlscan
		Path.DenyDot = false;				//allow dot in path
		URL.DenyHighBitShellcode = false;	//allow high bit characters in url
		URL.DenyAlternateStream = false;	//allow ':' in url
		Referrer.DenyHighBitShellcode = false;

		//Allow certain content-types (TODO: check this)
		AddNotInList(ContentType.AllowContentTypes.List,"");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-UTF8-encoded");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-form-urlencoded");
		AddNotInList(ContentType.AllowContentTypes.List,"multipart/form-data");

		//Allow certain attachments
		AddNotInList(Extensions.AllowExtensions.List,".dll"); RemoveFrom(Extensions.DenyExtensions.List,".dll");	//for ActiveSync

		//Allow these filename extensions (TODO: check this)
		AddNotInList(Extensions.AllowExtensions.List,".css"); RemoveFrom(Extensions.DenyExtensions.List,".css");
		AddNotInList(Extensions.AllowExtensions.List,".js");  RemoveFrom(Extensions.DenyExtensions.List,".js");
		AddNotInList(Extensions.AllowExtensions.List,".htc"); RemoveFrom(Extensions.DenyExtensions.List,".htc");
		AddNotInList(Extensions.AllowExtensions.List,".xsl"); RemoveFrom(Extensions.DenyExtensions.List,".xsl");
		AddNotInList(Extensions.AllowExtensions.List,".eml"); RemoveFrom(Extensions.DenyExtensions.List,".eml");
		AddNotInList(Extensions.AllowExtensions.List,".wav"); RemoveFrom(Extensions.DenyExtensions.List,".wav");

		//Allow # in url
		URL.DenyCharacters.Remove('#');

		//Allow certain characters in path
		Path.DenyCharacters.Remove(':');
		Path.DenyCharacters.Remove('=');
		Path.DenyCharacters.Remove('+');
		Path.DenyCharacters.Remove('$');
		Path.DenyCharacters.Remove('<');
		Path.DenyCharacters.Remove('>');
		Path.DenyCharacters.Remove('"');
		Path.DenyCharacters.Remove('*');
		Path.DenyCharacters.Remove('|');
		Path.DenyCharacters.Remove('#');
		Path.DenyCharacters.Remove('^');

		//Allow certain characters in filename
		Filename.DenyCharacters.Remove(':');
		Filename.DenyCharacters.Remove('=');
		Filename.DenyCharacters.Remove('+');
		Filename.DenyCharacters.Remove('$');
		Filename.DenyCharacters.Remove('<');
		Filename.DenyCharacters.Remove('>');
		Filename.DenyCharacters.Remove('"');
		Filename.DenyCharacters.Remove('*');
		Filename.DenyCharacters.Remove('|');
		Filename.DenyCharacters.Remove('#');
		Filename.DenyCharacters.Remove('^');

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableRPCoverHTTP()
{
	try{

		//MaxContentLength="2000000000";
		//MaxURL=16384;
		//MaxQueryString=4096;

		//Allow virtual dirs
		RemoveFrom(URL.DenySequences.List,"/rpc");
		//Allow proxy dll
		RemoveFrom(Filename.DenyFiles.List,"rpcproxy.dll");

		//Allow all post sequences (TODO: check this)
		Global.DenyPostEncodingExploits = false;
		Global.DenyPostSequences.Enabled = false;
		
		//Allow certain paths
		AddNotInList(Path.AllowPaths.List,"c:\\windows\\system32\\rpcproxy");

		//Allow certain verbs
		AddNotInList(Verbs.AllowVerbs.List,"RPC_OUT_DATA");RemoveFrom(Verbs.DenyVerbs.List,"RPC_OUT_DATA");	//TODO: exploit?
		AddNotInList(Verbs.AllowVerbs.List,"RPC_IN_DATA");RemoveFrom(Verbs.DenyVerbs.List,"RPC_IN_DATA");		//TODO: exploit?

		//Allow certain attachments
		AddNotInList(Extensions.AllowExtensions.List,".dll"); RemoveFrom(Extensions.DenyExtensions.List,".dll");
	
		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableColdfusion()
{
	try{
		//TODO

		//extensions
		AddNotInList(Extensions.AllowExtensions.List,".cfm");		//ColdFusion
		AddNotInList(Extensions.AllowExtensions.List,".cfml");		//ColdFusion

		//Content-Type
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-ColdFusion");
		//AddNotInList(ContentType.AllowContentTypes.Enabled,"application/x-ColdFusionIDE");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableFrontpageExtensions()
{
	try{
		//From urlscan and: http://support.microsoft.com/default.aspx?scid=kb;EN-US;318290

		//AllowLateScanning (from urlscan)
		Engine.AllowLateScanning = true;

		//Allow these headers (from urlscan)
		AddNotInList(Verbs.AllowVerbs.List,"GET");		RemoveFrom(Verbs.DenyVerbs.List,"GET");
		AddNotInList(Verbs.AllowVerbs.List,"HEAD");		RemoveFrom(Verbs.DenyVerbs.List,"HEAD");
		AddNotInList(Verbs.AllowVerbs.List,"POST");		RemoveFrom(Verbs.DenyVerbs.List,"POST");
		AddNotInList(Verbs.AllowVerbs.List,"OPTIONS");	RemoveFrom(Verbs.DenyVerbs.List,"OPTIONS");

		//Allow headers (from URLScan)
		RemoveFrom(Headers.DenyHeaders.List,"Translate:"); //"Translate:f" means: no script execution (is needed for script source access)

		//Allow certain url sequences (TODO: test this what can be left out?)
		//RemoveFrom(Deny_URL_Sequences,"/_vti_aut");
		//RemoveFrom(Deny_URL_Sequences,"/_vti_bin");
		//RemoveFrom(Deny_URL_Sequences,"/_vti_rpc");
		//RemoveFrom(Deny_URL_Sequences,"/_vti_script");
		//RemoveFrom(Deny_URL_Sequences,"/_mem_bin");
		//RemoveFrom(Deny_URL_Sequences,"/admcgi");
		//RemoveFrom(Deny_URL_Sequences,"/admisapi");
		//RemoveFrom(Deny_URL_Sequences,"/fpdb/");
		//RemoveFrom(Deny_URL_Sequences,"/scripts");

		//Allow certain paths
		AddNotInList(Path.AllowPaths.List,"c:\\program files\\common files\\microsoft shared\\web server extensions");

		//Allow certain filenames (TODO: test this what can be left out?)
		RemoveFrom(Filename.DenyFiles.List,"_vti_inf.html");
		//RemoveFrom(Filename.DenyFiles.List,"postinfo.html");  
		//RemoveFrom(Filename.DenyFiles.List,"fp30reg.dll");	 
		//RemoveFrom(Filename.DenyFiles.List,"fp4areg.dll");	 
		//RemoveFrom(Filename.DenyFiles.List,"shtml.dll");		 
		//RemoveFrom(Filename.DenyFiles.List,"shtml.exe");		 
		//RemoveFrom(Filename.DenyFiles.List,"fpsrvadm.exe");	 
		//RemoveFrom(Filename.DenyFiles.List,"fpremadm.exe");	 
		//RemoveFrom(Filename.DenyFiles.List,"fpadmin.htm");	 
		//RemoveFrom(Filename.DenyFiles.List,"fpadmcgi.exe");	 
		//RemoveFrom(Filename.DenyFiles.List,"cfgwiz.exe");	 
		//RemoveFrom(Filename.DenyFiles.List,"authors.pwd");	 
		//RemoveFrom(Filename.DenyFiles.List,"author.exe");	 
		//RemoveFrom(Filename.DenyFiles.List,"author.dll");	 
		//RemoveFrom(Filename.DenyFiles.List,"administrators.pwd"); 
		//RemoveFrom(Filename.DenyFiles.List,"access.cnf");	 
		//RemoveFrom(Filename.DenyFiles.List,"service.cnf");	 
		//RemoveFrom(Filename.DenyFiles.List,"service.pwd");	 
		//RemoveFrom(Filename.DenyFiles.List,"service.stp");	 
		//RemoveFrom(Filename.DenyFiles.List,"services.cnf");	 
		//RemoveFrom(Filename.DenyFiles.List,"svcacl.cnf");	 
		//RemoveFrom(Filename.DenyFiles.List,"users.pwd");		 
		//RemoveFrom(Filename.DenyFiles.List,"writeto.cnf");	 
		//RemoveFrom(Filename.DenyFiles.List,"dvwssr.dll");	 

		//Allow these filename extensions (from URLScan)
		AddNotInList(Extensions.AllowExtensions.List,".htm"); RemoveFrom(Extensions.DenyExtensions.List,".htm");
		AddNotInList(Extensions.AllowExtensions.List,".html");RemoveFrom(Extensions.DenyExtensions.List,".html");
		AddNotInList(Extensions.AllowExtensions.List,".dll"); RemoveFrom(Extensions.DenyExtensions.List,".dll");

		//Allow content-types:
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-vermeer-urlencoded");

		//allow empty URL request (Frontpage 2008 & one previous)
		URL.AllowURL.Enabled = false;

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableFlashMX()
{
	try{
		//TODO

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableWebDAV()
{
	/*
	 * Enable HTTP Extension: WebDAV
	 * See rfc 2518
	 */
	try{
		//Allow WebDAV verbs (from rfc 2518)
		AddNotInList(Verbs.AllowVerbs.List,"PROPFIND");	RemoveFrom(Verbs.DenyVerbs.List,"PROPFIND");
		AddNotInList(Verbs.AllowVerbs.List,"PROPPATCH");	RemoveFrom(Verbs.DenyVerbs.List,"PROPPATCH");
		AddNotInList(Verbs.AllowVerbs.List,"MKCOL");		RemoveFrom(Verbs.DenyVerbs.List,"MKCOL");
		AddNotInList(Verbs.AllowVerbs.List,"GET");		RemoveFrom(Verbs.DenyVerbs.List,"GET");
		AddNotInList(Verbs.AllowVerbs.List,"HEAD");		RemoveFrom(Verbs.DenyVerbs.List,"HEAD");
		AddNotInList(Verbs.AllowVerbs.List,"POST");		RemoveFrom(Verbs.DenyVerbs.List,"POST");
		AddNotInList(Verbs.AllowVerbs.List,"DELETE");	RemoveFrom(Verbs.DenyVerbs.List,"DELETE");
		AddNotInList(Verbs.AllowVerbs.List,"PUT");		RemoveFrom(Verbs.DenyVerbs.List,"PUT");
		AddNotInList(Verbs.AllowVerbs.List,"COPY");		RemoveFrom(Verbs.DenyVerbs.List,"COPY");
		AddNotInList(Verbs.AllowVerbs.List,"MOVE");		RemoveFrom(Verbs.DenyVerbs.List,"MOVE");
		AddNotInList(Verbs.AllowVerbs.List,"LOCK");		RemoveFrom(Verbs.DenyVerbs.List,"LOCK");
		AddNotInList(Verbs.AllowVerbs.List,"UNLOCK");	RemoveFrom(Verbs.DenyVerbs.List,"UNLOCK");
		AddNotInList(Verbs.AllowVerbs.List,"OPTIONS");	RemoveFrom(Verbs.DenyVerbs.List,"OPTIONS");

		//Allow MS-WebDAV verbs (from iishelp)
		AddNotInList(Verbs.AllowVerbs.List,"SEARCH");	RemoveFrom(Verbs.DenyVerbs.List,"SEARCH");

		//Allow all POST sequences
		Global.DenyPostEncodingExploits = false;
		Global.DenyPostSequences.Enabled = false;

		//Allow WebDAV headers (from rfc 2518)
		RemoveFrom(Headers.DenyHeaders.List,"Depth:");
		RemoveFrom(Headers.DenyHeaders.List,"Destination:");
		RemoveFrom(Headers.DenyHeaders.List,"If:");
		RemoveFrom(Headers.DenyHeaders.List,"Lock-Token:");
		RemoveFrom(Headers.DenyHeaders.List,"Overwrite:");
		RemoveFrom(Headers.DenyHeaders.List,"Timeout:");
		//Allow MS-WebDAV headers (from testing)
		RemoveFrom(Headers.DenyHeaders.List,"Translate:"); //"Translate:f" means: no script execution (is needed for script source access)

		//Allow text/xml content type (from rfc 2518);
		AddNotInList(ContentType.AllowContentTypes.List,"text/xml");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableIISADMPWD()
{
	try{
		//Allow virtual dir /iisadmpwd
		RemoveFrom(URL.DenySequences.List,"/iisadmpwd");
		
		//Allow certain filenames
		RemoveFrom(Filename.DenyFiles.List,"achg.htr");
		RemoveFrom(Filename.DenyFiles.List,"anot.htr");
		
		//Allow .htr extension
		AddNotInList(Extensions.AllowExtensions.List,".htr"); RemoveFrom(Extensions.DenyExtensions.List,".htr");
		
		//Allow iisadmpwd path
		AddNotInList(Path.AllowPaths.List,"c:\\winnt\\system32\\inetsrv\\iisadmpwd");

		//Allow certain verbs
		AddNotInList(Verbs.AllowVerbs.List,"GET");		RemoveFrom(Verbs.DenyVerbs.List,"GET");
		AddNotInList(Verbs.AllowVerbs.List,"HEAD");		RemoveFrom(Verbs.DenyVerbs.List,"HEAD");
		AddNotInList(Verbs.AllowVerbs.List,"POST");		RemoveFrom(Verbs.DenyVerbs.List,"POST");

		//Allow certain content types
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-form-urlencoded");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableSharePointPortalServer()
{
	try{

		EnableWebDAV();	//uses WebDAV

		//verbs not in WebDAV (from urlscan)
		AddNotInList(Verbs.AllowVerbs.List,"SUBSCRIBE");	RemoveFrom(Verbs.DenyVerbs.List,"SUBSCRIBE");
		AddNotInList(Verbs.AllowVerbs.List,"INVOKE");	RemoveFrom(Verbs.DenyVerbs.List,"INVOKE");
		AddNotInList(Verbs.AllowVerbs.List,"POLL");		RemoveFrom(Verbs.DenyVerbs.List,"POLL");
		AddNotInList(Verbs.AllowVerbs.List,"UNSUBSCRIBE");RemoveFrom(Verbs.DenyVerbs.List,"INVOKE");
		AddNotInList(Verbs.AllowVerbs.List,"TRANSLATE");	RemoveFrom(Verbs.DenyVerbs.List,"TRANSLATE");

		//Allow certain content-types
		AddNotInList(ContentType.AllowContentTypes.List,"");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-form-urlencoded");
		
		//Allow these filename extensions (from URLScan)
		EnableASP();
		EnableStaticHTML();

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableSharePointTeamServices()
{
	try{

		EnableFrontpageExtensions();	//uses frontpage

		//Allow certain content-types
		AddNotInList(ContentType.AllowContentTypes.List,"");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-form-urlencoded");
		
		//Allow these filename extensions (from URLScan)
		EnableASP();
		EnableStaticHTML();
		AddNotInList(Extensions.AllowExtensions.List,".htw"); RemoveFrom(Extensions.DenyExtensions.List,".htw");
		AddNotInList(Extensions.AllowExtensions.List,".ida"); RemoveFrom(Extensions.DenyExtensions.List,".ida");
		AddNotInList(Extensions.AllowExtensions.List,".idq"); RemoveFrom(Extensions.DenyExtensions.List,".idq");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableCertificateServices()
{
	try{
		//Allow virtual dirs
		RemoveFrom(URL.DenySequences.List,"/certsrv");
		RemoveFrom(URL.DenySequences.List,"/certcontrol");
		RemoveFrom(URL.DenySequences.List,"/certenroll");

		//Allow path
		AddNotInList(Path.AllowPaths.List,"c:\\winnt\\system32\\certsrv");

		//Allow these verbs
		AddNotInList(Verbs.AllowVerbs.List,"GET");		RemoveFrom(Verbs.DenyVerbs.List,"GET");
		AddNotInList(Verbs.AllowVerbs.List,"HEAD");		RemoveFrom(Verbs.DenyVerbs.List,"HEAD");
		AddNotInList(Verbs.AllowVerbs.List,"POST");		RemoveFrom(Verbs.DenyVerbs.List,"POST");

		//Allow certain content-types
		AddNotInList(ContentType.AllowContentTypes.List,"");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-form-urlencoded");
		
		//Allow these filename extensions (from testing)
		EnableASP();
		AddNotInList(Extensions.AllowExtensions.List,".cer"); RemoveFrom(Extensions.DenyExtensions.List,".cer"); //Certificate (X.509)
		AddNotInList(Extensions.AllowExtensions.List,".gif"); RemoveFrom(Extensions.DenyExtensions.List,".gif");
		AddNotInList(Extensions.AllowExtensions.List,".p7b"); RemoveFrom(Extensions.DenyExtensions.List,".p7b"); //Certificate PKCS#7
		AddNotInList(Extensions.AllowExtensions.List,".pfx"); RemoveFrom(Extensions.DenyExtensions.List,".pfx"); //Certificate PKCS#12
		AddNotInList(Extensions.AllowExtensions.List,".crl"); RemoveFrom(Extensions.DenyExtensions.List,".crl"); //revocation list
		AddNotInList(Extensions.AllowExtensions.List,".crt"); RemoveFrom(Extensions.DenyExtensions.List,".crt"); //authority
		AddNotInList(Extensions.AllowExtensions.List,".cab"); RemoveFrom(Extensions.DenyExtensions.List,".cab");
		AddNotInList(Extensions.AllowExtensions.List,".dll"); RemoveFrom(Extensions.DenyExtensions.List,".dll");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableBizTalkServer()
{
	try{
		EnableWebDAV(); //uses WebDAV

		//Allow virtual dirs
		RemoveFrom(URL.DenySequences.List,"/biztalkserverdocs");
		RemoveFrom(URL.DenySequences.List,"/biztalkserverrepository");
		RemoveFrom(URL.DenySequences.List,"/messagingmanager");
		RemoveFrom(URL.DenySequences.List,"/biztalktracking");

		//Allow path
		AddNotInList(Path.AllowPaths.List,"c:\\program files\\microsoft biztalk server\\docs");
		AddNotInList(Path.AllowPaths.List,"c:\\program files\\microsoft biztalk server\\biztalkserverrepository");
		AddNotInList(Path.AllowPaths.List,"c:\\program files\\microsoft biztalk server\\messagingmanager");
		AddNotInList(Path.AllowPaths.List,"c:\\program files\\microsoft biztalk server\\biztalktracking");

		//Allow certain content-types
		AddNotInList(ContentType.AllowContentTypes.List,"");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-form-urlencoded");
		
		//Allow these filename extensions (from research TODO: test this)
		EnableASP();
		EnableStaticHTML();
		AddNotInList(Extensions.AllowExtensions.List,".css"); RemoveFrom(Extensions.DenyExtensions.List,".css");
		AddNotInList(Extensions.AllowExtensions.List,".js");  RemoveFrom(Extensions.DenyExtensions.List,".js");
		//AddNotInList(Extensions.AllowExtensions.List,".xsl"); RemoveFrom(Extensions.DenyExtensions.List,".xsl");
		//AddNotInList(Extensions.AllowExtensions.List,".xml"); RemoveFrom(Extensions.DenyExtensions.List,".xml");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableCommerceServer()
{
	try{
		//TODO: requires SOAP? OWA? WebDAV?
		//TODO: check all those that are commented out

		//Allow virtual dir
		RemoveFrom(URL.DenySequences.List,"/widgets");
		//RemoveFrom(URL.DenySequences.List,"/retail");
		//RemoveFrom(URL.DenySequences.List,"/retailbizdesk");
		//RemoveFrom(URL.DenySequences.List,"/supplier");
		
		//Allow certain paths (from research)
		AddNotInList(Path.AllowPaths.List,"c:\\inetpub\\wwwroot");
		//AddNotInList(Path.AllowPaths.List,"c:\\program files\\microsoft commerce server"); //TODO: needed & subdirs?
		//AddNotInList(Path.AllowPaths.List,"c:\\program files\\microsoft commerce server\\widgets"); //TODO: needed?

		//allow verbs (from urlscan: HEAD not allowed!?)
		AddNotInList(Verbs.AllowVerbs.List,"GET");	RemoveFrom(Verbs.DenyVerbs.List,"GET");
		AddNotInList(Verbs.AllowVerbs.List,"POST");	RemoveFrom(Verbs.DenyVerbs.List,"POST");
		AddNotInList(Verbs.AllowVerbs.List,"PUT");	RemoveFrom(Verbs.DenyVerbs.List,"PUT");

		//Allow certain headers (from urlscan)
		RemoveFrom(Headers.DenyHeaders.List,"Translate:");
		RemoveFrom(Headers.DenyHeaders.List,"If:");
		RemoveFrom(Headers.DenyHeaders.List,"Lock-Token:");

		//Allow certain content-types
		AddNotInList(ContentType.AllowContentTypes.List,"");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-form-urlencoded");
		
		//Allow these filename extensions (from URLScan)
		EnableASP();
		EnableStaticHTML();

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableSmallBusinessServer()
{
	try{
		EnableOutlookWebAccess();

		return true;
	}catch(...){
		return false;
	}
}


bool CWebKnightSettings::EnableASPNET()
{
	try{
		//TODO: error Niels?
		//TODO: test this

		//allow verbs
		AddNotInList(Verbs.AllowVerbs.List,"GET");	RemoveFrom(Verbs.DenyVerbs.List,"GET");
		AddNotInList(Verbs.AllowVerbs.List,"HEAD");	RemoveFrom(Verbs.DenyVerbs.List,"HEAD");
		AddNotInList(Verbs.AllowVerbs.List,"POST");	RemoveFrom(Verbs.DenyVerbs.List,"POST");
		AddNotInList(Verbs.AllowVerbs.List,"DEBUG");	RemoveFrom(Verbs.DenyVerbs.List,"DEBUG");

		//Allow certain content-types
		AddNotInList(ContentType.AllowContentTypes.List,"");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-form-urlencoded");

		//enable SOAP
		EnableSOAP();

		//Allow certain filenames
		RemoveFrom(Filename.DenyFiles.List,"trace.axd");

		//allow these extensions
		//AddNotInList(Extensions.AllowExtensions.List,".asax");	RemoveFrom(Extensions.DenyExtensions.List,".asax");		//asa
		//AddNotInList(Extensions.AllowExtensions.List,".asa");	RemoveFrom(Extensions.DenyExtensions.List,".asa");			//".asax" contains ".asa"
		//AddNotInList(Extensions.AllowExtensions.List,".ascx");	RemoveFrom(Extensions.DenyExtensions.List,".ascx");		//forbidden by asp.net (custom user controls)
		AddNotInList(Extensions.AllowExtensions.List,".ashx");	RemoveFrom(Extensions.DenyExtensions.List,".ashx");		//custom httphandler
		AddNotInList(Extensions.AllowExtensions.List,".asmx");	RemoveFrom(Extensions.DenyExtensions.List,".asmx");		//web service
		AddNotInList(Extensions.AllowExtensions.List,".aspx");	RemoveFrom(Extensions.DenyExtensions.List,".aspx");		//asp
		AddNotInList(Extensions.AllowExtensions.List,".asp");		RemoveFrom(Extensions.DenyExtensions.List,".asp");			//".aspx" contains ".asp"
		AddNotInList(Extensions.AllowExtensions.List,".axd");		RemoveFrom(Extensions.DenyExtensions.List,".axd");			//WebResources (possible information disclosure)
		//AddNotInList(Extensions.AllowExtensions.List,".config");RemoveFrom(Extensions.DenyExtensions.List,".config");		//forbidden by asp.net (config)
		//AddNotInList(Extensions.AllowExtensions.List,".cs");	RemoveFrom(Extensions.DenyExtensions.List,".cs");			//forbidden by asp.net (c#)
		//AddNotInList(Extensions.AllowExtensions.List,".csproj");RemoveFrom(Extensions.DenyExtensions.List,".csproj");		//forbidden by asp.net (c# project)
		//AddNotInList(Extensions.AllowExtensions.List,".licx");	RemoveFrom(Extensions.DenyExtensions.List,".licx");		//forbidden by asp.net (licenses)
		AddNotInList(Extensions.AllowExtensions.List,".rem");		RemoveFrom(Extensions.DenyExtensions.List,".rem");			//remoting
		//AddNotInList(Extensions.AllowExtensions.List,".resources");RemoveFrom(Extensions.DenyExtensions.List,".resources");//forbidden by asp.net (resources)
		//AddNotInList(Extensions.AllowExtensions.List,".resx");	RemoveFrom(Extensions.DenyExtensions.List,".resx");		//forbidden by asp.net (resources in xml file)
		//AddNotInList(Extensions.AllowExtensions.List,".soap");	RemoveFrom(Extensions.DenyExtensions.List,".soap");		//remoting & handling soap requests //done by EnableSOAP()
		//AddNotInList(Extensions.AllowExtensions.List,".vb");	RemoveFrom(Extensions.DenyExtensions.List,".vb");			//forbidden by asp.net (vb code)
		//AddNotInList(Extensions.AllowExtensions.List,".vbproj");RemoveFrom(Extensions.DenyExtensions.List,".vbproj");		//forbidden by asp.net (vb project)
		//AddNotInList(Extensions.AllowExtensions.List,".vsdisco");RemoveFrom(Extensions.DenyExtensions.List,".vsdisco");	//dynamic discovery: security issue!!!! http://msdn.microsoft.com/msdnmag/issues/02/08/XMLFiles/default.aspx
		//AddNotInList(Extensions.AllowExtensions.List,".webinfo");RemoveFrom(Extensions.DenyExtensions.List,".webinfo");	//forbidden by asp.net

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableStaticHTML()
{
	try{

		//Allow these verbs
		AddNotInList(Verbs.AllowVerbs.List,"GET");		RemoveFrom(Verbs.DenyVerbs.List,"GET");
		AddNotInList(Verbs.AllowVerbs.List,"HEAD");		RemoveFrom(Verbs.DenyVerbs.List,"HEAD");
		
		//Allow these extensions
		AddNotInList(Extensions.AllowExtensions.List,".htm"); RemoveFrom(Extensions.DenyExtensions.List,".htm");
		AddNotInList(Extensions.AllowExtensions.List,".html");RemoveFrom(Extensions.DenyExtensions.List,".html");
		AddNotInList(Extensions.AllowExtensions.List,".txt"); RemoveFrom(Extensions.DenyExtensions.List,".txt");
		AddNotInList(Extensions.AllowExtensions.List,".jpg"); RemoveFrom(Extensions.DenyExtensions.List,".jpg");
		AddNotInList(Extensions.AllowExtensions.List,".jpeg");RemoveFrom(Extensions.DenyExtensions.List,".jpeg");
		AddNotInList(Extensions.AllowExtensions.List,".gif"); RemoveFrom(Extensions.DenyExtensions.List,".gif");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableASP()
{
	try{

		//Allow these verbs
		AddNotInList(Verbs.AllowVerbs.List,"GET");		RemoveFrom(Verbs.DenyVerbs.List,"GET");
		AddNotInList(Verbs.AllowVerbs.List,"HEAD");		RemoveFrom(Verbs.DenyVerbs.List,"HEAD");
		AddNotInList(Verbs.AllowVerbs.List,"POST");		RemoveFrom(Verbs.DenyVerbs.List,"POST");
		//AddNotInList(Verbs.AllowVerbs.List,"TRACE");		RemoveFrom(Verbs.DenyVerbs.List,"TRACE");

		//Allow certain content-types
		AddNotInList(ContentType.AllowContentTypes.List,"");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-form-urlencoded");

		//Allow these extensions
		AddNotInList(Extensions.AllowExtensions.List,".asp"); RemoveFrom(Extensions.DenyExtensions.List,".asp");
		AddNotInList(Extensions.AllowExtensions.List,".cer"); RemoveFrom(Extensions.DenyExtensions.List,".cer");
		AddNotInList(Extensions.AllowExtensions.List,".cdx"); RemoveFrom(Extensions.DenyExtensions.List,".cdx");
		//AddNotInList(Extensions.AllowExtensions.List,".asa"); RemoveFrom(Extensions.DenyExtensions.List,".asa");

		return true;
	}catch(...){
		return false;
	}
}


bool CWebKnightSettings::EnablePHP()
{
	try{

		//Allow these verbs
		AddNotInList(Verbs.AllowVerbs.List,"GET");		RemoveFrom(Verbs.DenyVerbs.List,"GET");
		AddNotInList(Verbs.AllowVerbs.List,"HEAD");		RemoveFrom(Verbs.DenyVerbs.List,"HEAD");
		AddNotInList(Verbs.AllowVerbs.List,"POST");		RemoveFrom(Verbs.DenyVerbs.List,"POST");
		//AddNotInList(Verbs.AllowVerbs.List,"TRACE");		RemoveFrom(Verbs.DenyVerbs.List,"TRACE");

		//Allow these extensions
		AddNotInList(Extensions.AllowExtensions.List,".php"); RemoveFrom(Extensions.DenyExtensions.List,".php");
		//TODO

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableBITS()
{
	try{

		//Allow these verbs
		//TODO: check list
		AddNotInList(Verbs.AllowVerbs.List,"GET");		RemoveFrom(Verbs.DenyVerbs.List,"GET");
		AddNotInList(Verbs.AllowVerbs.List,"HEAD");		RemoveFrom(Verbs.DenyVerbs.List,"HEAD");
		AddNotInList(Verbs.AllowVerbs.List,"POST");		RemoveFrom(Verbs.DenyVerbs.List,"POST");
		AddNotInList(Verbs.AllowVerbs.List,"BITS_POST");RemoveFrom(Verbs.DenyVerbs.List,"BITS_POST");	//BITS uploads

		//Allow certain content-types
		//TODO: check list
		AddNotInList(ContentType.AllowContentTypes.List,"");
		AddNotInList(ContentType.AllowContentTypes.List,"multipart/byteranges");
		AddNotInList(ContentType.AllowContentTypes.List,"multipart/x-byteranges");
	
		//Allow certain headers
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Packet-Type:");				//BITS upload REQUEST/RESPONSE
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Protocol:");					//BITS upload		  RESPONSE
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Session-Id:");				//BITS upload REQUEST/RESPONSE
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Supported-Protocols:");		//BITS upload REQUEST
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Host-Id:");					//BITS upload		  RESPONSE
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Host-Id-Fallback-Timeout:");	//BITS upload		  RESPONSE
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Received-Content-Range:");	//BITS upload		  RESPONSE
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Reply-URL:");					//BITS upload		  RESPONSE
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Error-Code:");				//BITS upload		  RESPONSE
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Error-Context:");				//BITS upload		  RESPONSE

		RemoveFrom(Headers.DenyHeaders.List,"BITS-Original-Request-URL:");		//BITS Notification REQUEST
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Request-DataFile-Name:");		//BITS Notification REQUEST
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Response-DataFile-Name:");	//BITS Notification REQUEST
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Static-Response-URL:");		//BITS Notification			RESPONSE
		RemoveFrom(Headers.DenyHeaders.List,"BITS-Copy-File-To-Destination:");	//BITS Notification			RESPONSE

		RemoveFrom(Headers.DenyHeaders.List,"Accept-Encoding:");	//BITS REQUEST/RESPONSE
		RemoveFrom(Headers.DenyHeaders.List,"Content-Name:");		//BITS REQUEST
		RemoveFrom(Headers.DenyHeaders.List,"Content-Length:");		//BITS REQUEST/RESPONSE
		RemoveFrom(Headers.DenyHeaders.List,"Content-Range:");		//BITS REQUEST
		RemoveFrom(Headers.DenyHeaders.List,"Content-Encoding:");	//BITS REQUEST

		return true;
	}catch(...){
		return false;
	}
}

void CWebKnightSettings::LoadAgents()
{
#define CWEBKNIGHTSETTINGS_LOADROBOTS(name) if(Agents.Block##name){\
	if(!Agents.AgentList.UserAgents##name.IsEmpty()){\
		UserAgent.DenyUserAgents.Enabled = true;\
	}\
	if(!Agents.AgentList.UserAgents##name##Sequences.IsEmpty()){\
		UserAgent.DenySequences.Enabled = true;\
	}\
	if(!Agents.AgentList.IPAddresses##name.IsEmpty()){\
		Connection.DenyAddresses.Enabled = true;\
	}\
	AddNotInList(UserAgent.DenyUserAgents.List,Agents.AgentList.UserAgents##name);\
	AddNotInListLCase(UserAgent.DenySequences.List,Agents.AgentList.UserAgents##name##Sequences);\
	AddNotInList(Connection.DenyAddresses.List,Agents.AgentList.IPAddresses##name);}

	//BOTS DATABASE
	CWEBKNIGHTSETTINGS_LOADROBOTS(DataMiningCommercial)
	CWEBKNIGHTSETTINGS_LOADROBOTS(DataMiningPublic)
	CWEBKNIGHTSETTINGS_LOADROBOTS(DownloadManagers)
	CWEBKNIGHTSETTINGS_LOADROBOTS(EmailHarvesting)
	CWEBKNIGHTSETTINGS_LOADROBOTS(GuestbookSpammers)
	CWEBKNIGHTSETTINGS_LOADROBOTS(HackTools)
	CWEBKNIGHTSETTINGS_LOADROBOTS(ImageDownloaders)
	CWEBKNIGHTSETTINGS_LOADROBOTS(Indexing)
	CWEBKNIGHTSETTINGS_LOADROBOTS(Monitoring)
	CWEBKNIGHTSETTINGS_LOADROBOTS(OfflineBrowsers)
	CWEBKNIGHTSETTINGS_LOADROBOTS(OtherBad)
	CWEBKNIGHTSETTINGS_LOADROBOTS(Trademark)
	CWEBKNIGHTSETTINGS_LOADROBOTS(ValidationTools)
	CWEBKNIGHTSETTINGS_LOADROBOTS(LinkChecking)
	CWEBKNIGHTSETTINGS_LOADROBOTS(Browsers)
	CWEBKNIGHTSETTINGS_LOADROBOTS(MediaPlayers)
	CWEBKNIGHTSETTINGS_LOADROBOTS(Proxies)
	CWEBKNIGHTSETTINGS_LOADROBOTS(Adware)
	CWEBKNIGHTSETTINGS_LOADROBOTS(BrowserExtensions)
	CWEBKNIGHTSETTINGS_LOADROBOTS(Spyware)
	CWEBKNIGHTSETTINGS_LOADROBOTS(Editing)
	CWEBKNIGHTSETTINGS_LOADROBOTS(Device)
	CWEBKNIGHTSETTINGS_LOADROBOTS(NewsFeed)
	CWEBKNIGHTSETTINGS_LOADROBOTS(SearchEngines)
	CWEBKNIGHTSETTINGS_LOADROBOTS(FilteringSoftware)
	CWEBKNIGHTSETTINGS_LOADROBOTS(SoftwareComponent)
}

bool CWebKnightSettings::CheckConstraints()
{
	return CHTTPFirewallSettings::CheckConstraints();
}

bool CWebKnightSettings::EnableIPP()
{
	try{
		//Internet Printing Protocol

		//Verbs
		AddNotInList(Verbs.AllowVerbs.List,"GET");		RemoveFrom(Verbs.DenyVerbs.List,"GET");
		AddNotInList(Verbs.AllowVerbs.List,"POST");		RemoveFrom(Verbs.DenyVerbs.List,"POST");

		//URL
		RemoveFrom(URL.DenySequences.List,"/ipp");
		RemoveFrom(URL.DenySequences.List,"/ipp/");

		//Content-Type
		AddNotInList(ContentType.AllowContentTypes.List,"application/ipp");

		//Headers
		RemoveFrom(Headers.DenyHeaders.List,"Transfer-Encoding:");	//Transfer-Encoding: chunked


		return true;
	}catch(...){
		return false;
	}
}

CString CWebKnightSettings::ToXML()
{
	CString xml;

	CWebKnightSettings d; //defaults
	xml += GetXMLScanningEngine(d);
	xml += GetXMLResponseHandling(d);
	xml += GetXMLLogging(d);
	xml += GetXMLConnectionControl(d);
	xml += GetXMLAuthentication(d);
	xml += GetXMLRequestLimits(d);
	xml += GetXMLURLScanning(d);
	xml += GetXMLMappedPath(d);
	xml += GetXMLRequestedFile(d);
	xml += GetXMLRequestedExtension(d);

	xml += GetXMLWebRobots(d);	//web robots
	
	xml += GetXMLHTTPHeaders(d);
	xml += GetXMLContentType(d);
	xml += GetXMLCookie(d);
	xml += GetXMLUserAgent(d);
	xml += GetXMLReferrer(d);
	xml += GetXMLHTTPVerbs(d);
	xml += GetXMLQuerystringScanning(d);
	xml += GetXMLGlobalFilterCapabilities(d);
	xml += GetXMLSQLInjection(d);

	xml += GetXMLWebApplications(d);	//web applications

	return xml;
}

CString CWebKnightSettings::GetXMLWebApplications(CWebKnightSettings& d)
{
#define CWEBKNIGHTSETTINGS_GETXML_WEBAPP(var,name,explanation) \
	xml += 	ConvertStringToXML("Allow_" #name,"Option",WebApp.Allow_##var?"1":"0",d.WebApp.Allow_##var?"1":"0",explanation);

	CString xml("");
	//Web Applications
	xml += "<Web_Applications App='Separator'/>\r\n";
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(FileUploads,File_Uploads,"Allows file uploads to your server using the HTTP POST command.")
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(Unicode,Unicode,"Allows Unicode encoding in the urls and other data sent to the server.")
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(OutlookWebAccess,Outlook_Web_Access,"Allows Outlook Web Access. This changes other settings so that OWA is enabled. This reduces the security of your system and it is not recommended that you run OWA as a virtual directory ('/Exchange'). It is better to assign a web site to OWA (Use MMC of Exchange Server: add HTTP server) and exclude this web instance from scanning!")
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(OutlookMobileAccess,Outlook_Mobile_Access,"Allows Outlook Mobile Access. This changes other settings so that OMA is enabled. Outlook Mobile Access is the successor of Mobile Information Server 2002 (MIS) and now comes with Microsoft Exchange Server. It enables access to Exchange Server from XHTML (WAP 2.x), and CHTML-based microbrowsers.")
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(ActiveSync,ActiveSync,"Allows Microsoft ActiveSync. ActiveSync is used for connection with Pocket PCs and similar devices that have Microsoft ActiveSync client software installed. One example is access from Pocket PCs to Exchange Server via Exchange Server ActiveSync.");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(RPCoverHTTP,RPC_over_HTTP,"Allows RPC over HTTP Proxy. RPC over HTTP was first introduced in Windows 2003 and Windows XP SP1. It allows RPC connections over an HTTP connection. Exchange 2003 uses this feature for direct remote access from Outlook without a VPN.");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(FrontpageExtensions,Frontpage_Extensions,"Allows Frontpage Extensions. This changes other settings so that the firewall will not block Frontpage Extensions. Enabling this reduces the security of your system and make sure you have the latest version of Frontpage installed and keep up with security patches!");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(Coldfusion,Coldfusion,"Allows Coldfusion. This changes other settings so that the firewall will not block these requests. This reduces the security of your system and you should follow security practices of Coldfusion and keep up with security fixes!");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(FlashMX,Flash_MX,"Allows Flash MX. This changes other settings so that the firewall will not block these requests. This reduces the security of your system and you should follow security practices of Flash MX and keep up with security fixes!");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(WebDAV,WebDAV,"Allows WebDAV. WebDAV is an HTTP extension for Distributed Authoring and Versioning. This changes other settings so that the firewall will not block these requests. This reduces the security of your system!");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(IISADMPWD,IISADMPWD,"Allows IISADMPWD. IISADMPWD is a virtual directory that allows users to change their domain/local password over the HTTP protocol. Outlook Web Access can use this feature. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(SharePointPortalServer,SharePoint_Portal_Server,"Allows SharePoint Portal Server. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(SharePointTeamServices,SharePoint_Team_Services,"Allows SharePoint Team Services. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(CertificateServices,Certificate_Services,"Allows Certificate Services Web Interface. Certificate Services installs a virtual directory in your default web site for managing certificates via your browser. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(BizTalkServer,BizTalk_Server,"Allows BizTalk Server. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(CommerceServer,Commerce_Server,"Allows Commerce Server. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(SmallBusinessServer,Small_Business_Server,"Allows Small Business Server. This is the same as enabling Outlook Web Access. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(ASPNET,ASP_NET,"Allows all features of ASP.NET. By default ASP.NET is partially enabled. You should select this option only if you really need debugging, tracing, remoting and SOAP for your ASP.NET.");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(ASP,ASP,"Allows Active Server Pages 3.0 (and previous versions). By default ASP is fully enabled, but if you changed the default settings and disabled ASP then you can use this option to re-enable ASP.");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(PHP,PHP,"Allows PHP. Use this to allow PHP isapi extension.");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(BITS,BITS,"Allows Background Intelligent Transfer Service (BITS). BITS uses an ISAPI to extend IIS to support upload jobs. Use this to enable this isapi extension.");
	CWEBKNIGHTSETTINGS_GETXML_WEBAPP(SOAP,SOAP,"Allows SOAP. By default SOAP is blocked. Enabling this changes other settings so that the firewall will not block these requests.");
	return xml;
}

CString CWebKnightSettings::GetXMLWebRobots(CWebKnightSettings& d)
{
	CString xml("");
	//BOTS
	xml += "<Robots App='Separator'/>\r\n";
	WRITE_XML_BOOL(Robots,AllowRobotsFile,"Allow_Bots_Robots_File","Allow requests for the file 'robots.txt', even for blocked robots. This is recommended because if the file robots.txt cannot be obtained, the robot thinks it has access and you have no other way to tell the robot that it is not allowed.")
	WRITE_XML_BOOL(Robots,DenyAll,"Deny_Bots_All","Deny requests from all bots. This is done by looking at the requests for the robots.txt file. Blocking is done by the combination of IP address and User Agent.")
	WRITE_XML_BOOL(Robots,DenyBad,"Deny_Bots_Bad","Deny requests from bad bots. Add the bot trap urls to your robots.txt file (you can find a sample robots.txt with this installation). Now, to lure a bad bot into those urls, add these urls with hidden anchors in your web site (<a href=/badbottrap/></a>). Blocking is done by the combination of IP address and User Agent.")
	WRITE_XML_CSTRINGLIST(Robots,BadBotTraps,"Deny_Bots_BotTraps","Lowercase and no ending slash preferred to catch all the bad bots. Add these urls to your robots.txt:\r\nUser-agent: *\r\nDisallow: /badbottrap/")
	WRITE_XML_CACHE(Robots,DenyAggressive,"Deny_Bots_Aggressive","Deny aggressive bots doing more than a certain amount of requests in a certain amount of time after their initial request for robots.txt.","The amount of requests to block the bot.","The time frame in which the requests are counted (in minutes).")
	WRITE_XML_INT(Robots,BotTimeout,"Deny_Bots_Timeout","The time-out (in hours) to block the bots. Blocking is done by looking at the IP address and User Agent.")

	//BOTS DATABASE
#define CWEBKNIGHTSETTINGS_GETXML_BOTS(var,name,explanation) \
	xml += 	ConvertStringToXML("Block_Bots_" #name,"Option",Agents.Block##var?"1":"0",d.Agents.Block##var?"1":"0",explanation##" This is done by looking at known user agents and/or IP address defined in Robots.xml.");

	CWEBKNIGHTSETTINGS_GETXML_BOTS(DataMiningCommercial,Data_Mining_Commercial,"Blocks commercial datamining robots.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(DataMiningPublic,Data_Mining_Public,"Blocks non-profit or public datamining robots.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(DownloadManagers,Download_Managers,"Blocks download managers.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(EmailHarvesting,Email_Harvesting,"Blocks email harvesting robots.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(GuestbookSpammers,Guestbook_Spammers,"Blocks guestbook spamming robots.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(HackTools,Hack_Tools,"Blocks certain hacking tools.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(ImageDownloaders,Image_Downloaders,"Blocks image download tools/robots.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(Indexing,Indexing,"Blocks indexing robots.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(Monitoring,Monitoring,"Blocks monitoring robots.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(OfflineBrowsers,Offline_Browsers,"Blocks offline browsers.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(OtherBad,Other_Bad,"Blocks other bad robots.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(Trademark,Trademark,"Blocks copyright/trademark robots.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(ValidationTools,Validation_Tools,"Blocks certain validation tools.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(LinkChecking,Link_Checking,"Blocks URL checking utilities.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(Browsers,Browsers,"Blocks browsers.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(MediaPlayers,Media_Players,"Blocks media players.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(Proxies,Proxies,"Blocks proxy servers.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(Adware,Adware,"Blocks adware.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(BrowserExtensions,Browser_Extensions,"Blocks browser extensions.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(Spyware,Spyware,"Blocks spyware.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(Editing,Editing,"Blocks web/html editing software.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(Device,Device,"Blocks devices.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(NewsFeed,News_Feed,"Blocks news feed utilities.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(SearchEngines,Search_Engines,"Blocks search engines.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(FilteringSoftware,Filtering_Software,"Blocks filtering software.")
	CWEBKNIGHTSETTINGS_GETXML_BOTS(SoftwareComponent,Software_Component,"Blocks certain software components.")	
	return xml;
}

bool CWebKnightSettings::EnableUnicode()
{
	try{

		//MS unicode url encoding (%u)
		RemoveFrom(QueryString.DenySequences.List,"%u");
		RemoveFrom(Headers.DenySequences.List,"%u");
		RemoveFrom(Global.DenyPostSequences.List,"%u");
		RemoveFrom(URL.DenySequences.List,"%u");	 //not in there by default, but still remove it
		RemoveFrom(Referrer.DenySequences.List,"%u");//not in there by default, but still remove it

		//TODO: allow %00 (for utf: %xx%xx and M$ %uxxxx:)
		//TODO: decoding aanpassen: replace %u0000 with %00%00 before urldecoding
		//TODO: allow hex encoding in the url
		//TODO: embedded encoding %25xx
		//TODO: http://www.eeye.com/html/Research/Advisories/AD20010705.html
		// http://home.tiscali.nl/t876506/utf8tbl.html

		
		//high bit shellcode
		URL.DenyHighBitShellcode = false;
		QueryString.DenyHighBitShellcode = false;
		Referrer.DenyHighBitShellcode = false;
		Headers.DenyHighBitShellcode = false;
		Global.DenyPostHighBitShellcode = false;

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableFileUploads()
{
	try{

		//allow verbs
		AddNotInList(Verbs.AllowVerbs.List,"POST");	RemoveFrom(Verbs.DenyVerbs.List,"POST");

		//Allow certain content-types
		AddNotInList(ContentType.AllowContentTypes.List,"");
		AddNotInList(ContentType.AllowContentTypes.List,"application/x-www-form-urlencoded");
		AddNotInList(ContentType.AllowContentTypes.List,"multipart/form-data");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableSOAP()
{
	try{

		//VERB
		AddNotInList(Verbs.AllowVerbs.List,"GET");	RemoveFrom(Verbs.DenyVerbs.List,"GET");
		AddNotInList(Verbs.AllowVerbs.List,"POST");	RemoveFrom(Verbs.DenyVerbs.List,"POST");
		//Content-Type
		AddNotInList(ContentType.AllowContentTypes.List,"text/xml");
		//Headers
		RemoveFrom(Headers.DenyHeaders.List,"SOAPAction:");
		//requested files
		AddNotInList(Extensions.AllowExtensions.List,".soap");	RemoveFrom(Extensions.DenyExtensions.List,".soap");		//remoting & handling soap requests

		return true;
	}catch(...){
		return false;
	}
}

